/****************************************************************************/
/*! \file
**  \brief  Versions (wfx_Versions) about-box script
**
**  This file implements functions used by the custom about-box.
**
**  \n
**  Copyright  ©  aNDreas Bolotă  aNDreas·Bolota(α)kirnbie·com
**
**  Versions (wfx_Versions) allows you to find out the current version number
**  of any program downloadable online.
**  Versions (wfx_Versions) emulates the behavior of Versions © Fabio Chelly
**  file-system plugin for Total Commander © Christian Ghisler.
**
**  This program is free software: you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation, either version 3 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program.  If not, see <http://www.gnu.org/licenses/>
*/


/****************************************************************************/
/*! \brief  Reference to the Tips & Tricks window
**
**  Used to determine if that window has already been opened
**
**  \see  wfxVersions_aboutOps_showTips()
*/
var  wfxVersions_aboutVars_TipsAndTricks = null;


/****************************************************************************/
/*! \brief  Called when the custom about box is loaded
**
**  Used for setting edit-boxes tooltips from label texts
**
**  \param  --
**
**  \return  --
*/
function  wfxVersions_aboutOps_loadDialog()
{
  /*  load configured colors  */
  wfxVersions_aboutOps_reColorize();

  var  oPrefs = Components.classes[ "@mozilla.org/preferences-service;1" ]
                          .getService(  Components.interfaces.nsIPrefBranch  );
  if(  !oPrefs.getBoolPref( "extensions.wfx_Versions.show.addIcons2Labels" )  )
  {
    window["noicons"] = true;
    wfxVersions_globalOps_removeIcons(  document.documentElement  );
  }

  if(  !( "wfxVersions_dialogOps_copyTip" in opener )  )
  {
    document.getElementById(  "wfxVersions_aboutIDs_hidden"  ).setAttribute(  "hidden",  "true"  );
    document.getElementById(  "wfxVersions_aboutIDs_tips"    ).setAttribute(  "hidden",  "true"  );
  }

  /*  add link to 'license.txt' from the install location  */
  var  dir2file = Components.classes[ "@mozilla.org/extensions/manager;1" ]
                            .getService(  Components.interfaces.nsIExtensionManager  )
                            .getInstallLocation(  "wfx_Versions@firefox.ND"  )
                            .getItemFile(  "wfx_Versions@firefox.ND",  "license.txt"  );
  var  file2url = Components.classes[ "@mozilla.org/network/io-service;1" ]
                            .getService(  Components.interfaces.nsIIOService  )
                            .getProtocolHandler(  "file"  )
                            .QueryInterface(  Components.interfaces.nsIFileProtocolHandler  );
  document.getElementById(  "wfxVersions_aboutIDs_licenseGPL"  ).setAttribute(  "tooltiptext",  file2url.getURLSpecFromFile( dir2file )  );

  /*  get extension localized description and homepageURL  */
  var  azLocalizedPrefs = document.getElementById(  "wfxVersions_aboutIDs_stringBundle"  );
  document.getElementById(  "wfxVersions_aboutIDs_extensionDescription"  ).textContent = azLocalizedPrefs.getString(  "extensions.wfx_Versions@firefox.ND.description"  );
  document.getElementById(  "wfxVersions_aboutIDs_extensionName"         ).value       = azLocalizedPrefs.getString(  "extensions.wfx_Versions@firefox.ND.name"  );
  document.getElementById(  "wfxVersions_aboutIDs_homePageURL"  ).setAttribute(  "tooltiptext",  azLocalizedPrefs.getString( "extensions.wfx_Versions@firefox.ND.homepageURL" )  );

  /*  get extension properties (version+iconURL)  */
  var  i,  aExtensions = Components.classes[ "@mozilla.org/extensions/manager;1" ]
                                   .getService( Components.interfaces.nsIExtensionManager )
                                   .getItemList(  Components.interfaces.nsIUpdateItem.TYPE_EXTENSION,  {}  );
  for(  i = 0;  i != aExtensions.length;  i++  )
  {
    if(  aExtensions[i].name == azLocalizedPrefs.getString( "extensions.wfx_Versions@firefox.ND.name" )  )
    {
      document.getElementById(  "wfxVersions_aboutIDs_versionMain"  ).value = aExtensions[i].version;
      document.getElementById(  "wfxVersions_aboutIDs_iconMain"     ).src   = aExtensions[i].iconURL;
    }
  }

  /*  fix alignment problems if too long version string  */
  var  szFix,  oFix;

  oFix = document.getElementById(  "wfxVersions_aboutIDs_homePageURL"  );
  szFix = oFix.value.replace(  /(\S) (\S)/g,  "$1"+String.fromCharCode(160)+"$2"  );
  oFix.value = szFix;

  oFix = document.getElementById(  "wfxVersions_aboutIDs_giantList"  );
  szFix = oFix.value.replace(  /(\S) (\S)/g,  "$1"+String.fromCharCode(160)+"$2"  );
  oFix.value = szFix;

  wfxVersions_globalOps_setTimeout(  "sizeToContent()",  120  );
}


/****************************************************************************/
/*! \brief  Set custom colors for URLs
**
**  Also called when configuration changes
**
**  \param  --
**
**  \return  --
*/
function  wfxVersions_aboutOps_reColorize()
{
  wfxVersions_globalOps_loadConfigColors();
  wfxVersions_globalOps_styleConfig();
}


/****************************************************************************/
/*! \brief  special handling of keys pressed
**
**  Associate ESC with [Close] button, and ENTER with {onclick} event
**
**  \param[in]  eKey  the onkey event
**
**  \return  event further handling allowed or not
*/
function  wfxVersions_aboutOps_keyPress(  eKey  )
{
  if(  eKey.keyCode == eKey.DOM_VK_ESCAPE  )
  {
    /*  ESC = the [Cancel] button  */
    wfxVersions_globalOps_windowClose(  window  );
    return  false;
  }

  return  true;
}


/****************************************************************************/
/*! \brief  opens the Hidden Options window
**
**  \param  --
**
**  \return  --
*/
function  wfxVersions_aboutOps_hiddenOptions()
{
  /*  dependant from opener window (i.e. the normal Options window)  */
  opener.wfxVersions_globalOps_setTimeout(  "openDialog('chrome://wfx_versions/content/wfx_versions_hidden.xul','','chrome,centerscreen,modal')",  40  );
  wfxVersions_globalOps_setTimeout(  "wfxVersions_globalOps_windowClose(window)",  10  );
}


/****************************************************************************/
/*! \brief  opens the Tips & Tricks window
**
**  \param  --
**
**  \return  --
*/
function  wfxVersions_aboutOps_showTips()
{
  if((  wfxVersions_aboutVars_TipsAndTricks != null  )&&(  wfxVersions_aboutVars_TipsAndTricks.document != null  ))
  {
    wfxVersions_globalOps_windowFocus(  wfxVersions_aboutVars_TipsAndTricks  );
  }
  else
  {
    wfxVersions_aboutVars_TipsAndTricks = openDialog(  "chrome://wfx_versions/content/wfx_versions_tips.xul",
                                                       "",
                                                       "chrome,centerscreen,dependent"  );
  }
}
