/****************************************************************************/
/*! \file
**  \brief  Versions (wfx_Versions) item-edit script
**
**  This file implements functions used by the Editing individual .list items.
**
**  \n
**  Copyright  ©  aNDreas Bolotă  aNDreas·Bolota(α)kirnbie·com
**
**  Versions (wfx_Versions) allows you to find out the current version number
**  of any program downloadable online.
**  Versions (wfx_Versions) emulates the behavior of Versions © Fabio Chelly
**  file-system plugin for Total Commander © Christian Ghisler.
**
**  This program is free software: you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation, either version 3 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program.  If not, see <http://www.gnu.org/licenses/>
*/


/****************************************************************************/
/*! \brief  Called when the edit window loads
**
**  The item parameters are transmitted here via window.arguments[]
**
**  \param  --
**
**  \return  --
*/
function  wfxVersions_editemOps_onLoad()
{
  document.title = window.arguments[1];

  var  oPrefs = Components.classes[ "@mozilla.org/preferences-service;1" ]
                          .getService(  Components.interfaces.nsIPrefBranch  );
  if(  !oPrefs.getBoolPref( "extensions.wfx_Versions.show.addIcons2Labels" )  )
  {
    window["noicons"] = true;
    wfxVersions_globalOps_removeIcons(  document.documentElement  );
  }

  /*  set radiogroup margin equal to textbox margin  */
  var  iCopyTop    = getComputedStyle(  document.getElementById( "wfxVersions_editemIDs_editVersion" ),  null  ).getPropertyValue(  "margin-top"  );
  var  iCopyLeft   = getComputedStyle(  document.getElementById( "wfxVersions_editemIDs_editVersion" ),  null  ).getPropertyValue(  "margin-left"  );
  var  iCopyBottom = getComputedStyle(  document.getElementById( "wfxVersions_editemIDs_editVersion" ),  null  ).getPropertyValue(  "margin-bottom"  );
  var  iCopyRight  = getComputedStyle(  document.getElementById( "wfxVersions_editemIDs_editVersion" ),  null  ).getPropertyValue(  "margin-right"  );
  document.getElementById(  "wfxVersions_editemIDs_editForwards"  ).style.setProperty(  "margin-top",     iCopyTop,     "important"  );
  document.getElementById(  "wfxVersions_editemIDs_editForwards"  ).style.setProperty(  "margin-left",    iCopyLeft,    "important"  );
  document.getElementById(  "wfxVersions_editemIDs_editForwards"  ).style.setProperty(  "margin-bottom",  iCopyBottom,  "important"  );
  document.getElementById(  "wfxVersions_editemIDs_editForwards"  ).style.setProperty(  "margin-right",   iCopyRight,   "important"  );

  document.getElementById(  "wfxVersions_editemIDs_editLine"      ).value         = window.arguments[0].line;
  document.getElementById(  "wfxVersions_editemIDs_editHome"      ).value         = window.arguments[0].home;
  document.getElementById(  "wfxVersions_editemIDs_editDown"      ).value         = window.arguments[0].down;
  document.getElementById(  "wfxVersions_editemIDs_editURL"       ).value         = window.arguments[0].url;
  document.getElementById(  "wfxVersions_editemIDs_editStart"     ).value         = window.arguments[0].start;
  document.getElementById(  "wfxVersions_editemIDs_editPre"       ).value         = window.arguments[0].pre;
  document.getElementById(  "wfxVersions_editemIDs_editPost"      ).value         = window.arguments[0].post;
  document.getElementById(  "wfxVersions_editemIDs_editVersion"   ).value         = window.arguments[0].version;
  document.getElementById(  "wfxVersions_editemIDs_editForwards"  ).selectedIndex = window.arguments[0].dir % 2;

  if(  oPrefs.getBoolPref( "extensions.wfx_Versions.show.alternate.URLs" )  )
  {
    document.getElementById(  "wfxVersions_editemIDs_editHome"  ).parentNode.removeAttribute(  "hidden"  );
    document.getElementById(  "wfxVersions_editemIDs_editDown"  ).parentNode.removeAttribute(  "hidden"  );
  }
  else
  {
    document.getElementById(  "wfxVersions_editemIDs_editHome"  ).parentNode.setAttribute(  "hidden",  "true"  );
    document.getElementById(  "wfxVersions_editemIDs_editDown"  ).parentNode.setAttribute(  "hidden",  "true"  );
  }
}


/****************************************************************************/
/*! \brief  Called when the user clicks the [Ok] button
**
**  Saves the changes into the 
**
**  \param[in]  --
**
**  \return  --
*/
function  wfxVersions_editemOps_onSave()
{
  /*  copy information from textboxes  */
  window.arguments[0].line  = document.getElementById(  "wfxVersions_editemIDs_editLine"      ).value;
  window.arguments[0].home  = document.getElementById(  "wfxVersions_editemIDs_editHome"      ).value;
  window.arguments[0].down  = document.getElementById(  "wfxVersions_editemIDs_editDown"      ).value;
  window.arguments[0].url   = document.getElementById(  "wfxVersions_editemIDs_editURL"       ).value;
  window.arguments[0].start = document.getElementById(  "wfxVersions_editemIDs_editStart"     ).value;
  window.arguments[0].pre   = document.getElementById(  "wfxVersions_editemIDs_editPre"       ).value;
  window.arguments[0].post  = document.getElementById(  "wfxVersions_editemIDs_editPost"      ).value;
  window.arguments[0].dir   = document.getElementById(  "wfxVersions_editemIDs_editForwards"  ).selectedIndex;

  if((  window.arguments[0].home.length != 0  )||(  window.arguments[0].down.length != 0  ))
  {
    window.arguments[0].dir += 2;
  }

  window.arguments[2].write = true;
  wfxVersions_globalOps_windowClose(  window  );
}


/****************************************************************************/
/*! \brief  special handling of keys pressed
**
**  Associate ESC with [Close] button, and ENTER with [Write] button
**
**  \param[in]  eKey  the onkey event
**
**  \return  event further handling allowed or not
*/
function  wfxVersions_editemOps_keyPress(  eKey  )
{
  if(  eKey.keyCode == eKey.DOM_VK_RETURN  )
  {
    /*  ENTER = the [Write] button  */
    wfxVersions_editemOps_onSave();
    return  false;
  }
  else
  if(  eKey.keyCode == eKey.DOM_VK_ESCAPE  )
  {
    /*  ESC = the [Close] button  */
    wfxVersions_globalOps_windowClose(  window  );
    return  false;
  }
  else
  {}

  return  true;
}
