/****************************************************************************/
/*! \file
**  \brief  Versions (wfx_Versions) hidden-options script
**
**  This file implements functions used by the Hidden Options dialog.
**
**  \n
**  Copyright  ©  aNDreas Bolotă  aNDreas·Bolota(α)kirnbie·com
**
**  Versions (wfx_Versions) allows you to find out the current version number
**  of any program downloadable online.
**  Versions (wfx_Versions) emulates the behavior of Versions © Fabio Chelly
**  file-system plugin for Total Commander © Christian Ghisler.
**
**  This program is free software: you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation, either version 3 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program.  If not, see <http://www.gnu.org/licenses/>
*/


/****************************************************************************/
/*! \brief  Called when the Hidden Options dialog is opened
**
**  \param  --
**
**  \return  --
*/
function  wfxVersions_hiddenOps_onLoad()
{
  var  oPrefs = Components.classes[ "@mozilla.org/preferences-service;1" ]
                          .getService(  Components.interfaces.nsIPrefBranch  );

  if(  !oPrefs.getBoolPref( "extensions.wfx_Versions.show.addIcons2Labels" )  )
  {
    window["noicons"] = true;
    wfxVersions_globalOps_removeIcons(  document.documentElement  );
  }

  document.getElementById(  "wfxVersions_hiddenIDs_reScanOnStart"  ).checked = oPrefs.getBoolPref(  "extensions.wfx_Versions.force.Scan.onBrowserStart"  );
  document.getElementById(  "wfxVersions_hiddenIDs_reScanOnDone"   ).value   = oPrefs.getIntPref(   "extensions.wfx_Versions.force.reScan.onScanDone"  );
  document.getElementById(  "wfxVersions_hiddenIDs_newEntry"       ).checked = oPrefs.getBoolPref(  "extensions.wfx_Versions.oldStyle.Edit.newEntry"  );
  document.getElementById(  "wfxVersions_hiddenIDs_logEdit"        ).checked = oPrefs.getBoolPref(  "extensions.wfx_Versions.enable.logging.Edit"  );
  document.getElementById(  "wfxVersions_hiddenIDs_logScan"        ).checked = oPrefs.getBoolPref(  "extensions.wfx_Versions.enable.logging.Scan"  );
  document.getElementById(  "wfxVersions_hiddenIDs_ctrlClick"      ).checked = oPrefs.getBoolPref(  "extensions.wfx_Versions.click.like.ctrlClick"  );
  document.getElementById(  "wfxVersions_hiddenIDs_showComments"   ).checked = oPrefs.getBoolPref(  "extensions.wfx_Versions.comments.display"  );
  document.getElementById(  "wfxVersions_hiddenIDs_expandTools"    ).checked = oPrefs.getBoolPref(  "extensions.wfx_Versions.expanded.toolsmenu"  );
  document.getElementById(  "wfxVersions_hiddenIDs_showIcons"      ).checked = oPrefs.getBoolPref(  "extensions.wfx_Versions.show.addIcons2Labels"  );
  document.getElementById(  "wfxVersions_hiddenIDs_tabArrows"      ).checked = oPrefs.getBoolPref(  "extensions.wfx_Versions.show.tabButton.Arrows"  );
  document.getElementById(  "wfxVersions_hiddenIDs_tabClose"       ).checked = oPrefs.getBoolPref(  "extensions.wfx_Versions.show.tabButton.Close"  );
  document.getElementById(  "wfxVersions_hiddenIDs_singleScan"     ).checked = oPrefs.getBoolPref(  "extensions.wfx_Versions.show.individual.scans"  );
  document.getElementById(  "wfxVersions_hiddenIDs_otherURLs"      ).checked = oPrefs.getBoolPref(  "extensions.wfx_Versions.show.alternate.URLs"  );
  document.getElementById(  "wfxVersions_hiddenIDs_confirmScan"    ).checked = oPrefs.getBoolPref(  "extensions.wfx_Versions.confirm.scan.update"  );
  document.getElementById(  "wfxVersions_hiddenIDs_confirmMove"    ).checked = oPrefs.getBoolPref(  "extensions.wfx_Versions.confirm.edit.move"  );
  document.getElementById(  "wfxVersions_hiddenIDs_confirmCopy"    ).checked = oPrefs.getBoolPref(  "extensions.wfx_Versions.confirm.edit.copy"  );
  document.getElementById(  "wfxVersions_hiddenIDs_confirmDelete"  ).checked = oPrefs.getBoolPref(  "extensions.wfx_Versions.confirm.edit.delete"  );
  document.getElementById(  "wfxVersions_hiddenIDs_confirmAppend"  ).checked = oPrefs.getBoolPref(  "extensions.wfx_Versions.confirm.edit.append"  );
  document.getElementById(  "wfxVersions_hiddenIDs_confirmEdit"    ).checked = oPrefs.getBoolPref(  "extensions.wfx_Versions.confirm.edit.modify"  );
  document.getElementById(  "wfxVersions_hiddenIDs_confirmSort"    ).checked = oPrefs.getBoolPref(  "extensions.wfx_Versions.confirm.edit.sort"  );
  document.getElementById(  "wfxVersions_hiddenIDs_confirmSave"    ).checked = oPrefs.getBoolPref(  "extensions.wfx_Versions.confirm.quickedit.save"  );
  document.getElementById(  "wfxVersions_hiddenIDs_confirmErase"   ).checked = oPrefs.getBoolPref(  "extensions.wfx_Versions.confirm.file.delete"  );
  document.getElementById(  "wfxVersions_hiddenIDs_confirmCreate"  ).checked = oPrefs.getBoolPref(  "extensions.wfx_Versions.confirm.create.default.lists"  );
  document.getElementById(  "wfxVersions_hiddenIDs_confirmKiLoad"  ).checked = oPrefs.getBoolPref(  "extensions.wfx_Versions.confirm.loadProgress.cancel"  );
  document.getElementById(  "wfxVersions_hiddenIDs_skipEdit"       ).checked = oPrefs.getBoolPref(  "extensions.wfx_Versions.selections.skipSingle.Edit"  );
  document.getElementById(  "wfxVersions_hiddenIDs_skipScan"       ).checked = oPrefs.getBoolPref(  "extensions.wfx_Versions.selections.skipSingle.Scan"  );
  document.getElementById(  "wfxVersions_hiddenIDs_expandScan"     ).checked = oPrefs.getBoolPref(  "extensions.wfx_Versions.expanded.context.Scan"  );

  var  iOps = oPrefs.getIntPref(  "extensions.wfx_Versions.resize.shrinkBack"  );
  if(  iOps < 0  )
  {
    iOps = 0;
  }
  if(  iOps > 2  )
  {
    iOps = 2;
  }
  document.getElementById(  "wfxVersions_hiddenIDs_shrinkBack"  ).selectedIndex = iOps;

  wfxVersions_globalOps_styleInt(  document.getElementById( "wfxVersions_hiddenIDs_reScanOnDone" )  );
  wfxVersions_hiddenOps_logCheck(  oPrefs  );
}


/****************************************************************************/
/*! \brief  Enable/disable the logging.format textbox
**
**  Only enabled when either the logging.Scan or logging.Edit are enabled.
**
**  \param[in]  oPrefs  read config at initialization (otherwise null)
**
**  \return  --
*/
function  wfxVersions_hiddenOps_logCheck(  oPrefs  )
{
  var  oFormat = document.getElementById(  "wfxVersions_hiddenIDs_logFormat"  );
  var  oName   = document.getElementById(  "wfxVersions_hiddenIDs_logName"  );

  if(  oPrefs  )
  {
    oFormat.lastChild.value = oPrefs.getComplexValue(  "extensions.wfx_Versions.logging.format",  Components.interfaces.nsISupportsString  )
                                 .data;

    oName.lastChild.value = oPrefs.getComplexValue(  "extensions.wfx_Versions.logging.name",  Components.interfaces.nsISupportsString  )
                               .data;
    wfxVersions_hiddenOps_logTip(  oName.lastChild  );

    var  wFormat = oFormat.firstChild.boxObject.width;
    var  wName   = oName  .firstChild.boxObject.width;
    if(  wFormat > wName  )
    {
      wName = wFormat;
    }
    oFormat.firstChild.style.setProperty(  "width",  wName + "px",  "important"  );
    oName  .firstChild.style.setProperty(  "width",  wName + "px",  "important"  );
  }

  if((  document.getElementById( "wfxVersions_hiddenIDs_logEdit" ).checked  )
   ||(  document.getElementById( "wfxVersions_hiddenIDs_logScan" ).checked  ))
  {
    oFormat.firstChild.removeAttribute(  "disabled"  );
    oFormat. lastChild.removeAttribute(  "disabled"  );

    oName  .firstChild.removeAttribute(  "disabled"  );
    oName  . lastChild.removeAttribute(  "disabled"  );
  }
  else
  {
    oFormat.firstChild.setAttribute(  "disabled",  "true"  );
    oFormat. lastChild.setAttribute(  "disabled",  "true"  );

    oName  .firstChild.setAttribute(  "disabled",  "true"  );
    oName  . lastChild.setAttribute(  "disabled",  "true"  );
  }
}


/****************************************************************************/
/*! \brief  Prepare the tooltiptext of the logging.name textbox
**
**  \param[in]  oBox  the textbox object
**
**  \return  --
*/
function  wfxVersions_hiddenOps_logTip(  oBox  )
{
  var  szValue = oBox.value.replace(  "[Y]",  wfxVersions_globalOps_YMD( new Date() )  )
                           .replace(  "[M]",  opener.document.getElementById( "wfxVersions_dialogIDs_buttonScan" ).getAttribute( "label" )  );

  oBox.setAttribute(  "tooltiptext",  "extensions.wfx_Versions.logging.name = *.list → *" + wfxVersions_globalOps_CLR( szValue )  );
}


/****************************************************************************/
/*! \brief  special handling of keys pressed
**
**  Associate ESC with [Close] button, and ENTER with {onclick} event
**
**  \param[in]  eKey  the onkey event
**
**  \return  event further handling allowed or not
*/
function  wfxVersions_hiddenOps_keyPress(  eKey  )
{
  if(  eKey.keyCode == eKey.DOM_VK_RETURN  )
  {
    /*  ENTER = the {onclick} event  */
    if((  eKey.target  )&&(  eKey.target.nodeName  )&&(  eKey.target.click  ))
    {
      if(  eKey.target.nodeName == "checkbox"  )
      {
        eKey.target.click();
      }
    }
    return  false;
  }
  else
  if(  eKey.keyCode == eKey.DOM_VK_ESCAPE  )
  {
    /*  ESC = the {onclose} event  */
    wfxVersions_globalOps_windowClose(  window  );
    return  false;
  }
  else
  {}

  return  true;
}


/****************************************************************************/
/*! \brief  ensure the focused element scrolled is visible
**
**  \param[in]  oItem  the child node of the scrollbox which has focus
**
**  \return  --
*/
function  wfxVersions_hiddenOps_makeVisible(  oItem  )
{
  var  oScrollBox = document.getElementById(  "wfxVersions_hiddenIDs_scrollBox"  );
  oScrollBox.boxObject.QueryInterface(  Components.interfaces.nsIScrollBoxObject  )
                      .ensureElementIsVisible(  oItem  );
}


/****************************************************************************/
/*! \brief  save preferences on exit
**
**  \param  --
**
**  \return  --
*/
function  wfxVersions_hiddenOps_onSave()
{
  var  oPrefs = Components.classes[ "@mozilla.org/preferences-service;1" ]
                          .getService(  Components.interfaces.nsIPrefBranch  );

  oPrefs.setBoolPref(  "extensions.wfx_Versions.force.Scan.onBrowserStart",     document.getElementById( "wfxVersions_hiddenIDs_reScanOnStart" ).checked  );

  var  iOps = parseInt(  document.getElementById( "wfxVersions_hiddenIDs_reScanOnDone" ).value  );
  oPrefs.setIntPref(  "extensions.wfx_Versions.force.reScan.onScanDone",  iOps  );

  oPrefs.setBoolPref(  "extensions.wfx_Versions.oldStyle.Edit.newEntry",        document.getElementById( "wfxVersions_hiddenIDs_newEntry" ).checked  );
  oPrefs.setBoolPref(  "extensions.wfx_Versions.enable.logging.Edit",           document.getElementById( "wfxVersions_hiddenIDs_logEdit" ).checked  );
  oPrefs.setBoolPref(  "extensions.wfx_Versions.enable.logging.Scan",           document.getElementById( "wfxVersions_hiddenIDs_logScan" ).checked  );
  oPrefs.setBoolPref(  "extensions.wfx_Versions.click.like.ctrlClick",          document.getElementById( "wfxVersions_hiddenIDs_ctrlClick" ).checked  );
  oPrefs.setBoolPref(  "extensions.wfx_Versions.comments.display",              document.getElementById( "wfxVersions_hiddenIDs_showComments" ).checked  );
  oPrefs.setBoolPref(  "extensions.wfx_Versions.expanded.toolsmenu",            document.getElementById( "wfxVersions_hiddenIDs_expandTools" ).checked  );
  oPrefs.setBoolPref(  "extensions.wfx_Versions.show.addIcons2Labels",          document.getElementById( "wfxVersions_hiddenIDs_showIcons" ).checked  );
  oPrefs.setBoolPref(  "extensions.wfx_Versions.show.tabButton.Arrows",         document.getElementById( "wfxVersions_hiddenIDs_tabArrows" ).checked  );
  oPrefs.setBoolPref(  "extensions.wfx_Versions.show.tabButton.Close",          document.getElementById( "wfxVersions_hiddenIDs_tabClose" ).checked  );
  oPrefs.setBoolPref(  "extensions.wfx_Versions.show.individual.scans",         document.getElementById( "wfxVersions_hiddenIDs_singleScan" ).checked  );
  oPrefs.setBoolPref(  "extensions.wfx_Versions.show.alternate.URLs",           document.getElementById( "wfxVersions_hiddenIDs_otherURLs" ).checked  );
  oPrefs.setBoolPref(  "extensions.wfx_Versions.confirm.scan.update",           document.getElementById( "wfxVersions_hiddenIDs_confirmScan" ).checked  );
  oPrefs.setBoolPref(  "extensions.wfx_Versions.confirm.edit.move",             document.getElementById( "wfxVersions_hiddenIDs_confirmMove" ).checked  );
  oPrefs.setBoolPref(  "extensions.wfx_Versions.confirm.edit.copy",             document.getElementById( "wfxVersions_hiddenIDs_confirmCopy" ).checked  );
  oPrefs.setBoolPref(  "extensions.wfx_Versions.confirm.edit.delete",           document.getElementById( "wfxVersions_hiddenIDs_confirmDelete" ).checked  );
  oPrefs.setBoolPref(  "extensions.wfx_Versions.confirm.edit.append",           document.getElementById( "wfxVersions_hiddenIDs_confirmAppend" ).checked  );
  oPrefs.setBoolPref(  "extensions.wfx_Versions.confirm.edit.modify",           document.getElementById( "wfxVersions_hiddenIDs_confirmEdit" ).checked  );
  oPrefs.setBoolPref(  "extensions.wfx_Versions.confirm.edit.sort",             document.getElementById( "wfxVersions_hiddenIDs_confirmSort" ).checked  );
  oPrefs.setBoolPref(  "extensions.wfx_Versions.confirm.quickedit.save",        document.getElementById( "wfxVersions_hiddenIDs_confirmSave" ).checked  );
  oPrefs.setBoolPref(  "extensions.wfx_Versions.confirm.file.delete",           document.getElementById( "wfxVersions_hiddenIDs_confirmErase" ).checked  );
  oPrefs.setBoolPref(  "extensions.wfx_Versions.confirm.create.default.lists",  document.getElementById( "wfxVersions_hiddenIDs_confirmCreate" ).checked  );
  oPrefs.setBoolPref(  "extensions.wfx_Versions.confirm.loadProgress.cancel",   document.getElementById( "wfxVersions_hiddenIDs_confirmKiLoad" ).checked  );
  oPrefs.setBoolPref(  "extensions.wfx_Versions.selections.skipSingle.Edit",    document.getElementById( "wfxVersions_hiddenIDs_skipEdit" ).checked  );
  oPrefs.setBoolPref(  "extensions.wfx_Versions.selections.skipSingle.Scan",    document.getElementById( "wfxVersions_hiddenIDs_skipScan" ).checked  );
  oPrefs.setBoolPref(  "extensions.wfx_Versions.expanded.context.Scan",         document.getElementById( "wfxVersions_hiddenIDs_expandScan" ).checked  );

  iOps = document.getElementById(  "wfxVersions_hiddenIDs_shrinkBack"  ).selectedIndex;
  if(  iOps < 0  )
  {
    iOps = 0;
  }
  if(  iOps > 2  )
  {
    iOps = 2;
  }
  oPrefs.setIntPref(  "extensions.wfx_Versions.resize.shrinkBack",  iOps  );

  var  szNew = Components.classes[ "@mozilla.org/supports-string;1" ]
                         .createInstance(  Components.interfaces.nsISupportsString  );
  szNew.data = document.getElementById(  "wfxVersions_hiddenIDs_logFormat"  ).lastChild.value;
  oPrefs.setComplexValue(  "extensions.wfx_Versions.logging.format",
                           Components.interfaces.nsISupportsString,
                           szNew  );
  szNew.data = document.getElementById(  "wfxVersions_hiddenIDs_logName"  ).lastChild.value;
  oPrefs.setComplexValue(  "extensions.wfx_Versions.logging.name",
                           Components.interfaces.nsISupportsString,
                           szNew  );

  wfxVersions_globalOps_windowClose(  window  );
}
