/****************************************************************************/
/*! \file
**  \brief  Versions (wfx_Versions) overlay script
**
**  This file implements functions used by the status-bar icons and menu,
**  as well as some other common functions (shared with other scripts).
**
**  \n
**  Copyright  ©  aNDreas Bolotă  aNDreas·Bolota(α)kirnbie·com
**
**  Versions (wfx_Versions) allows you to find out the current version number
**  of any program downloadable online.
**  Versions (wfx_Versions) emulates the behavior of Versions © Fabio Chelly
**  file-system plugin for Total Commander © Christian Ghisler.
**
**  This program is free software: you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation, either version 3 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program.  If not, see <http://www.gnu.org/licenses/>
*/


/****************************************************************************/
/*! \mainpage
**
**  Copyright  ©  aNDreas Bolotă  aNDreas·Bolota(α)kirnbie·com
**
**  Versions (wfx_Versions) allows you to find out the current version number
**  of any program downloadable online.
**  Versions (wfx_Versions) emulates the behavior of Versions © Fabio Chelly
**  file-system plugin for Total Commander © Christian Ghisler.
**
**  This program is free software: you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation, either version 3 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program.  If not, see <http://www.gnu.org/licenses/>
**
**  \n
**  Paraphrasing what Fabio Chelly said about his TC plugin:
**  Every day I check on many different websites if there's a new version of
**  my favorite programs available for download.  So I decided to create this
**  extension which groups the pages you want to monitor in Firefox.
**
**  I see no reason why the extension should not work on all OS-s that Firefox
**  supports, but there are no guarantees.  Personally I've only tested on:
**  - Win XP Pro (FireFox 1.5.0.7, 2.0.0.x, 3.x, SeaMonkey 2.x, Flock 2.x)
**  - knoppix DVD (FireFox 1.5.0.3, IceWeasel 2.0.0.1, 2.0.0.12, 3.5.5)
**  Starting with version 0.10 the extension also works on:
**  - Mac OS (thanks to maelcum for testing)
**
**  Original implementation (inspiration):
**  - http://totalcommander.free.fr/fabio/index.htm
**    (the 'Versions' file-system plugin, version 1.51, by Fabio Chelly)
**  - http://www.ghisler.com/
**    (Total Commander, version 7.50, by Christian Ghisler)
**
**  Comparison with TC plugin
**  - advantages:
**    - because of browser integration, also works with https:// and ftp://
**    - because of browser integration, also works on other OS-s
**    - groups applications with equal URL-s to load their web-page only once
**    - works in background (TC plugin locks TC while scanning for versions)
**    - special treatment for the '|' character so that next scan isn't broken
**  - disadvantages:
**    - my extension will not support "Search"es like Fabio Chelly's TC plugin
**
**  The format of the .list files is defined by Fabio's TC plugin, and in fact
**  the extension is meant to work with the same .list files that the TC
**  plugin uses.  However, presence of TC plugin or even TC is not required.
*/


/****************************************************************************/
var  wfxVersions_globalVars_showStatusBar = true;  /*!< \brief  [wfx_Versions] in StatusBar   */
var  wfxVersions_globalVars_showToolsBar  = true;  /*!< \brief  [wfx_Versions] in ToolBar     */
var  wfxVersions_globalVars_showToolsMenu = true;  /*!< \brief  [wfx_Versions] in Tools-Menu  */


/****************************************************************************/
var  wfxVersions_globalVars_bgColorDark    = "#D4D0C8";    /*!< \brief  main background color         */
var  wfxVersions_globalVars_bgColorLight   = "#E4E0D8";    /*!< \brief  alternating background color  */
var  wfxVersions_globalVars_linkColorText  = "blue";       /*!< \brief  link color for scanned item   */
var  wfxVersions_globalVars_editColorText  = "darkblue";   /*!< \brief  link color for edited item    */
var  wfxVersions_globalVars_linkColorHover = "green";      /*!< \brief  hover color for scanned item  */
var  wfxVersions_globalVars_editColorHover = "darkgreen";  /*!< \brief  hover color for edited item   */
var  wfxVersions_globalVars_fgColorError   = "red";        /*!< \brief  error indication color        */
var  wfxVersions_globalVars_fgColorOk      = "black";      /*!< \brief  miscellaneous text color      */


/****************************************************************************/
/*! \brief  Loads the text-link/background colors from configuration
**
**  \param  --
**
**  \return  --
*/
function  wfxVersions_globalOps_loadConfigColors()
{
  var  oPrefs = Components.classes[ "@mozilla.org/preferences-service;1" ]
                          .getService(  Components.interfaces.nsIPrefBranch  );

  wfxVersions_globalVars_bgColorDark    = oPrefs.getCharPref(  "extensions.wfx_Versions.background.main"  );
  wfxVersions_globalVars_bgColorLight   = oPrefs.getCharPref(  "extensions.wfx_Versions.background.alternated"  );
  wfxVersions_globalVars_linkColorText  = oPrefs.getCharPref(  "extensions.wfx_Versions.link.text.main"  );
  wfxVersions_globalVars_editColorText  = oPrefs.getCharPref(  "extensions.wfx_Versions.link.text.edited"  );
  wfxVersions_globalVars_linkColorHover = oPrefs.getCharPref(  "extensions.wfx_Versions.link.hover.main"  );
  wfxVersions_globalVars_editColorHover = oPrefs.getCharPref(  "extensions.wfx_Versions.link.hover.edited"  );
  wfxVersions_globalVars_fgColorOk      = oPrefs.getCharPref(  "extensions.wfx_Versions.scan.text.idle"  );
  wfxVersions_globalVars_fgColorError   = oPrefs.getCharPref(  "extensions.wfx_Versions.scan.text.error"  );
}


/****************************************************************************/
/*! \brief  Dynamically adjust the CSS stiling based on configured colors
**
**  \param  --
**
**  \return  --
*/
function  wfxVersions_globalOps_styleConfig()
{
  var  i,  iCss = document.styleSheets.length;
  for(  i = 0;  i < iCss;  i++  )
  {
    if(  document.styleSheets[i].href == "chrome://wfx_versions/skin/wfx_versions_custom.css"  )
    {
      iCss = i;
    }
  }

  if(  iCss >= document.styleSheets.length  )
  {
    return;
  }

  for(  i = 0;  i < document.styleSheets[iCss].cssRules.length;  i++  )
  {
    var  oRule = document.styleSheets[iCss].cssRules[i];
    if(  oRule.selectorText  )
    {
      if(  oRule.selectorText == ".wfxVersions_skinButton"  )
      {
        oRule.style.setProperty(  "color",  wfxVersions_globalVars_linkColorHover,  "important"  );
      }
      else
      if(  oRule.selectorText == ".wfxVersions_skinButton:hover"  )
      {
        oRule.style.setProperty(  "color",  wfxVersions_globalVars_linkColorText,  "important"  );
      }
      else
      if(  oRule.selectorText == ".wfxVersions_skinLink"  )
      {
        oRule.style.setProperty(  "color",  wfxVersions_globalVars_linkColorText,  "important"  );
      }
      else
      if(  oRule.selectorText == ".wfxVersions_skinLink:hover"  )
      {
        oRule.style.setProperty(  "color",  wfxVersions_globalVars_linkColorHover,  "important"  );
      }
      else
      if(  oRule.selectorText == ".wfxVersions_skinLink[wfxIsError=\"true\"]"  )
      {
        oRule.style.setProperty(  "color",  wfxVersions_globalVars_fgColorError,  "important"  );
      }
      else
      if(  oRule.selectorText == ".wfxVersions_skinLink[wfxIsEdit=\"true\"]"  )
      {
        oRule.style.setProperty(  "color",  wfxVersions_globalVars_editColorText,  "important"  );
      }
      else
      if(  oRule.selectorText == ".wfxVersions_skinLink[wfxIsEdit=\"true\"]:hover"  )
      {
        oRule.style.setProperty(  "color",  wfxVersions_globalVars_editColorHover,  "important"  );
      }
      else
      if(  oRule.selectorText == ".wfxVersions_skinLink:focus"  )
      {
        oRule.style.setProperty(  "border-color",  wfxVersions_globalVars_editColorText,  "important"  );
      }
      else
      if(  oRule.selectorText == "#wfxVersions_dialogIDs_editVersion"  )
      {
        oRule.style.setProperty(  "background",  "none " + wfxVersions_globalVars_bgColorLight,  "important"  );
      }
      else
      if(  oRule.selectorText == "#wfxVersions_dialogIDs_editVersion .textbox-input-box"  )
      {
        oRule.style.setProperty(  "color",  wfxVersions_globalVars_fgColorOk,  "important"  );
      }
      else
      if(  oRule.selectorText == "#wfxVersions_dialogIDs_editVersion[wfxIsError=\"true\"] .textbox-input-box"  )
      {
        oRule.style.setProperty(  "color",  wfxVersions_globalVars_fgColorError,  "important"  );
      }
      else
      if(  oRule.selectorText == "#wfxVersions_dialogIDs_editVersion[wfxIsError=\"false\"] .textbox-input-box"  )
      {
        oRule.style.setProperty(  "color",  wfxVersions_globalVars_linkColorHover,  "important"  );
      }
      else
      if(  oRule.selectorText == ".wfxVersions_skinText"  )
      {
        oRule.style.setProperty(  "color",  wfxVersions_globalVars_fgColorOk,  "important"  );
      }
      else
      if(  oRule.selectorText == ".wfxVersions_skinText[wfxIsError=\"true\"]"  )
      {
        oRule.style.setProperty(  "color",  wfxVersions_globalVars_fgColorError,  "important"  );
      }
      else
      if(  oRule.selectorText == ".wfxVersions_skinText[wfxIsError=\"false\"]"  )
      {
        oRule.style.setProperty(  "color",  wfxVersions_globalVars_linkColorHover,  "important"  );
      }
      else
      if(  oRule.selectorText == ".wfxVersions_skinBGnd"  )
      {
        oRule.style.setProperty(  "background-color",  wfxVersions_globalVars_bgColorDark,  "important"  );
      }
      else
      if(  oRule.selectorText == ".wfxVersions_skinBGnd[wfxIsOdd=\"0\"]"  )
      {
        oRule.style.setProperty(  "background-color",  wfxVersions_globalVars_bgColorLight,  "important"  );
      }
      else
      {}
    }
  }
}


/****************************************************************************/
/*! \brief  recursively removes icons from all buttons/tabs/etc. of a window
**
**  \param[in]  oNode  the parent node from which to recursively search depth
**
**  \return  --
*/
function  wfxVersions_globalOps_removeIcons(  oNode  )
{
  if((  oNode.getAttribute  )&&(  oNode.getAttribute( "label" )  ))
  {
    if(  oNode.getAttribute( "image" )  )
    {
      oNode.removeAttribute(  "image"  );
    }
    if(  oNode.style.getPropertyValue( "list-style-image" )  )
    {
      oNode.style.removeProperty(  "list-style-image"  );
    }
  }

  if((  oNode.childNodes  )&&(  oNode.childNodes.length  ))
  {
    var  i;
    for(  i = 0;  i < oNode.childNodes.length;  i++  )
    {
      wfxVersions_globalOps_removeIcons(  oNode.childNodes[i]  );
    }
  }
}


/****************************************************************************/
/*! \brief  Loads the [wfx_Versions] items visibility
**
**  [wfx_Versions] is displayed in:
**  - statusbar
**  - Tools menu
**  - toolbar (button itself is added manually by user)
**
**  \param[in]  e  the on-load event
**
**  \return  --
*/
function  wfxVersions_globalOps_loadVisibilityPlaces(  e  )
{
  if((  e != null  )&&(  e.type == "load"  ))
  {
    removeEventListener(  "load",  wfxVersions_globalOps_loadVisibilityPlaces,  false  );
  }

  var  oPrefs = Components.classes[ "@mozilla.org/preferences-service;1" ]
                          .getService(  Components.interfaces.nsIPrefBranch  );
  var  bToolsMenu  = oPrefs.getBoolPref(  "extensions.wfx_Versions.show.toolsmenuitem"  );
  var  bExpandMenu = oPrefs.getBoolPref(  "extensions.wfx_Versions.expanded.toolsmenu"  );

  var  oChrome = wfxVersions_overlayOps_findChrome();
  if(  oChrome  )
  {
    oChrome.document.getElementById(  "wfxVersions_dialogIDs_startToolsMenu"  ).parentNode.style.setProperty(  "visibility",  bExpandMenu?"hidden":"visible",  "important"  );
  }

  var  oEnum = Components.classes[ "@mozilla.org/appshell/window-mediator;1" ]
                         .getService(  Components.interfaces.nsIWindowMediator  )
                         .getEnumerator(  "navigator:browser"  );
  while(  oEnum.hasMoreElements()  )
  {
    var  oBrowser = oEnum.getNext().document;

    var  oItem = oBrowser.getElementById(  "wfxVersions_overlayIDs_toolsMenu"  );
    if(  oItem  )
    {
      if((  bToolsMenu  )&&((  !bExpandMenu  )||(  oChrome != null  )))
      {
        oItem.removeAttribute(  "hidden"  );
      }
      else
      {
        oItem.setAttribute(  "hidden",  "true"  );
      }
    }

    oItem = oBrowser.getElementById(  "wfxVersions_overlayIDs_toolsMenuExtended"  );
    if(  oItem  )
    {
      if((  !bToolsMenu  )||(  !bExpandMenu  )||(  oChrome != null  ))
      {
        oItem.setAttribute(  "hidden",  "true"  );
      }
      else
      {
        oItem.removeAttribute(  "hidden"  );
      }
    }

    oItem = oBrowser.getElementById(  "wfxVersions_overlayIDs_toolsBar"  );
    if(  oItem  )
    {
      if(  oPrefs.getBoolPref( "extensions.wfx_Versions.show.toolbar" )  )
      {
        oItem.removeAttribute(  "hidden"  );
      }
      else
      {
        oItem.setAttribute(  "hidden",  "true"  );
      }
    }

    oItem = oBrowser.getElementById(  "wfxVersions_overlayIDs_statusBar"  );
    if(  oItem  )
    {
      if(  oPrefs.getBoolPref( "extensions.wfx_Versions.show.statusbar" )  )
      {
        oItem.removeAttribute(  "hidden"  );
      }
      else
      {
        oItem.setAttribute(  "hidden",  "true"  );
      }
    }
  }

  if((  e != null  )&&(  e.type == "load"  ))
  {
    wfxVersions_globalOps_redoVisibilityPlaces();
  }
}


/****************************************************************************/
/*! \brief  memorize changes visibility of [wfx_Versions] buttons
**
**  \param  --
**
**  \return  --
*/
function  wfxVersions_globalOps_redoVisibilityPlaces()
{
  var  oPrefs = Components.classes[ "@mozilla.org/preferences-service;1" ]
                          .getService( Components.interfaces.nsIPrefBranch );

  wfxVersions_globalVars_showStatusBar = oPrefs.getBoolPref(  "extensions.wfx_Versions.show.statusbar"  );
  wfxVersions_globalVars_showToolsBar  = oPrefs.getBoolPref(  "extensions.wfx_Versions.show.toolbar"  );
  wfxVersions_globalVars_showToolsMenu = oPrefs.getBoolPref(  "extensions.wfx_Versions.show.toolsmenuitem"  );
}


/****************************************************************************/
/*! \brief  memorize initial visibility of [wfx_Versions] buttons
**
**  Can also be called by overlay functions (non-chrome)
**
**  \param[in]  oChromeDoc  the chrome window`s content document
**
**  \return  --
*/
function  wfxVersions_globalOps_memorizeVisibilityPlaces(  oChromeDoc  )
{
  if(  oChromeDoc != null  )
  {
    var  oPrefs = Components.classes[ "@mozilla.org/preferences-service;1" ]
                            .getService(  Components.interfaces.nsIPrefBranch  );

    var  oItem = oChromeDoc.getElementById(  "wfxVersions_dialogIDs_showStatusBar"  );
    oPrefs.setBoolPref(  "extensions.wfx_Versions.show.statusbar",  oItem.checked  );
    wfxVersions_globalOps_recursiveDisable(  oItem.nextSibling,  !oItem.checked  );

    var  oItem = oChromeDoc.getElementById(  "wfxVersions_dialogIDs_showToolsBar"  );
    oPrefs.setBoolPref(  "extensions.wfx_Versions.show.toolbar",  oItem.checked  );
    if(  oItem.getAttribute( "disabled" ) == "true"  )
    {
      wfxVersions_globalOps_recursiveDisable(  oItem.nextSibling,  true  );
    }
    else
    {
      wfxVersions_globalOps_recursiveDisable(  oItem.nextSibling,  !oItem.checked  );
    }

    var  oItem = oChromeDoc.getElementById(  "wfxVersions_dialogIDs_showToolsMenu"  );
    oPrefs.setBoolPref(  "extensions.wfx_Versions.show.toolsmenuitem",  oItem.checked  );
    wfxVersions_globalOps_recursiveDisable(  oItem.nextSibling,  !oItem.checked  );
  }
  wfxVersions_globalOps_loadVisibilityPlaces(  null  );
}


/****************************************************************************/
/*! \brief  restore memorized visibility of [wfx_Versions] buttons
**
**  \param  --
**
**  \return  --
*/
function  wfxVersions_globalOps_undoVisibilityPlaces()
{
  var  oPrefs = Components.classes[ "@mozilla.org/preferences-service;1" ]
                          .getService(  Components.interfaces.nsIPrefBranch  );

  oPrefs.setBoolPref(  "extensions.wfx_Versions.show.statusbar",
                       wfxVersions_globalVars_showStatusBar  );
  oPrefs.setBoolPref(  "extensions.wfx_Versions.show.toolbar",
                       wfxVersions_globalVars_showToolsBar  );
  oPrefs.setBoolPref(  "extensions.wfx_Versions.show.toolsmenuitem",
                       wfxVersions_globalVars_showToolsMenu  );

  wfxVersions_globalOps_loadVisibilityPlaces(  null  );
}


/****************************************************************************/
/*! \brief  Starts/Stops the throbber animation
**
**  An own throbber is used instead of the FF global one
**
**  \param[in]  bOnOff  enable throbber / disable throbber
**
**  \return  --
*/
function  wfxVersions_overlayOps_changeThrobber(  bOnOff  )
{
  var  oEnum = Components.classes[ "@mozilla.org/appshell/window-mediator;1" ]
                         .getService( Components.interfaces.nsIWindowMediator )
                         .getEnumerator(  "navigator:browser"  );
  while(  oEnum.hasMoreElements()  )
  {
    var  oBrowser = oEnum.getNext();

    var  oItem = oBrowser.document.getElementById(  "wfxVersions_overlayIDs_obsThrobber"  );
    if(  oItem  )
    {
      oItem.setAttribute(  "wfxIsThrobbing",  bOnOff  );
    }
  }
}


/****************************************************************************/
/*! \brief  Shows a popup panel, different function starting with Firefox 3
**
**  \param[in]  oPop  the popup object
**  \param[in]  posX  screenX position for the popup
**  \param[in]  posY  screenY position for the popup
**
**  \return  --
*/
function  wfxVersions_globalOps_showPopup(  oPop,  posX,  posY  )
{
  try
  {
    /*  code for toolkit 1.9.x  */
    oPop.openPopupAtScreen(  posX,  posY,  false  );
  }
  catch(  e  )
  {
    /*  code for toolkit 1.8.x  */
    try
    {
      oPop.showPopup(  null,  posX,  posY,  "popup",  "none",  "none"  );
    }
    catch(  e  )
    {
      var  szID = oPop.getAttribute(  "id"  );
      oPop.removeAttribute(  "id"  );

      var  oNew = document.createElement(  "popupset"  );
      document.documentElement.appendChild(  oNew  );

      oNew = document.createElement(  "popup"  );
      oNew.setAttribute(  "id",  szID  );
      document.documentElement.lastChild.appendChild(  oNew  );

      oNew = document.importNode(  oPop,  true  );
      oNew.setAttribute(  "orient",  "vertical"  );
      document.documentElement.lastChild.lastChild.appendChild(  oNew  );

      oPop.setAttribute(  "hidden",  "true"  );
      oNew.parentNode.showPopup(  null,  posX,  posY,  "popup",  "none",  "none"  );
    }
  }
}


/****************************************************************************/
/*! \brief  Hides a popup panel, different function starting with Firefox 3
**
**  \param[in]  oPop  the popup object
**
**  \return  --
*/
function  wfxVersions_globalOps_hidePopup(  oPop  )
{
  try
  {
    oPop.hidePopup();
  }
  catch(  e  )
  {
    /*  popup not yet initialized  */
  }
}


/****************************************************************************/
/*! \brief  Style a numeric textbox to indicate ¨0 = disabled¨
**
**  \param[in]  the textbox object
**
**  \return  --
*/
function  wfxVersions_globalOps_styleInt(  oBox  )
{
  var  oTxt = oBox.previousSibling;
  if(  oTxt.firstChild  )
  {
    oTxt = oTxt.firstChild;
    if(  oTxt.nextSibling  )
    {
      oTxt = oTxt.nextSibling;
    }
  }

  var  iTime = parseInt(  oBox.value  );
  if(  iTime > 0  )
  {
    oBox.style.removeProperty(  "opacity"  );
    oTxt.style.removeProperty(  "opacity"  );
  }
  else
  {
    oBox.style.setProperty(  "opacity",  "0.8",  "important"  );
    oTxt.style.setProperty(  "opacity",  "0.6",  "important"  );
  }
}


/****************************************************************************/
/*! \brief  focus() causes exception for windows covred by modal windows
**
**  This function searches recursively which windows are covering the current
**  window, via the window.opener attribute
**
**  \param[in]  oWindow  the window which we want to focus()
**
**  \return  boolean (true = success / false = fail)
*/
function  wfxVersions_globalOps_windowFocus(  oWindow  )
{
  try
  {
    oWindow.focus();
    if(  "hasFocus" in oWindow.document  )
    {
      /*  new in Firefox 3  */
      if(  oWindow.document.hasFocus()  )
      {
        return  true;
      }
    }
    else
    {
      /*  Firefox 2 or older  */
      return  true;
    }
  }
  catch(  e  )
  {}

  var  oEnum = Components.classes[ "@mozilla.org/appshell/window-mediator;1" ]
                         .getService( Components.interfaces.nsIWindowMediator )
                         .getEnumerator(  null  );
  while(  oEnum.hasMoreElements()  )
  {
    var  oModal = oEnum.getNext();
    if(  oModal.opener == oWindow  )
    {
      /*  multiple sub-windows, maybe not all are modal  */
      if(  wfxVersions_globalOps_windowFocus( oModal )  )
      {
        return  true;
      }
    }
  }

  return  false;
}


/****************************************************************************/
/*! \brief  Recursively disable/enable a node and all it`s children
**
**  Used for e.g. disabling while still showing an entire tab panel
**
**  \param[in]  oParent   the topmost node to be modified
**  \param[in]  bDisable  disable (true) or enable (false)
**
**  \return  --
*/
function  wfxVersions_globalOps_recursiveDisable(  oParent,  bDisable  )
{
  if(  oParent.setAttribute  )
  {
    if(  bDisable  )
    {
      oParent.setAttribute(  "disabled",  "true"  );
    }
    else
    {
      oParent.removeAttribute(  "disabled"  );
    }
  }

  if(  oParent.childNodes  )
  {
    var  i;
    for(  i = 0;  i < oParent.childNodes.length;  i++  )
    {
      wfxVersions_globalOps_recursiveDisable(  oParent.childNodes[i],  bDisable  );
    }
  }
}


/****************************************************************************/
/*! \brief  Attempts a "clean" removeChild procedure
**
**  First removes the grandchildred, and only afterwards the children
**
**  \param[in]  oParent  the parent node for which we're removeChild-ing
**
**  \return  --
*/
function  wfxVersions_globalOps_nodeCleanUp(  oParent  )
{
  if(  oParent.childNodes  )
  {
    while(  oParent.childNodes.length > 0  )
    {
      var  oldNode = oParent.childNodes[0];
      wfxVersions_globalOps_nodeCleanUp(  oldNode  );
      oParent.removeChild(  oldNode  );
    }
  }
}


/****************************************************************************/
/*! \brief  close() the sub-windows first, then close() this window
**
**  This function searches recursively which windows are covering the current
**  window, via the window.opener attribute
**
**  \param[in]  oWindow  the window which we want to close()
**
**  \return  false, to disable event processing afterwards
*/
function  wfxVersions_globalOps_windowClose(  oWindow  )
{
  if((  "wfxVersions_dialogVars_httpRequest" in oWindow  )&&(  oWindow.wfxVersions_dialogVars_httpRequest != null  ))
  {
    oWindow.wfxVersions_httpOps_ghost();
  }

  var  oEnum = Components.classes[ "@mozilla.org/appshell/window-mediator;1" ]
                         .getService( Components.interfaces.nsIWindowMediator )
                         .getEnumerator(  null  );
  while(  oEnum.hasMoreElements()  )
  {
    var  oSubWin = oEnum.getNext();
    if(  oSubWin.opener == oWindow  )
    {
      /*  multiple sub-windows  */
      wfxVersions_globalOps_windowClose(  oSubWin  );
    }
  }

  if(  "wfxVersions_aTimers" in oWindow  )
  {
    while(  oWindow["wfxVersions_aTimers"].length  )
    {
      var  xTimer = oWindow["wfxVersions_aTimers"].shift();
      xTimer.wfxObj.cancel();
    }
  }

  oWindow.close();

  /*  workaround for window:
  **  close() not working when alert()/confirm() are currently opened from it
  */
  if(  !oWindow.closed  )
  {
    /*  use the classic setTimeout(), since we killed all custom handlers  */
    oWindow.setTimeout(  "close()",  10  );
  }
}


/****************************************************************************/
/*! \brief  Determine if this chrome window is already opened
**
**  Used to avoid multiple opening of same window
**
**  \param  --
**
**  \return  the window-object if already open, or null otherwise
*/
function  wfxVersions_overlayOps_findChrome()
{
  var  oEnum = Components.classes[ "@mozilla.org/appshell/window-mediator;1" ]
                         .getService( Components.interfaces.nsIWindowMediator )
                         .getEnumerator(  null  );
  while(  oEnum.hasMoreElements()  )
  {
    var  oWindow = oEnum.getNext();
    if(  "wfxVersions_dialogVars_progress" in oWindow  )
    {
      /*  the [wfx_Versions] window is already open!  */
      if(  oWindow != window  )
      {
        return  oWindow;
      }
    }
  }

  return  null;
}


/****************************************************************************/
/*! \brief  Choose between simple/extended Scan menu-item
**
**  \param[in]  ePop  the popup event
**
**  \return  --
*/
function  wfxVersions_overlayOps_extendContext(  ePop  )
{
  if((  ePop.target  )&&(  ePop.target.childNodes  )&&(  ePop.target.childNodes.length == 4  ))
  {
    var  bExtendedMenu = Components.classes[ "@mozilla.org/preferences-service;1" ]
                                   .getService(  Components.interfaces.nsIPrefBranch  )
                                   .getBoolPref(  "extensions.wfx_Versions.expanded.context.Scan"  );
    if(  bExtendedMenu  )
    {
      /*  extend menu  */
      ePop.target.childNodes[1].removeAttribute(  "hidden"  );
      ePop.target.childNodes[2].setAttribute(  "hidden",  "true"  );
    }
    else
    {
      /*  only menulist  */
      ePop.target.childNodes[1].setAttribute(  "hidden",  "true"  );
      ePop.target.childNodes[2].removeAttribute(  "hidden"  );
    }
  }
}


/****************************************************************************/
/*! \brief  Enable/disable the statusbar/toolbar popup menu
**
**  If [wfx_Versions] is already running, set focus to it (no popup)
**
**  \param  --
**
**  \return  event handling continued or not (boolean)
*/
function  wfxVersions_overlayOps_showContext()
{
  /*  close the menu(s) when showing the popup  */
  var  oItem = document.getElementById(  "menu_ToolsPopup"  );
  if(  oItem  )
  {
    while((  oItem.parentNode  )&&(  oItem.parentNode.parentNode  )&&(  oItem.parentNode.parentNode.nodeName == "menupopup"  ))
    {
      oItem = oItem.parentNode.parentNode;
    }
    oItem.hidePopup();
    if((  oItem.parentNode  )&&(  oItem.parentNode.nodeName == "menu"  ))
    {
      oItem.parentNode.click();
    }
  }

  /*  disable popup if chrome window already open  */
  var  oldWindow = wfxVersions_overlayOps_findChrome();
  if(  oldWindow != null  )
  {
    oldWindow.getAttention();
    wfxVersions_globalOps_windowFocus(  oldWindow  );
    return  false;
  }

  return  true;
}


/****************************************************************************/
/*! \brief  Initializes the window object and loads wfx_Versions if new
**
**  This is a "click" event notification.
**  Called when user clicks on status-bar button [wfx_Versions]
**
**  \param[in]  szOps  source of the launch (toolbar / menus / statusbar)
**
**  \return  --
*/
function  wfxVersions_overlayOps_launchChrome(  szOps  )
{
  /*  avoid multiple copies of this window  */
  var  oldWindow = wfxVersions_overlayOps_findChrome();
  if(  oldWindow != null  )
  {
    oldWindow.getAttention();
    wfxVersions_globalOps_windowFocus(  oldWindow  );
    return;
  }

  /*  if window does not exist, create it...  */
  openDialog(  "chrome://wfx_versions/content/wfx_versions_window.xul",
               "",
               "chrome,all,dialog=no"  )["overlay"] = szOps;
}


/****************************************************************************/
/*! \brief  Force scanning when the browser is started
**
**  \param[in]  e  the on-load event
**
**  \return  --
*/
function  wfxVersions_globalOps_ScanOnBrowserStart(  e  )
{
  removeEventListener(  "load",  wfxVersions_globalOps_ScanOnBrowserStart,  false  );

  var  oEnum = Components.classes[ "@mozilla.org/appshell/window-mediator;1" ]
                         .getService(  Components.interfaces.nsIWindowMediator  )
                         .getEnumerator(  "navigator:browser"  );
  while(  oEnum.hasMoreElements()  )
  {
    var  oWindow = oEnum.getNext();
    if((  "wfxVersions_wasLoaded" in oWindow  )&&(  oWindow["wfxVersions_wasLoaded"]  ))
    {
      return;
    }
  }
  window[ "wfxVersions_wasLoaded" ] = true;

  var  bForce = Components.classes[ "@mozilla.org/preferences-service;1" ]
                          .getService(  Components.interfaces.nsIPrefBranch  )
                          .getBoolPref(  "extensions.wfx_Versions.force.Scan.onBrowserStart"  );
  if(  bForce  )
  {
    openDialog(  "chrome://wfx_versions/content/wfx_versions_window.xul",
                 "",
                 "chrome,all,dialog=no"  )["overlay"] = "scan-all";
  }
}


/****************************************************************************/
/*! \brief  At window on-close also close this chrome window
*/
{
  addEventListener(  "unload",  wfxVersions_globalOps_onBrowserClosed,       false  );
  addEventListener(  "load",    wfxVersions_globalOps_ScanOnBrowserStart,    false  );
  addEventListener(  "load",    wfxVersions_globalOps_loadVisibilityPlaces,  false  );
  addEventListener(  "focus",   wfxVersions_globalOps_loadVisibilityPlaces,  false  );
  addEventListener(  "blur",    wfxVersions_globalOps_loadVisibilityPlaces,  false  );
}


/****************************************************************************/
/*! \brief  Called when any window is closed
**
**  Close this window if all browser windows are closed
**
**  \param[in]  e  the on-close event
**
**  \return  --
*/
function  wfxVersions_globalOps_onBrowserClosed(  e  )
{
  removeEventListener(  "unload",  wfxVersions_globalOps_onBrowserClosed,  false  );
  if(  "wfxVersions_dialogVars_progress" in window  )
  {
    if(  wfxVersions_dialogVars_httpRequest != null  )
    {
      wfxVersions_httpOps_ghost();
    }
    if(  wfxVersions_dialogVars_progress >= 0  )
    {
      wfxVersions_overlayOps_changeThrobber(  false  );
    }
  }
  if(  !( "getBrowser" in window )  )
  {
    /*  it`s not a browser window that closes  */
    wfxVersions_globalOps_undoVisibilityPlaces();
    return;
  }

  var  oEnum = Components.classes[ "@mozilla.org/appshell/window-mediator;1" ]
                         .getService( Components.interfaces.nsIWindowMediator )
                         .getEnumerator(  "navigator:browser"  );
  if(  oEnum.hasMoreElements()  )
  {
    /*  this is not the last window that closes  */
    return;
  }

  /*  kill the old chrome window  */
  var  oldWindow = wfxVersions_overlayOps_findChrome();
  if(  oldWindow != null  )
  {
    wfxVersions_globalOps_windowClose(  oldWindow  );
  }
}


/****************************************************************************/
/*! \brief  Opens a browser page with the given URL address
**
**  The click behavior is according to browser configuration, e.g.
**  - normal click = open in same tab
**  - Ctrl + click = open in new window/tab
**
**  \param[in]  eMouse  the mouse-click event
**
**  \return  --
*/
function  wfxVersions_globalOps_clickURL(  eMouse  )
{
  /*  sub-function which looks for root browser (gBrowser in Firefox)  */
  function  wfxVersions_globalOps_gBrowser(  oWindow  )
  {
    var  oRoot = null;
    if(  oWindow.opener  )
    {
      /*  go to root-most window, which should know how to open links  */
      oRoot = wfxVersions_globalOps_gBrowser(  oWindow.opener  );
    }
    if((  !oRoot  )&&(  "openUILink" in oWindow  ))
    {
      /*  if current window knows how to open links, look no further  */
      oRoot = oWindow;
    }
    return(  oRoot  );
  }

  /*  get the URL address from the event target's tooltiptext  */
  var  szURL = eMouse.target.getAttribute(  "tooltiptext"  );
  if(  eMouse.target.hasAttribute( "wfxURL" )  )
  {
    szURL = eMouse.target.getAttribute(  "wfxURL"  );
  }

  /*  Method 1:  using openUILink()  */
  var  oBrowser = wfxVersions_globalOps_gBrowser(  window  );
  if(  oBrowser  )
  {
    /*  convert click into ctrl+click  */
    if(  eMouse.button == 0  )
    {
      var  newMouse =
      {
        button:    0,
        clientX:   eMouse.clientX,
        clientY:   eMouse.clientY,
        ctrlKey:   eMouse.ctrlKey,
        shiftKey:  eMouse.shiftKey,
        altKey:    eMouse.altKey,
        metaKey:   eMouse.metaKey
      };
      var  oPrefs = Components.classes[ "@mozilla.org/preferences-service;1" ]
                              .getService(  Components.interfaces.nsIPrefBranch  );
      if(  oPrefs.getBoolPref( "extensions.wfx_Versions.click.like.ctrlClick" )  )
      {
        newMouse.ctrlKey = true;
      }

      oBrowser.openUILink(  szURL,  newMouse,  false,  true  );
    }
    else
    {
      oBrowser.openUILink(  szURL,  eMouse,  false,  true  );
    }

    /*  the browser is activated, so reset focus to the preferences window  */
    wfxVersions_globalOps_setTimeout(  "wfxVersions_globalOps_windowFocus(window)",  90  );

    return;
  }

  /*  Method 2:  using nsIExternalProtocolService  */
  var  xURI = Components.classes[ "@mozilla.org/network/simple-uri;1" ]
                        .getService(  Components.interfaces.nsIURI  );
  xURI.spec = szURL;

  Components.classes[ "@mozilla.org/uriloader/external-protocol-service;1" ]
            .getService(  Components.interfaces.nsIExternalProtocolService  )
            .loadUrl(  xURI  );
}


/****************************************************************************/
/*! \brief  Convert a date to a 'yyyymmdd' format
**
**  \param[in]  xDate  the input date object
**
**  \return  the formatted string
*/
function  wfxVersions_globalOps_YMD(  xDate  )
{
  var  szYMD = "" + xDate.getDate();
  if(  szYMD.length < 2  )
  {
    szYMD = "0" + szYMD;
  }

  szYMD = xDate.getMonth() + szYMD;
  if(  szYMD.length < 4  )
  {
    szYMD = "0" + szYMD;
  }

  szYMD = xDate.getFullYear() + szYMD;

  return  szYMD;
}


/****************************************************************************/
/*! \brief  Clean-up the operation-mode (Edit/Scan) for the log-file name
**
**  Replace by underscores:
**  - spaces
**  - less-than / greater-than chars
**  - reserved chars
**
**  \param[in]  szOps  input operation name
**
**  \return  the formatted string
*/
function  wfxVersions_globalOps_CLR(  szOps  )
{
  return  szOps.replace(  /[\s‹›<>\\\/\*\?\:\|'"]+/g,  '_'  )
               .replace(  /__/g,  '_'  )
               .replace(  /_\./g,  '.'  );
}


/****************************************************************************/
/*! \brief  Convert HTML entities to characters (like \&nbsp; into Space)
**
**  The trick with .innerHTML only works on actual web pages (but not in XUL)
**
**  \param[in]  txt  the string (with HTML entities) to be converted
**
**  \return  the converted string (with actual characters)
*/
function  wfxVersions_globalOps_convertHTML(  txt  )
{
  return  txt.replace(  /&#((\d{1,7})|(x[\da-f]{1,6}));?/gi,  function(  match,  p1,  p2,  p3,  offset,  s  )
                                                              {
                                                                /**  \&#nnn;  \&#xnnn;  */
                                                                return  String.fromCharCode(  p2 || ( "0" + p3 )  );
                                                              }
                     )
             .replace(  /&(\w)(grave|acute|circ|tilde|uml|ring|cedil|slash|caron);/g,  "$1"  )  /**  latin letters with diacritical marks  */
             .replace(  /&(nb|en|em|thin)sp;/g,  " "  )  /**  non-break|en|em|thin spaces  */
             .replace(  /&(n|m)dash;/g,          "-"  )  /**  en|em dash  */
             .replace(  /&quot;/g,          "\u0022"  )  /**  quotation mark  */
             .replace(  /&amp;/g,           "\u0026"  )  /**  ampersand  */
             .replace(  /&copy;/g,          "\u00A9"  )  /**  copyright sign  */
             .replace(  /&reg;/g,           "\u00AE"  )  /**  registered trade mark sign  */
             .replace(  /&micro;/g,         "\u00B5"  )  /**  micro sign  */
             .replace(  /&middot;/g,        "\u00B7"  )  /**  middle dot = Georgian comma  */
             .replace(  /&szlig;/g,         "\u00DF"  )  /**  latin small sharp s  */
             .replace(  /&AElig;/g,         "\u00C6"  )  /**  latin  big  ligature ae  */
             .replace(  /&aelig;/g,         "\u00E6"  )  /**  latin small ligature ae  */
             .replace(  /&ETH;/g,           "\u00D0"  )  /**  latin  big  eth  */
             .replace(  /&eth;/g,           "\u00F0"  )  /**  latin small eth  */
             .replace(  /&THORN;/g,         "\u00DE"  )  /**  latin  big  thorn  */
             .replace(  /&thorn;/g,         "\u00FE"  )  /**  latin small thorn  */
             .replace(  /&OElig;/g,         "\u0152"  )  /**  latin  big  ligature oe  */
             .replace(  /&oelig;/g,         "\u0153"  )  /**  latin small ligature oe  */
             .replace(  /&Dagger;/g,        "\u2021"  )  /**  double dagger sign  */
             .replace(  /&dagger;/g,        "\u2020"  )  /**         dagger sign  */
             .replace(  /&trade;/g,         "\u2122"  )  /**  trade mark sign  */
             .replace(  /&Alpha;/g,         "\u0391"  )  /**  greek  big  letter alpha  */
             .replace(  /&Beta;/g,          "\u0392"  )  /**  greek  big  letter beta  */
             .replace(  /&Gamma;/g,         "\u0393"  )  /**  greek  big  letter gamma  */
             .replace(  /&Delta;/g,         "\u0394"  )  /**  greek  big  letter delta  */
             .replace(  /&Epsilon;/g,       "\u0395"  )  /**  greek  big  letter epsilon  */
             .replace(  /&Zeta;/g,          "\u0396"  )  /**  greek  big  letter zeta  */
             .replace(  /&Eta;/g,           "\u0397"  )  /**  greek  big  letter eta  */
             .replace(  /&Theta;/g,         "\u0398"  )  /**  greek  big  letter theta  */
             .replace(  /&Iota;/g,          "\u0399"  )  /**  greek  big  letter iota  */
             .replace(  /&Kappa;/g,         "\u039A"  )  /**  greek  big  letter kappa  */
             .replace(  /&Lambda;/g,        "\u039B"  )  /**  greek  big  letter lambda  */
             .replace(  /&Mu;/g,            "\u039C"  )  /**  greek  big  letter mu  */
             .replace(  /&Nu;/g,            "\u039D"  )  /**  greek  big  letter nu  */
             .replace(  /&Xi;/g,            "\u039E"  )  /**  greek  big  letter xi  */
             .replace(  /&Omicron;/g,       "\u039F"  )  /**  greek  big  letter omicron  */
             .replace(  /&Pi;/g,            "\u03A0"  )  /**  greek  big  letter pi  */
             .replace(  /&Rho;/g,           "\u03A1"  )  /**  greek  big  letter rho  */
             .replace(  /&Sigma;/g,         "\u03A3"  )  /**  greek  big  letter sigma  */
             .replace(  /&Tau;/g,           "\u03A4"  )  /**  greek  big  letter tau  */
             .replace(  /&Upsilon;/g,       "\u03A5"  )  /**  greek  big  letter upsilon  */
             .replace(  /&Phi;/g,           "\u03A6"  )  /**  greek  big  letter phi  */
             .replace(  /&Chi;/g,           "\u03A7"  )  /**  greek  big  letter chi  */
             .replace(  /&Psi;/g,           "\u03A8"  )  /**  greek  big  letter psi  */
             .replace(  /&Omega;/g,         "\u03A9"  )  /**  greek  big  letter omega  */
             .replace(  /&alpha;/g,         "\u03B1"  )  /**  greek small letter alpha  */
             .replace(  /&beta;/g,          "\u03B2"  )  /**  greek small letter beta  */
             .replace(  /&gamma;/g,         "\u03B3"  )  /**  greek small letter gamma  */
             .replace(  /&delta;/g,         "\u03B4"  )  /**  greek small letter delta  */
             .replace(  /&epsilon;/g,       "\u03B5"  )  /**  greek small letter epsilon  */
             .replace(  /&zeta;/g,          "\u03B6"  )  /**  greek small letter zeta  */
             .replace(  /&eta;/g,           "\u03B7"  )  /**  greek small letter eta  */
             .replace(  /&theta(sym)?;/g,   "\u03B8"  )  /**  greek small letter theta  */
             .replace(  /&iota;/g,          "\u03B9"  )  /**  greek small letter iota  */
             .replace(  /&kappa;/g,         "\u03BA"  )  /**  greek small letter kappa  */
             .replace(  /&lambda;/g,        "\u03BB"  )  /**  greek small letter lambda  */
             .replace(  /&mu;/g,            "\u03BC"  )  /**  greek small letter mu  */
             .replace(  /&nu;/g,            "\u03BD"  )  /**  greek small letter nu  */
             .replace(  /&xi;/g,            "\u03BE"  )  /**  greek small letter xi  */
             .replace(  /&omicron;/g,       "\u03BF"  )  /**  greek small letter omicron  */
             .replace(  /&pi;/g,            "\u03C0"  )  /**  greek small letter pi  */
             .replace(  /&rho;/g,           "\u03C1"  )  /**  greek small letter rho  */
             .replace(  /&sigmaf?;/g,       "\u03C3"  )  /**  greek small letter sigma  */
             .replace(  /&tau;/g,           "\u03C4"  )  /**  greek small letter tau  */
             .replace(  /&upsilon;/g,       "\u03C5"  )  /**  greek small letter upsilon  */
             .replace(  /&phi;/g,           "\u03C6"  )  /**  greek small letter phi  */
             .replace(  /&chi;/g,           "\u03C7"  )  /**  greek small letter chi  */
             .replace(  /&psi;/g,           "\u03C8"  )  /**  greek small letter psi  */
             .replace(  /&omega;/g,         "\u03C9"  )  /**  greek small letter omega  */
             /*  special treatment for display  */
             .replace(  /^\s+/,             ""        )  /**  trim all spaces from left  */
             .replace(  /\s+$/,             ""        )  /**  trim all spaces from right  */
             .replace(  /(\S)\s+(\S)/g,     "$1 $2"   )  /**  replace multiple spaces by single  */
  ;
}


/****************************************************************************/
/*! \brief  converts a text from whatever encoding to Unicode
**
**  \param[in]  szTxt  the input text
**  \param[in]  zType  encoding according to the server (not part of text)
**
**  \return  converted text
*/
function  wfxVersions_globalOps_ConvertToUnicode(  szTxt,  zType  )
{
  var  cType = zType.value;
  var  oConverter = Components.classes[ "@mozilla.org/intl/scriptableunicodeconverter" ]
                              .getService(  Components.interfaces.nsIScriptableUnicodeConverter  );

  /*! handle single-byte meta-charset strings
  **  - UTF-8
  **  - ISO-8859-1 / ISO-8859-2 / …
  **  - ANSI encodings
  */
  try
  {
    var  aMatch = szTxt.replace(  /(\s+)/g,  ""  ).match(  /charset=([^'"]+)['"]/i  );
    if(  aMatch  )
    {
      oConverter.charset = aMatch[1];
      zType.value        = aMatch[1];
      return  oConverter.ConvertToUnicode(  szTxt  );
    }
  }
  catch(  e  )
  {}

  /*! handle multi-byte meta-charset strings
  **  - UTF-8 with BOM
  **  - UTF-16 (UCS-2)
  **  - UTF-32
  */
  try
  {
    const  aCharSets =
    [
      {  aiBOM: [ 0x00, 0x00, 0xFE, 0xFF ],  szUNI: "UTF-32BE"  },
      {  aiBOM: [ 0xFF, 0xFE, 0x00, 0x00 ],  szUNI: "UTF-32LE"  },
      {  aiBOM: [ 0xFE, 0xFF ],              szUNI: "UTF-16BE"  },
      {  aiBOM: [ 0xFF, 0xFE ],              szUNI: "UTF-16LE"  },
      {  aiBOM: [ 0xEF, 0xBB, 0xBF ],        szUNI: "UTF-8"     }
    ];

    for(  var  i = 0;  i < aCharSets.length;  i++  )
    {
      var  bAttempt = true;
      for(  var  j = 0;  (  bAttempt  )&&(  j < aCharSets[i].aiBOM.length  );  j++  )
      {
        if(  ( szTxt.charCodeAt(j) & 0xFF ) != aCharSets[i].aiBOM[j]  )
        {
          bAttempt = false;
        }
      }

      if(  bAttempt  )
      {
        oConverter.charset = aCharSets[i].szUNI;
        zType.value        = aCharSets[i].szUNI;
        return  oConverter.ConvertToUnicode(  szTxt  );
      }
    }
  }
  catch(  e  )
  {}

  /*! handle missing meta-charset strings
  **  - try with the server response-header content-type charset
  **  - try with preference ¨intl.charset.default¨
  **  - try with UTF-8 without BOM
  */
  try
  {
    try
    {
      if((  cType  )&&(  cType.length  ))
      {
        oConverter.charset = cType;
        zType.value        = cType;
        return  oConverter.ConvertToUnicode(  szTxt  );
      }
    }
    catch(  e  )
    {}

    var  szDefault = "ISO-8859-1";
    try
    {
      szDefault = Components.classes[ "@mozilla.org/preferences-service;1" ]
                            .getService(  Components.interfaces.nsIPrefBranch  )
                            .getComplexValue(  "intl.charset.default",  Components.interfaces.nsISupportsString  )
                            .data;
    }
    catch(  e  )
    {}

    try
    {
      oConverter.charset = szDefault;
      zType.value        = szDefault;
      return  oConverter.ConvertToUnicode(  szTxt  );
    }
    catch(  e  )
    {
      try
      {
        if(  szDefault.toUpperCase() != "UTF-8"  )
        {
          oConverter.charset = "UTF-8";
          zType.value        = "UTF-8";
          return  oConverter.ConvertToUnicode(  szTxt  );
        }
      }
      catch(  e  )
      {}
    }

    if(  szDefault.toUpperCase() != "ISO-8859-1"  )
    {
      oConverter.charset = "ISO-8859-1";
      zType.value        = "ISO-8859-1";
      return  oConverter.ConvertToUnicode(  szTxt  );
    }
  }
  catch(  e  )
  {}

  zType.value = "";
  return  szTxt;
}


/****************************************************************************/
/*! \brief  Launch a one-shot timer (timeout)
**
**  Creating a nsITimer object is better than using window.setTimeout()
**  because it works under modal windows on Firefox 3.
**
**  \param[in]  szCode  code to be executed at timeout
**  \param[in]  iDelay  the timeout delay in miliseconds
**
**  \return  timer object
**
**  \see  wfxVersions_globalOps_onTimer()
*/
function  wfxVersions_globalOps_setTimeout(  szCode,  iDelay  )
{
  var  szTryCatch = "try"              +
                    "{"                +
                       szCode + ";"    +
                    "}"                +
                    "catch(  e  )"     +
                    "{"                +
                       /*  protect against e.g. window closed before timer  */
                    "}";
  var  oTimer = null;

  try
  {
    /** 1.
    **  The example with dynamic function from
    **    https://developer.mozilla.org/En/NsITimer
    **  works, but causes memory leaks (as shown by Leak Monitor addon).
    **  \code
    **    callback = {  notify:  function(  timer  )
    **                           {
    **                             code
    **                           }
    **               };
    **  \endcode
    **  Therefore a static function is used instead.
    **
    **  2.
    **  XPCOM objects may not be modified by adding new properties.
    **  \code
    **    timer.newProperty = code;
    **  \endcode
    **  Therefore we store the code for _setTimeout() as window properties,
    **  so the timer-callback must search in the window properties to match
    **  the timer-object and retrieve the corresponding code to execute.
    */
    oTimer = Components.classes[ "@mozilla.org/timer;1" ]
                       .createInstance(  Components.interfaces.nsITimer  );
    oTimer.initWithCallback(  { notify: wfxVersions_globalOps_onTimer },
                              iDelay,
                              Components.interfaces.nsITimer.TYPE_ONE_SHOT  );

    /*  store the code as window properties  */
    if(  !( "wfxVersions_aTimers" in window )  )
    {
      window["wfxVersions_aTimers"] = [];
    }
    window["wfxVersions_aTimers"].push(  { wfxObj: oTimer, wfxCode: szTryCatch }  );
  }
  catch(  e  )
  {
    oTimer = setTimeout(  szTryCatch,  iDelay  );
  }

  return(  oTimer  );
}


/****************************************************************************/
/*! \brief  Timer dispatch function, compatible with nsITimerCallback
**
**  Only used in the _setTimeout() function, but not defined dynamically
**  because it causes memory leaks.
**
**  \param[in]  eTimer  the timer object
**
**  \return  --
*/
function  wfxVersions_globalOps_onTimer(  eTimer  )
{
  if(  !( "wfxVersions_aTimers" in window )  )
  {
    return;
  }

  /*  retrieve the code from window properties  */
  for(  var  iTimer = 0;  iTimer < window["wfxVersions_aTimers"].length;  iTimer++  )
  {
    var  xTimer = window["wfxVersions_aTimers"][iTimer];
    if(  xTimer.wfxObj == eTimer  )
    {
      window["wfxVersions_aTimers"].splice(  iTimer,  1  );

      var  oBox = document.getElementById(  "wfxVersions_windowIDs_hboxTimer"  );
      if(  !oBox  )
      {
        oBox = document.createElement(  "hbox"  );
        oBox.setAttribute(  "id",  "wfxVersions_windowIDs_hboxTimer"  );
        document.documentElement.appendChild(  oBox  );
      }
      oBox.setAttribute(  "oncommand",  xTimer.wfxCode  );
      oBox.doCommand();

      return;
    }
  }
}


/****************************************************************************/
/*! \brief  Called for stopping a timeout from firing
**
**  \param[in]  eTimer  the timer object
**
**  \return  --
*/
function  wfxVersions_globalOps_stopTimer(  eTimer  )
{
  try
  {
    eTimer.cancel();

    if(  !( "wfxVersions_aTimers" in window )  )
    {
      return;
    }

    /*  retrieve the code from window properties  */
    for(  var  iTimer = 0;  iTimer < window["wfxVersions_aTimers"].length;  iTimer++  )
    {
      var  xTimer = window["wfxVersions_aTimers"][iTimer];
      if(  xTimer.wfxObj == eTimer  )
      {
        window["wfxVersions_aTimers"].splice(  iTimer,  1  );
        return;
      }
    }
  }
  catch(  e  )
  {}
}


/****************************************************************************/
/*! \brief  Display a simple alert
**
**  Better than the window.alert because it accepts a title
**
**  \param[in]  szTitle  title for the alert dialog
**  \param[in]  szText   message text
**
**  \return  --
*/
function  wfxVersions_globalOps_alert(  szTitle,  szText  )
{
  var  oPrompts = Components.classes[ "@mozilla.org/embedcomp/prompt-service;1" ]
                            .getService(  Components.interfaces.nsIPromptService  );
  oPrompts.alert(  window,  szTitle,  szText  );
}


/****************************************************************************/
/*! \brief  Display a simple confirm dialog
**
**  Better than the window.confirm because it accepts a title
**
**  \param[in]  szTitle  title for the confirm dialog
**  \param[in]  szText   message text
**
**  \return  boolean = Ok / Cancel
*/
function  wfxVersions_globalOps_confirm(  szTitle,  szText  )
{
  var  oPrompts = Components.classes[ "@mozilla.org/embedcomp/prompt-service;1" ]
                            .getService(  Components.interfaces.nsIPromptService  );
  return  oPrompts.confirm(  window,  szTitle,  szText  );
}


/****************************************************************************/
/*! \brief  Display all properties of an object (and their types/values)
**
**  This is a function "for development use only"
**
**  \param[in]  obj  the object for which the query is made
**
**  \return  --
*/
function  wfxVersions_globalOps_queryProperties(  obj  )
{try
{
  var  txt = "";
  var  lin = 0;
  for(  property in obj  )
  {
    var  spa = lin + "> " + property;
    try
    {
      spa += " (" + typeof( obj[property] ) + ") = ";
    }
    catch(  e  )
    {
      spa += " (" + e.name + ") = ";
    }
    try
    {
      if(  typeof( obj[property] ) == "function"  )
      {
        spa += "{...}";
      }
      else
      {
        spa += obj[property];
      }
    }
    catch(  e  )
    {
      spa += e.name;
    }

    txt += spa.replace(  /[ \t\n\r]/g,  String.fromCharCode(160)  );
    lin++;
    if(  (lin % 64) == 0  )
    {
      alert(  txt  );
      txt = "";
    }
    else
    {
      txt += "\n";
    }
  }
  if(  txt.length != 0  )
  {
    alert(  txt  );
  }
}
catch(e){alert(e);}}
