/****************************************************************************/
/*! \file
**  \brief  Versions (wfx_Versions) popup-box script
**
**  This file implements functions used by the popup to select between .lists
**  for Scan and Edit modes.
**
**  This is special handling for Firefox 2 or less, because
**  'panel' was introduced only in Fx3+.
**
**  \n
**  Copyright  ©  aNDreas Bolotă  aNDreas·Bolota(α)kirnbie·com
**
**  Versions (wfx_Versions) allows you to find out the current version number
**  of any program downloadable online.
**  Versions (wfx_Versions) emulates the behavior of Versions © Fabio Chelly
**  file-system plugin for Total Commander © Christian Ghisler.
**
**  This program is free software: you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation, either version 3 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program.  If not, see <http://www.gnu.org/licenses/>
*/


/****************************************************************************/
/*! \brief  After a delay from loading, move the popup over the opener button
**
**  The delay is required such that the popup window is drawn and we can
**  retrieve it's sizes
**
**  \param[in]  iNewX  screenX of the opener button (our target)
**  \param[in]  iNewY  screenY of the opener button (our target)
**
**  \return  --
*/
function  wfxVersions_popupOps_moveWindow(  iNewX,  iNewY  )
{
  if(  opener.wfxVersions_dialogVars_popupAdjust.moved  )
  {
    /*  second time inside popup, so it should be placed correctly now  */
  }
  else
  {
    /*  first time inside popup, need to evaluate move coords and reload  */

    var  iOldX = document.getElementById(  "wfxVersions_popupIDs_button"  ).boxObject.screenX;
    var  iOldY = document.getElementById(  "wfxVersions_popupIDs_button"  ).boxObject.screenY;

    opener.wfxVersions_dialogVars_popupAdjust.moveX += iNewX - iOldX;
    opener.wfxVersions_dialogVars_popupAdjust.moveY += iNewY - iOldY;
    opener.wfxVersions_dialogVars_popupAdjust.moved = true;

    opener.wfxVersions_globalOps_setTimeout(  "wfxVersions_dialogOps_reloadPopup()",  40  );
    wfxVersions_popupOps_exitOnBlur(  null  );
  }
}


/****************************************************************************/
/*! \brief  Handles the `onload` event
**
**  Fills the popup dialog with the corresponding .lists selection depending
**  on Scan/Edit operation
**
**  \param  --
**
**  \return  --
*/
function  wfxVersions_popupOps_populateLists()
{
  wfxVersions_globalOps_nodeCleanUp(  document.documentElement  );

  switch(  opener.wfxVersions_dialogVars_popupAdjust.popup  )
  {
    default:
      wfxVersions_popupOps_populateScan();
    break;

    case  1:
      wfxVersions_popupOps_populateEdit();
    break;

    case  2:
      wfxVersions_popupOps_populateCopy();
    break;
  }
}


/****************************************************************************/
/*! \brief  Populates the popup window with .lists for Scanning selection
**
**  The \e opener.wfxVersions_dialogVars_filesList is used for creating
**  checkboxes with the .lists to be selected
**
**  \param  --
**
**  \return  --
*/
function  wfxVersions_popupOps_populateScan()
{
  var  oButton = opener.document.getElementById(  "wfxVersions_dialogIDs_buttonScan"  );

  var  szNoReads = "";
  var  oLists = [];
  while(  opener.wfxVersions_dialogVars_filesList.length != 0  )
  {
    var  fsList = opener.wfxVersions_dialogVars_filesList.pop();
    if(  fsList.isReadable()  )
    {
      oLists.push(  fsList  );
    }
    else
    {
      szNoReads += "\n" + fsList.path;
    }
  }

  if(  oLists.length == 0  )
  {
    /*  files were found, but could not be opened  */
    opener.wfxVersions_globalOps_setTimeout(  "wfxVersions_globalOps_alert('" + oButton.getAttribute("tooltiptext") + "',wfxVersions_dialogOps_localeParams('szFileAccess') + '\n" + szNoReads + "')",  10  );
    wfxVersions_popupOps_exitOnBlur(  null  );
    return;
  }

  var  oBox = document.createElement(  "hbox"  );
  oBox.setAttribute(  "id",  "wfxVersions_popupIDs_topHorizontal"  );
  oBox.style.setProperty(  "margin",   "0px",  "important"  );
  oBox.style.setProperty(  "padding",  "0px",  "important"  );
  document.documentElement.appendChild(  oBox  );

  var  oTop = document.createElement(  "vbox"  );
  oTop.setAttribute(  "id",  "wfxVersions_popupIDs_topVertical"  );
  oBox.appendChild(  oTop  );

  oBox = document.createElement(  "vbox"  );
  oBox.setAttribute(  "id",  "wfxVersions_popupIDs_checkboxGroup"  );
  oTop.appendChild(  oBox  );

  var  iParent = oLists[0].path.lastIndexOf(  '\\'  );
  if(  iParent < 0  )
  {
    iParent = oLists[0].path.lastIndexOf(  '/'  );
  }
  var  szParent = oLists[0].path.substring(  0,  1 + iParent  );

  var  szAllChecked = "true";
  if(  wfxVersions_popupOps_anyMemorized( szParent )  )
  {
    szAllChecked = "false";
  }

  var  iTarget   = oLists.length;
  var  iProgress = 0;

  var  oItem;
  while(  oLists.length != 0  )
  {
    var  szOneChecked = "false";
    if((  szAllChecked == "true"  )||(  wfxVersions_popupOps_isMemorized( oLists[ oLists.length - 1 ].path )  ))
    {
      szOneChecked = "true";
      iProgress++;
    }

    oItem = document.createElement(  "checkbox"  );
    oItem.setAttribute(  "id",           "wfxVersions_popupIDs_checkbox" + oLists.length  );
    oItem.setAttribute(  "label",        oLists[ oLists.length - 1 ].leafName + String.fromCharCode(160)  );
    oItem.setAttribute(  "tooltiptext",  oLists[ oLists.length - 1 ].path  );
    oItem.setAttribute(  "oncommand",    "wfxVersions_popupOps_checkOne()"  );
    oItem.setAttribute(  "checked",      szOneChecked );
    oBox.appendChild(  oItem  );

    opener.wfxVersions_dialogVars_filesList.push(  oLists.pop()  );
  }

  oBox = document.createElement(  "hbox"  );
  oBox.setAttribute(  "id",  "wfxVersions_popupIDs_hbox"  );
  oBox.style.setProperty(  "padding",        "1px",  "important"  );
  oBox.style.setProperty(  "margin-top",     "3px",  "important"  );
  oBox.style.setProperty(  "margin-bottom",  "3px",  "important"  );
  oBox.style.setProperty(  "margin-left",    "0px",  "important"  );
  oBox.style.setProperty(  "margin-right",   "0px",  "important"  );
  oTop.appendChild(  oBox  );

  if(  iProgress > (iTarget / 2)  )
  {
    szAllChecked = "true";
  }

  oItem = document.createElement(  "checkbox"  );
  oItem.setAttribute(  "id",           "wfxVersions_popupIDs_checkboxAll"  );
  oItem.setAttribute(  "label",        "*" + String.fromCharCode(160)  );
  oItem.setAttribute(  "tooltiptext",  opener.wfxVersions_dialogVars_filesList[0].path.replace( /(\/|\\)[^\/\\]+\.list$/, "$1*.list" )  );
  oItem.setAttribute(  "oncommand",    "wfxVersions_popupOps_checkAll(event.target.checked)"  );
  oItem.setAttribute(  "checked",      szAllChecked  );
  oBox.appendChild(  oItem  );

  oItem = document.createElement(  "spacer"  );
  oItem.setAttribute(  "id",    "wfxVersions_popupIDs_spacer"  );
  oItem.setAttribute(  "flex",  "1"  );
  oItem.style.setProperty(  "padding",  "0px",  "important"  );
  oItem.style.setProperty(  "margin",   "0px",  "important"  );
  oBox.appendChild(  oItem  );

  oItem = document.createElement(  "button"  );
  oItem.setAttribute(  "id",           "wfxVersions_popupIDs_button"  );
  oItem.setAttribute(  "label",        oButton.getAttribute( "label" )  );
  oItem.setAttribute(  "image",        oButton.getAttribute( "image" )  );
  oItem.setAttribute(  "accesskey",    oButton.getAttribute( "accesskey" )  );
  oItem.setAttribute(  "tooltiptext",  oButton.getAttribute( "tooltiptext" )  );
  oItem.setAttribute(  "oncommand",    "wfxVersions_popupOps_selectScan(event.target.getAttribute('tooltiptext'))"  );
  oBox.appendChild(  oItem  );

  oItem = document.createElement(  "label"  );
  oItem.setAttribute(  "value",  "  "  );
  oItem.setAttribute(  "id",     "wfxVersions_popupIDs_widthAdjust"  );
  oItem.style.setProperty(  "margin",      "0px",     "important"  );
  oItem.style.setProperty(  "padding",     "0px",     "important"  );
  oItem.style.setProperty(  "visibility",  "hidden",  "important"  );
  document.getElementById(  "wfxVersions_popupIDs_topHorizontal"  ).appendChild(  oItem  );

  wfxVersions_globalOps_setTimeout(  "wfxVersions_popupOps_moveWindow(" + oButton.boxObject.screenX + "," + oButton.boxObject.screenY + ")",  10  );
}


/****************************************************************************/
/*! \brief  Checks/unchecks the `All` checkbox when the others are clicked
**
**  The `All` checkbox is a sum of all other checkboxes
**
**  \param  --
**
**  \return  --
*/
function  wfxVersions_popupOps_checkOne()
{
  var  oGroup  = document.getElementById(  "wfxVersions_popupIDs_checkboxGroup"  );
  var  oBoxAll = document.getElementById(  "wfxVersions_popupIDs_checkboxAll"  );
  var  iYes    = 0;
  var  iNot    = 0;

  var  iPos = oGroup.childNodes.length - 1;
  while(  iPos >= 0  )
  {
    if(  oGroup.childNodes[iPos].checked  )
    {
      iYes++;
    }
    else
    {
      iNot++;
    }
    iPos--;
  }

  if(  iYes == 0  )
  {
    oBoxAll.checked = false;
  }
  else
  if(  iNot == 0  )
  {
    oBoxAll.checked = true;
  }
  else
  {}
}


/****************************************************************************/
/*! \brief  Checks/unchecks all .lists when the `All` checkbox is clicked
**
**  \param[in]  bChecked  state of the `All` checkbox
**
**  \return  --
*/
function  wfxVersions_popupOps_checkAll(  bChecked  )
{
  var  oGroup = document.getElementById(  "wfxVersions_popupIDs_checkboxGroup"  );
  var  iPos   = oGroup.childNodes.length - 1;
  while(  iPos >= 0  )
  {
    oGroup.childNodes[iPos].checked = bChecked;
    iPos--;
  }
}


/****************************************************************************/
/*! \brief  Activated after the user has selected which .lists to Scan
**
**  The .list files not checked by the user must be removed from scanning
**
**  \param[in]  szTip  the tooltip of the Scan button
**
**  \return  --
*/
function  wfxVersions_popupOps_selectScan(  szTip  )
{
  var  oGroup = document.getElementById(  "wfxVersions_popupIDs_checkboxGroup"  );
  var  iPos   = oGroup.childNodes.length - 1;
  var  oLists = [];

  while(  iPos >= 0  )
  {
    if(  oGroup.childNodes[iPos].checked  )
    {
      oLists.push(  opener.wfxVersions_dialogVars_filesList.pop()  );
    }
    else
    {
      opener.wfxVersions_dialogVars_filesList.pop();
    }
    iPos--;
  }

  while(  oLists.length != 0  )
  {
    opener.wfxVersions_dialogVars_filesList.push(  oLists.pop()  );
  }
  opener.wfxVersions_globalOps_setTimeout(  "wfxVersions_scanOps_start('" + szTip + "')",  10  );
  wfxVersions_popupOps_exitOnBlur(  null  );
}


/****************************************************************************/
/*! \brief  Populates the popup window with .lists for Editing selection
**
**  The \e opener.wfxVersions_dialogVars_filesList is used for creating
**  radiobuttons with the .lists to be selected
**
**  \param  --
**
**  \return  --
*/
function  wfxVersions_popupOps_populateEdit()
{
  var  oButton = opener.document.getElementById(  "wfxVersions_dialogIDs_buttonEdit"  );
  var  oDelete = opener.document.getElementById(  "wfxVersions_dialogIDs_editDelete"  );

  var  szNoWrites = "";
  var  oLists = [];
  while(  opener.wfxVersions_dialogVars_filesList.length != 0  )
  {
    var  fsList = opener.wfxVersions_dialogVars_filesList.pop();
    if(  fsList.isReadable()  )
    {
      oLists.push(  fsList  );
    }
    else
    {
      szNoWrites += "\n" + fsList.path;
    }
  }

  if(  oLists.length == 0  )
  {
    /*  files were found, but could not be opened  */
    opener.wfxVersions_globalOps_setTimeout(  "wfxVersions_globalOps_alert('" + oButton.getAttribute("tooltiptext") + "',wfxVersions_dialogOps_localeParams('szFileAccess') + '\n" + szNoWrites + "')",  10  );
    wfxVersions_popupOps_exitOnBlur(  null  );
    return;
  }

  var  oBox = document.createElement(  "hbox"  );
  oBox.setAttribute(  "id",  "wfxVersions_popupIDs_topHorizontal"  );
  oBox.style.setProperty(  "margin",   "0px",  "important"  );
  oBox.style.setProperty(  "padding",  "0px",  "important"  );
  document.documentElement.appendChild(  oBox  );

  var  oTop = document.createElement(  "vbox"  );
  oTop.setAttribute(  "id",  "wfxVersions_popupIDs_topVertical"  );
  oBox.appendChild(  oTop  );

  oBox = document.createElement(  "radiogroup"  );
  oBox.setAttribute(  "id",         "wfxVersions_popupIDs_radiogroup"  );
  oBox.setAttribute(  "oncommand",  "wfxVersions_popupOps_deletableEdit()"  );
  oTop.appendChild(  oBox  );

  var  iPos = oLists[0].path.lastIndexOf(  '\\'  );
  if(  iPos < 0  )
  {
    iPos = oLists[0].path.lastIndexOf(  '/'  );
  }
  var  szParent = oLists[0].path.substring(  0,  1 + iPos  );

  var  oItem;
  while(  oLists.length != 0  )
  {
    oItem = document.createElement(  "radio"  );
    oItem.setAttribute(  "id",           "wfxVersions_popupIDs_radio" + oLists.length  );
    oItem.setAttribute(  "label",        oLists[ oLists.length - 1 ].leafName + String.fromCharCode(160)  );
    oItem.setAttribute(  "tooltiptext",  oLists[ oLists.length - 1 ].path  );
    oBox.appendChild(  oItem  );

    opener.wfxVersions_dialogVars_filesList.push(  oLists.pop()  );
  }

  oItem = document.createElement(  "radio"  );
  oItem.setAttribute(  "id",           "wfxVersions_popupIDs_radioNew"  );
  oItem.setAttribute(  "label",        opener.wfxVersions_dialogOps_localeParams( "szNewList" ) + String.fromCharCode(160)  );
  oItem.setAttribute(  "tooltiptext",  opener.wfxVersions_dialogOps_localeParams( "szNewListTip" )  );
  oBox.appendChild(  oItem  );

  for(  iPos = 0;  iPos < (oBox.childNodes.length - 1);  iPos++  )
  {
    if(  wfxVersions_popupOps_isMemorized( oBox.childNodes[iPos].getAttribute("tooltiptext") )  )
    {
      break;
    }
  }
  if((  iPos == (oBox.childNodes.length - 1)  )&&(  !wfxVersions_popupOps_anyMemorized( szParent )  ))
  {
    iPos = 0;
  }

  oBox.childNodes[iPos].setAttribute(  "selected",  "true"  );

  oBox = document.createElement(  "hbox"  );
  oBox.setAttribute(  "id",  "wfxVersions_popupIDs_hbox"  );
  oBox.style.setProperty(  "padding",        "1px",  "important"  );
  oBox.style.setProperty(  "margin-top",     "3px",  "important"  );
  oBox.style.setProperty(  "margin-bottom",  "3px",  "important"  );
  oBox.style.setProperty(  "margin-left",    "0px",  "important"  );
  oBox.style.setProperty(  "margin-right",   "0px",  "important"  );
  oTop.appendChild(  oBox  );

  oItem = document.createElement(  "button"  );
  oItem.setAttribute(  "id",           "wfxVersions_popupIDs_button"  );
  oItem.setAttribute(  "label",        oButton.getAttribute( "label" )  );
  oItem.setAttribute(  "image",        oButton.getAttribute( "image" )  );
  oItem.setAttribute(  "accesskey",    oButton.getAttribute( "accesskey" )  );
  oItem.setAttribute(  "tooltiptext",  oButton.getAttribute( "tooltiptext" )  );
  oItem.setAttribute(  "oncommand",    "wfxVersions_popupOps_selectEdit(event.target.getAttribute('tooltiptext'))"  );
  oBox.appendChild(  oItem  );

  oItem = document.createElement(  "spacer"  );
  oItem.setAttribute(  "id",    "wfxVersions_popupIDs_spacer"  );
  oItem.setAttribute(  "flex",  "1"  );
  oItem.style.setProperty(  "padding",  "0px",  "important"  );
  oItem.style.setProperty(  "margin",   "0px",  "important"  );
  oBox.appendChild(  oItem  );

  oItem = document.createElement(  "button"  );
  oItem.setAttribute(  "id",           "wfxVersions_popupIDs_delete"  );
  oItem.setAttribute(  "label",        oDelete.getAttribute( "label" )  );
  oItem.setAttribute(  "image",        oDelete.getAttribute( "image" )  );
  oItem.setAttribute(  "accesskey",    oDelete.getAttribute( "accesskey" )  );
  oItem.setAttribute(  "tooltiptext",  opener.wfxVersions_dialogOps_localeParams( "szDelFile" )  );
  oItem.setAttribute(  "oncommand",    "wfxVersions_popupOps_selectDelete('" + opener.wfxVersions_dialogOps_localeParams( "szDelFile" ) + "')"  );
  oBox.appendChild(  oItem  );

  oItem = document.createElement(  "label"  );
  oItem.setAttribute(  "value",  "  "  );
  oItem.setAttribute(  "id",     "wfxVersions_popupIDs_widthAdjust"  );
  oItem.style.setProperty(  "margin",      "0px",     "important"  );
  oItem.style.setProperty(  "padding",     "0px",     "important"  );
  oItem.style.setProperty(  "visibility",  "hidden",  "important"  );
  document.getElementById(  "wfxVersions_popupIDs_topHorizontal"  ).appendChild(  oItem  );

  wfxVersions_globalOps_setTimeout(  "wfxVersions_popupOps_moveWindow(" + oButton.boxObject.screenX + "," + oButton.boxObject.screenY + ")",  10  );
}


/****************************************************************************/
/*! \brief  Show/hide the [Delete] button
**
**  The (new .list) entry cannot be deleted, so the button is disabled if this
**  entry is selected
**
**  \param  --
**
**  \return  --
*/
function  wfxVersions_popupOps_deletableEdit()
{
  var  oGroup = document.getElementById(  "wfxVersions_popupIDs_radiogroup"  );
  if(  oGroup.selectedIndex >= opener.wfxVersions_dialogVars_filesList.length  )
  {
    document.getElementById(  "wfxVersions_popupIDs_delete"  ).setAttribute(  "disabled",  "true"  );
  }
  else
  {
    document.getElementById(  "wfxVersions_popupIDs_delete"  ).removeAttribute(  "disabled"  );
  }
}


/****************************************************************************/
/*! \brief  Activated after the user has selected which .list to Edit
**
**  The .list files not checked by the user must be removed from editing
**
**  \param[in]  szTip  the tooltip of the Edit button
**
**  \return  --
*/
function  wfxVersions_popupOps_selectEdit(  szTip  )
{
  var  oGroup = document.getElementById(  "wfxVersions_popupIDs_radiogroup"  );
  var  iSel   = oGroup.selectedIndex;
  if(  iSel >= opener.wfxVersions_dialogVars_filesList.length  )
  {
    while(  opener.wfxVersions_dialogVars_filesList.length > 1  )
    {
      opener.wfxVersions_dialogVars_filesList.pop();
    }
    opener.wfxVersions_dialogVars_filesList[0] = opener.wfxVersions_dialogVars_filesList[0].parent;
    opener.wfxVersions_globalOps_setTimeout(  "wfxVersions_editOps_createFile('" + opener.wfxVersions_dialogOps_localeParams("szNewListTip") + "')",  10  );
  }
  else
  {
    var  iPos = oGroup.childNodes.length - 2;
    var  oLists = [];

    while(  iPos >= 0  )
    {
      if(  iPos == iSel  )
      {
        oLists.push(  opener.wfxVersions_dialogVars_filesList.pop()  );
      }
      else
      {
        opener.wfxVersions_dialogVars_filesList.pop();
      }
      iPos--;
    }

    while(  oLists.length != 0  )
    {
      opener.wfxVersions_dialogVars_filesList.push(  oLists.pop()  );
    }
    opener.wfxVersions_globalOps_setTimeout(  "wfxVersions_editOps_start('" + szTip + "')",  10  );
  }
  wfxVersions_popupOps_exitOnBlur(  null  );
}


/****************************************************************************/
/*! \brief  Delete a .list file
**
**  \param[in]  szTip  name of operation performed, used in error alerts
**
**  \return  --
*/
function  wfxVersions_popupOps_selectDelete(  szTip  )
{
  var  oGroup = document.getElementById(  "wfxVersions_popupIDs_radiogroup"  );
  var  iPos   = oGroup.childNodes.length - 2;
  var  iSel   = oGroup.selectedIndex;
  var  oLists = [];

  while(  iPos >= 0  )
  {
    if(  iPos == iSel  )
    {
      oLists.push(  opener.wfxVersions_dialogVars_filesList.pop()  );
    }
    else
    {
      opener.wfxVersions_dialogVars_filesList.pop();
    }
    iPos--;
  }

  while(  oLists.length != 0  )
  {
    opener.wfxVersions_dialogVars_filesList.push(  oLists.pop()  );
  }
  opener.wfxVersions_globalOps_setTimeout(  "wfxVersions_editOps_deleteFile('" + szTip + "')",  10  );
  wfxVersions_popupOps_exitOnBlur(  null  );
}


/****************************************************************************/
/*! \brief  Populates the popup window with .lists for Copying selection
**
**  The \e opener.wfxVersions_dialogVars_filesList is used for creating
**  radiobuttons with the .lists to be selected
**
**  \param  --
**
**  \return  --
*/
function  wfxVersions_popupOps_populateCopy()
{
  var  oButton = opener.document.getElementById(  "wfxVersions_dialogIDs_editCopy"  );

  var  szNoWrites = "";
  var  oLists = [];
  while(  opener.wfxVersions_dialogVars_filesList.length != 0  )
  {
    var  fsList = opener.wfxVersions_dialogVars_filesList.pop();
    if(  fsList.isReadable()  )
    {
      oLists.push(  fsList  );
    }
    else
    {
      szNoWrites += "\n" + fsList.path;
    }
  }

  if(  oLists.length == 0  )
  {
    /*  files were found, but could not be opened  */
    opener.wfxVersions_globalOps_setTimeout(  "wfxVersions_globalOps_alert('" + oButton.getAttribute("tooltiptext") + "',wfxVersions_dialogOps_localeParams('szFileAccess') + '\n" + szNoWrites + "')",  10  );
    wfxVersions_popupOps_exitOnBlur(  null  );
    return;
  }

  var  oBox = document.createElement(  "hbox"  );
  oBox.setAttribute(  "id",  "wfxVersions_popupIDs_topHorizontal"  );
  oBox.style.setProperty(  "margin",   "0px",  "important"  );
  oBox.style.setProperty(  "padding",  "0px",  "important"  );
  document.documentElement.appendChild(  oBox  );

  var  oTop = document.createElement(  "vbox"  );
  oTop.setAttribute(  "id",  "wfxVersions_popupIDs_topVertical"  );
  oBox.appendChild(  oTop  );

  oBox = document.createElement(  "radiogroup"  );
  oBox.setAttribute(  "id",  "wfxVersions_popupIDs_radiogroup"  );
  oTop.appendChild(  oBox  );

  var  iPos = oLists[0].path.lastIndexOf(  '\\'  );
  if(  iPos < 0  )
  {
    iPos = oLists[0].path.lastIndexOf(  '/'  );
  }
  var  szParent = oLists[0].path.substring(  0,  1 + iPos  );

  var  oItem;
  while(  oLists.length != 0  )
  {
    oItem = document.createElement(  "radio"  );
    oItem.setAttribute(  "id",           "wfxVersions_popupIDs_radio" + oLists.length  );
    oItem.setAttribute(  "label",        oLists[ oLists.length - 1 ].leafName + String.fromCharCode(160)  );
    oItem.setAttribute(  "tooltiptext",  oLists[ oLists.length - 1 ].path  );
    oBox.appendChild(  oItem  );

    opener.wfxVersions_dialogVars_filesList.push(  oLists.pop()  );
  }

  var  iSelf = 0;
  var  szSelf = opener.document.getElementById(  "wfxVersions_dialogIDs_tabs"  ).firstChild.getAttribute(  "tooltiptext"  );

  for(  iPos = 0;  iPos < oBox.childNodes.length;  iPos++  )
  {
    var  szPos = oBox.childNodes[iPos].getAttribute(  "tooltiptext"  );

    if(  wfxVersions_popupOps_isMemorized( szPos )  )
    {
      break;
    }
    if(  szPos == szSelf  )
    {
      iSelf = iPos;
    }
  }
  if(  iPos == oBox.childNodes.length  )
  {
    iPos = iSelf;
  }
  oBox.childNodes[iPos].setAttribute(  "selected",  "true"  );

  oBox = document.createElement(  "hbox"  );
  oBox.setAttribute(  "id",  "wfxVersions_popupIDs_hbox"  );
  oBox.style.setProperty(  "padding",        "1px",  "important"  );
  oBox.style.setProperty(  "margin-top",     "3px",  "important"  );
  oBox.style.setProperty(  "margin-bottom",  "3px",  "important"  );
  oBox.style.setProperty(  "margin-left",    "0px",  "important"  );
  oBox.style.setProperty(  "margin-right",   "0px",  "important"  );
  oTop.appendChild(  oBox  );

  oItem = document.createElement(  "button"  );
  oItem.setAttribute(  "id",           "wfxVersions_popupIDs_button"  );
  oItem.setAttribute(  "label",        oButton.getAttribute( "label" )  );
  oItem.setAttribute(  "image",        oButton.getAttribute( "image" )  );
  oItem.setAttribute(  "accesskey",    oButton.getAttribute( "accesskey" )  );
  oItem.setAttribute(  "tooltiptext",  oButton.getAttribute( "tooltiptext" )  );
  oItem.setAttribute(  "oncommand",    "wfxVersions_popupOps_selectCopy(event.target.getAttribute('tooltiptext'))"  );
  oBox.appendChild(  oItem  );

  oItem = document.createElement(  "spacer"  );
  oItem.setAttribute(  "id",    "wfxVersions_popupIDs_spacer"  );
  oItem.setAttribute(  "flex",  "1"  );
  oItem.style.setProperty(  "padding",  "0px",  "important"  );
  oItem.style.setProperty(  "margin",   "0px",  "important"  );
  oBox.appendChild(  oItem  );

  oItem = document.createElement(  "label"  );
  oItem.setAttribute(  "value",  "  "  );
  oItem.setAttribute(  "id",     "wfxVersions_popupIDs_widthAdjust"  );
  oItem.style.setProperty(  "margin",      "0px",     "important"  );
  oItem.style.setProperty(  "padding",     "0px",     "important"  );
  oItem.style.setProperty(  "visibility",  "hidden",  "important"  );
  document.getElementById(  "wfxVersions_popupIDs_topHorizontal"  ).appendChild(  oItem  );

  wfxVersions_globalOps_setTimeout(  "wfxVersions_popupOps_moveWindow(" + oButton.boxObject.screenX + "," + oButton.boxObject.screenY + ")",  10  );
}


/****************************************************************************/
/*! \brief  Activated after the user has selected in which .list to Copy
**
**  The .list files not checked by the user must be removed from copying
**
**  \param[in]  szTip  the tooltip of the Copy button
**
**  \return  --
*/
function  wfxVersions_popupOps_selectCopy(  szTip  )
{
  var  oGroup = document.getElementById(  "wfxVersions_popupIDs_radiogroup"  );
  var  iSel   = oGroup.selectedIndex;
  var  iPos   = oGroup.childNodes.length - 1;

  var  oLists = [];
  while(  iPos >= 0  )
  {
    if(  iPos == iSel  )
    {
      oLists.push(  opener.wfxVersions_dialogVars_filesList.pop()  );
    }
    else
    {
      opener.wfxVersions_dialogVars_filesList.pop();
    }
    iPos--;
  }

  while(  oLists.length != 0  )
  {
    opener.wfxVersions_dialogVars_filesList.push(  oLists.pop()  );
  }
  opener.wfxVersions_globalOps_setTimeout(  "wfxVersions_editOps_copyStart('" + szTip + "')",  10  );
  wfxVersions_popupOps_exitOnBlur(  null  );
}


/****************************************************************************/
/*! \brief  special handling of keys pressed
**
**  Associate ESC with {onclose} event, and ENTER with {onclick} event.
**
**  \param[in]  eKey  the onkey event
**
**  \return  event further handling allowed or not
*/
function  wfxVersions_popupOps_keyPress(  eKey  )
{
  if(  eKey.keyCode == eKey.DOM_VK_RETURN  )
  {
    /*  ENTER = the {onclick} event  */
    if((  eKey.target  )&&(  eKey.target.nodeName  )&&(  eKey.target.click  ))
    {
      if(  eKey.target.nodeName == "checkbox"  )
      {
        eKey.target.click();
      }
    }
    return  false;
  }
  else
  if(  eKey.keyCode == eKey.DOM_VK_ESCAPE  )
  {
    /*  ESC = the {onclose} event  */
    wfxVersions_popupOps_exitOnBlur(  null  );
    return  false;
  }
  else
  {}

  return  true;
}


/****************************************************************************/
/*! \brief  When focus is outside current popup window, close it
**
**  This ensures an open/close behavior similar to a normal popup, but allows
**  any elements on the dialog window
**
**  \param[in]  oFocus  the new window/element which is focused
**
**  \return  --
*/
function  wfxVersions_popupOps_exitOnBlur(  oFocus  )
{
  if(  closed  )
  {
    /*  current window is already closed  */
    return;
  }

  if(  opener.closed  )
  {
    /*  [wfx_Versions] main window was closed from the [X] button  */
    wfxVersions_globalOps_windowClose(  window  );
    return;
  }

  var  szID = "";
  if((  oFocus  )&&(  oFocus.getAttribute  ))
  {
    szID = oFocus.getAttribute(  "id"  );
  }

  if(  szID.match( /^wfxVersions_popupIDs_/ )  )
  {
    /*  focus is kept in current dialog, but on a different element  */
  }
  else
  {
    /*  focus is moved away from current window  */
    opener.wfxVersions_globalOps_setTimeout(  "wfxVersions_dialogOps_onPathChange()",  10  );
    wfxVersions_globalOps_windowClose(  window  );
  }
}


/****************************************************************************/
/*! \brief  Determine if a .list file is selected or not
**
**  \param[in]  szFile  path of the .list file to check
**
**  \return  boolean: true (memorized) / false (not memorized)
*/
function  wfxVersions_popupOps_isMemorized(  szFile  )
{
  var  iPos;
  for(  iPos = 0;  iPos < opener.wfxVersions_dialogVars_selections.length;  iPos++  )
  {
    if(  opener.wfxVersions_dialogVars_selections[iPos] == szFile  )
    {
      return  true;
    }
  }

  return  false;
}


/****************************************************************************/
/*! \brief  Determines if current directory of .lists has been selected before
**
**  \param[in]  szDir  path of the directory with .lists
**
**  \return  boolean: true (memorized) / false (not memorized)
*/
function  wfxVersions_popupOps_anyMemorized(  szDir  )
{
  var  iPos;
  for(  iPos = 0;  iPos < opener.wfxVersions_dialogVars_selections.length;  iPos++  )
  {
    var  iDir = opener.wfxVersions_dialogVars_selections[iPos].lastIndexOf(  '\\'  );
    if(  iDir < 0  )
    {
      iDir = opener.wfxVersions_dialogVars_selections[iPos].lastIndexOf(  '/'  );
    }

    if(  opener.wfxVersions_dialogVars_selections[iPos].substring( 0, 1 + iDir ) == szDir  )
    {
      return  true;
    }
  }

  return  false;
}
