/****************************************************************************/
/*! \file
**  \brief  Versions (wfx_Versions) main script
**
**  This file implements functions used by the Editing and Scanning modes.
**
**  \n
**  Copyright  ©  aNDreas Bolotă  aNDreas·Bolota(α)kirnbie·com
**
**  Versions (wfx_Versions) allows you to find out the current version number
**  of any program downloadable online.
**  Versions (wfx_Versions) emulates the behavior of Versions © Fabio Chelly
**  file-system plugin for Total Commander © Christian Ghisler.
**
**  This program is free software: you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation, either version 3 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program.  If not, see <http://www.gnu.org/licenses/>
*/


/****************************************************************************/
/*! \brief  Specific handling for Firefox 2 or less
**
**  e.g. <panel> as a popup was introduced only in Firefox 3
*/
var  wfxVersions_dialogVars_Fx3 = false;


/****************************************************************************/
/*! \brief  An array of .list files ( \e Components.interfaces.nsIFile )
**
**  Created on demand, when the user wants to Scan or Edit, from the contents
**  of the configured path
**
**  The files are not checked for accessibility (Read/Write) which must be
**  done later depending on the operation to be performed (Scan/Edit)
*/
var  wfxVersions_dialogVars_filesList = [];


/****************************************************************************/
/*! \brief  Memorized selections for Scan/Edit/Copy buttons
**
**  \see  wfxVersions_memoOps_recallSelections(),
**        wfxVersions_memoOps_storeSelections()
*/
var  wfxVersions_dialogVars_selections = [];


/****************************************************************************/
/*! \brief  Window resize in progress
**
**  Used for delayed event handling
*/
var  wfxVersions_dialogVars_resizeInProgress = false;


/****************************************************************************/
/*! \brief  Top/Left coords of the screen
**
**  The coords are NOT (0,0) with Firefox 3 on dual monitors, but the
**  window.screen still returns zeroes, so we need a workaround...
**
**  \see  wfxVersions_dialogOps_resizeOrigins()
*/
var  wfxVersions_dialogVars_resizeOrigin = {  X: 0,  Y: 0  };


/****************************************************************************/
/*! \brief  Stored return value of openDialog() for the custom about-box
**
**  So that the about-box window is opened only once
*/
var  wfxVersions_dialogVars_aboutBoxWindow = null;


/****************************************************************************/
/*! \brief  Holds relevant info for each application checked for version
**
**  Used during background scan (HTTP Requests) to compare new page with
**  old version
*/
var  wfxVersions_dialogVars_itemsList = null;


/****************************************************************************/
/*! \brief  Indicates current item being scanned
**
**  Index in the items-list
*/
var  wfxVersions_dialogVars_progress = -1;


/****************************************************************************/
/*! \brief  This object contains information about .list popup properties
**
**  \see  wfxVersions_dialogOps_reloadPopup
*/
var  wfxVersions_dialogVars_popupAdjust = {  moveX: 0,  moveY: 0,  moved: false,  popup: 0  };


/****************************************************************************/
/*! \brief  Timer object for periodic re-scanning
**
**  The timer can be canceled on user operation (e.g. re-scan individual item)
*/
var  wfxVersions_dialogVars_reScanTimer = null;


/****************************************************************************/
/*! \brief  Launches the custom about-box
**
**  To avoid opening multiple times, the openDialog() return value is stored
**
**  \param  --
**
**  \return  --
*/
function  wfxVersions_dialogOps_openAboutBox()
{
  if((  wfxVersions_dialogVars_aboutBoxWindow != null  )&&(  wfxVersions_dialogVars_aboutBoxWindow.document != null  ))
  {
    wfxVersions_globalOps_windowFocus(  wfxVersions_dialogVars_aboutBoxWindow  );
  }
  else
  {
    wfxVersions_dialogVars_aboutBoxWindow = openDialog(  "chrome://wfx_versions/content/wfx_versions_about.xul",
                                                         "",
                                                         "chrome,centerscreen,dependent"  );
  }
}


/****************************************************************************/
/*! \brief  convert localized string with parameters
**
**  \see  wfxVersions_dialogOps_localeParamsMod()
**
**  \param[in]  szString  the localized string
**
**  \return  the converted string
*/
function  wfxVersions_dialogOps_localeParams(  szString  )
{
  if((  wfxVersions_dialogVars_itemsList == null  )||(  wfxVersions_dialogVars_progress < 0  )||(  wfxVersions_dialogVars_progress >= wfxVersions_dialogVars_itemsList.length  ))
  {
    /*  nothing to replace from  */
    return  document.getElementById(  "wfxVersions_dialogIDs_strings"  ).getAttribute(  szString  );
  }

  var  oScan = wfxVersions_dialogVars_itemsList[wfxVersions_dialogVars_progress];

  var  oEdit = oScan;
  if(  document.getElementById( "wfxVersions_dialogIDs_tabs" ).childNodes.length <= 2  )
  {
    /*  edit-mode active  */
    var  szEditItem = document.getElementById(  "wfxVersions_dialogIDs_editLine"     ).value.replace(  /\|/g,  "&#124;"  )
              + "|" + document.getElementById(  "wfxVersions_dialogIDs_editHome"     ).value.replace(  /\|/g,  "&#124;"  )
              + "|" + document.getElementById(  "wfxVersions_dialogIDs_editDown"     ).value.replace(  /\|/g,  "&#124;"  )
              + "|" + document.getElementById(  "wfxVersions_dialogIDs_editURL"      ).value.replace(  /\|/g,  "&#124;"  )
              + "|" + document.getElementById(  "wfxVersions_dialogIDs_editStart"    ).value.replace(  /\|/g,  "&#124;"  )
              + "|" + document.getElementById(  "wfxVersions_dialogIDs_editPre"      ).value.replace(  /\|/g,  "&#124;"  )
              + "|" + document.getElementById(  "wfxVersions_dialogIDs_editPost"     ).value.replace(  /\|/g,  "&#124;"  )
              + "|" + document.getElementById(  "wfxVersions_dialogIDs_editVersion"  ).value.replace(  /\|/g,  "&#124;"  );

    oEdit = new wfxVersions_dialogOps_createItem(  szEditItem,  oScan.list  );
    oEdit.update = oEdit.version;
    oEdit.line   = oEdit.line .replace(  /&#124;/g,  "|"  );
    oEdit.home   = oEdit.home .replace(  /&#124;/g,  "|"  );
    oEdit.down   = oEdit.down .replace(  /&#124;/g,  "|"  );
    oEdit.url    = oEdit.url  .replace(  /&#124;/g,  "|"  );
    oEdit.start  = oEdit.start.replace(  /&#124;/g,  "|"  );
    oEdit.pre    = oEdit.pre  .replace(  /&#124;/g,  "|"  );
    oEdit.post   = oEdit.post .replace(  /&#124;/g,  "|"  );
  }

  /*  call the conversion function with the actual items  */
  return  wfxVersions_dialogOps_localeParamsMod(  szString,  oScan,  oEdit  );
}


/****************************************************************************/
/*! \brief  convert localized string with parameters
**
**  The possible parameters are taken from current items under scan/edit:
**  - [P] full .list file path
**  - [L] .list file name (without path)
**  - [A] string corresponding to {application name}
**  - [H] string corresponding to {application web-page (homepage)}
**  - [D] string corresponding to {application web-page (download link)}
**  - [W] string corresponding to {application web-page (scan page)}
**  - [S] string corresponding to {page-source point where to start the scan}
**  - [B] string corresponding to {page-source exactly before version string}
**  - [O] string corresponding to {page-source exactly after version string}
**  - [V] old version (stored), i.e. {the version}
**  - [N] modified [A] during Edit mode
**  - [C] modified [H] during Edit mode
**  - [K] modified [D] during Edit mode
**  - [R] modified [W] during Edit mode
**  - [Z] modified [S] during Edit mode
**  - [I] modified [B] during Edit mode
**  - [F] modified [O] during Edit mode
**  - [U] new version (update), also dueing Edit mode
**  - [T] scan timer (e.g. for timeout error, but also as `scan duration`)
**
**  \remark  [E] can also be used as replace mask, but only in case of error
**           at scanning, so it`s not replaced in this function
**
**  \param[in]  szString  the localized string
**  \param[in]  oScan     the scanned item, normally _itemsList[_progress]
**  \param[in]  oEdit     the edited item, normally getElementById(_edit)
**
**  \return  the converted string
*/
function  wfxVersions_dialogOps_localeParamsMod(  szString,  oScan,  oEdit  )
{
  return  document.getElementById( "wfxVersions_dialogIDs_strings" ).getAttribute( szString )
                  .replace(  "[P]",                                     oScan.list       )
                  .replace(  "[L]",       wfxVersions_fileOps_leafName( oScan.list )     )
                  .replace(  "[A]",                                     oScan.line       )
                  .replace(  "[H]",                                     oScan.home       )
                  .replace(  "[D]",                                     oScan.down       )
                  .replace(  "[W]",                                     oScan.url        )
                  .replace(  "[S]",                                     oScan.start      )
                  .replace(  "[B]",                                     oScan.pre        )
                  .replace(  "[O]",                                     oScan.post       )
                  .replace(  "[V]",  wfxVersions_globalOps_convertHTML( oScan.version )  )
                  .replace(  "[N]",                                     oEdit.line       )
                  .replace(  "[C]",                                     oEdit.home       )
                  .replace(  "[K]",                                     oEdit.down       )
                  .replace(  "[R]",                                     oEdit.url        )
                  .replace(  "[Z]",                                     oEdit.start      )
                  .replace(  "[I]",                                     oEdit.pre        )
                  .replace(  "[F]",                                     oEdit.post       )
                  .replace(  "[U]",  wfxVersions_globalOps_convertHTML( oEdit.update )   )
                  .replace(  "[T]",                                     oScan.step       );
}


/****************************************************************************/
/*! \brief  Returns the line corresponding to a scanned item
**
**  \param[in]  iPos  position in wfxVersions_dialogVars_itemsList[]
**
**  \return  line-string corresponding to that item
**
**  \see  wfxVersions_dialogOps_lineFromItem()
*/
function  wfxVersions_dialogOps_lineFromItem(  iPos  )
{
  if((  wfxVersions_dialogVars_itemsList == null  )||(  iPos >= wfxVersions_dialogVars_itemsList.length  ))
  {
    return  wfxVersions_dialogOps_localeParams( "szEmpty" );
  }

  return  wfxVersions_dialogOps_lineFromItemArray(  iPos,  wfxVersions_dialogVars_itemsList  );
}




/****************************************************************************/
/*! \brief  Returns the line corresponding to a scanned item
**
**  \param[in]  iPos   position in aList[]
**  \param[in]  aList  wfxVersions_dialogVars_itemsList[] or another array
**
**  \return  line-string corresponding to that item
*/
function  wfxVersions_dialogOps_lineFromItemArray(  iPos,  aList  )
{
  if(  aList[iPos].status == 6  )
  {
    return  aList[iPos].line;
  }

  var  szMod = "";
  if(  (aList[iPos].dir % 2) != 0  )
  {
    szMod = "|";
  }

  var  szURLs = "";
  if((  aList[iPos].home.length != 0  )||(  aList[iPos].down.length != 0  ))
  {
    szURLs = aList[iPos].home + "|"
           + aList[iPos].down + "|";
  }

  return  szMod
        + aList[iPos].line  + "|" + szURLs
        + aList[iPos].url   + "|"
        + aList[iPos].start + "|"
        + aList[iPos].pre   + "|"
        + aList[iPos].post  + "|"
        + aList[iPos].version;
}


/****************************************************************************/
/*! \brief  Returns the line corresponding to an edited item
**
**  \param[in]  iPos  position in wfxVersions_dialogVars_itemsList
**
**  \return  line-string corresponding to that item
*/
function  wfxVersions_dialogOps_lineFromEdit(  iPos  )
{
  if((  wfxVersions_dialogVars_itemsList == null  )||(  iPos >= wfxVersions_dialogVars_itemsList.length  ))
  {
    return  wfxVersions_dialogOps_localeParams( "szEmpty" );
  }

  if(  wfxVersions_dialogVars_itemsList[iPos].status == 6  )
  {
    return  wfxVersions_dialogVars_itemsList[iPos].line;
  }

  var  szMod = "";
  if(  document.getElementById( "wfxVersions_dialogIDs_editForwards" ).selectedIndex != 0  )
  {
    szMod = "|";
  }

  var  szURLs = "";
  if((  document.getElementById( "wfxVersions_dialogIDs_editHome" ).value.length != 0  )||(  document.getElementById( "wfxVersions_dialogIDs_editDown" ).value.length != 0  ))
  {
    szURLs = document.getElementById(  "wfxVersions_dialogIDs_editHome"  ).value + "|"
           + document.getElementById(  "wfxVersions_dialogIDs_editDown"  ).value + "|";
  }

  return  szMod
        + document.getElementById(  "wfxVersions_dialogIDs_editLine"     ).value + "|" + szURLs
        + document.getElementById(  "wfxVersions_dialogIDs_editURL"      ).value + "|"
        + document.getElementById(  "wfxVersions_dialogIDs_editStart"    ).value + "|"
        + document.getElementById(  "wfxVersions_dialogIDs_editPre"      ).value + "|"
        + document.getElementById(  "wfxVersions_dialogIDs_editPost"     ).value + "|"
        + wfxVersions_dialogVars_itemsList[iPos].version;
}


/****************************************************************************/
/*! \brief  Determine if a string is a comment or a valid .list item
**
**  \param[in]  szLine  line string taken from a .list file
**
**  \return  true (comment) / false (valid)
*/
function  wfxVersions_dialogOps_isComment(  szLine  )
{
  var  szArray = szLine.split(  '|'  );
  var  eDir    = szArray.length - 6;
  if(  ((  eDir == 0  )&&(  szArray[0].length != 0  )&&(  szArray[1].length != 0  ))
    || ((  eDir == 1  )&&(  szArray[0].length == 0  )&&(  szArray[1].length != 0  )&&(  szArray[2].length != 0  ))
    || ((  eDir == 2  )&&(  szArray[0].length != 0  )&&(  szArray[3].length != 0  ))
    || ((  eDir == 3  )&&(  szArray[0].length == 0  )&&(  szArray[1].length != 0  )&&(  szArray[4].length != 0  ))  )
  {
    return  false;
  }
  return  true;
}


/****************************************************************************/
/*! \brief  Creates a new item for the global items list
**
**  An item will be added for each valid line from a .list file
**
**  \param[in]  szLine  line string taken from a .list file
**  \param[in]  szFile  the .list file name
**
**  \return  --
*/
function  wfxVersions_dialogOps_createItem(  szLine,  szFile  )
{
  var  szArray = szLine.split(  '|'  );

  /*  scan direction  */
  this.dir     = szArray.length - 6;

  /*  information for .list file updating  */
  this.list    = szFile;
  this.line    = szArray[ this.dir % 2 ];
  this.update  = "";

  /*  information about searching inside the HTTP Request page  */
  this.url     = szArray[ 1 + this.dir ];
  this.start   = szArray[ 2 + this.dir ];
  this.pre     = szArray[ 3 + this.dir ];
  this.post    = szArray[ 4 + this.dir ];
  this.version = szArray[ 5 + this.dir ];

  /*  item status = idle  */
  this.step    = 0;
  this.status  = 0;

  /*  additional URLs  */
  this.home = "";
  this.down = "";
  if(  this.dir > 1  )
  {
    this.home = szArray[ 1 + (this.dir % 2) ];
    this.down = szArray[ 2 + (this.dir % 2) ];
  }
}


/****************************************************************************/
/*! \brief  Creates a new dummy item for unrecognized .list lines
**
**  \see  wfxVersions_dialogOps_createItem()
**
**  \param[in]  szLine  line string taken from a .list file
**  \param[in]  szFile  the .list file name
**
**  \return  --
*/
function  wfxVersions_dialogOps_createNonItem(  szLine,  szFile  )
{
  /*  scan direction  */
  this.dir     = 0;

  /*  information for .list file updating  */
  this.list    = szFile;
  this.line    = szLine;

  /*  unused information about searching inside the HTTP Request page  */
  this.update  = "";
  this.url     = "";
  this.start   = "";
  this.pre     = "";
  this.post    = "";
  this.version = "";

  /*  item status = hidden  */
  this.step     = 0;
  this.status   = 6;

  /*  additional URLs  */
  this.home    = "";
  this.down    = "";
}


/****************************************************************************/
/*! \brief  Stop the throbber animation
**
**  An own throbber is used instead of the FF global one
**
**  \param  --
**
**  \return  --
*/
function  wfxVersions_dialogOps_throbberIdle()
{
  wfxVersions_overlayOps_changeThrobber(  false  );

  if((  wfxVersions_dialogVars_itemsList != null  )&&(  wfxVersions_dialogVars_progress == (wfxVersions_dialogVars_itemsList.length - 1)  ))
  {
    /*  cannot delete the empty "new entry"  */
    document.getElementById(  "wfxVersions_dialogIDs_editCopy"    ).setAttribute(  "disabled",  "true"  );
    document.getElementById(  "wfxVersions_dialogIDs_editDelete"  ).setAttribute(  "disabled",  "true"  );
  }
  else
  {
    document.getElementById(  "wfxVersions_dialogIDs_editCopy"    ).removeAttribute(  "disabled"  );
    document.getElementById(  "wfxVersions_dialogIDs_editDelete"  ).removeAttribute(  "disabled"  );
  }
  document.getElementById(  "wfxVersions_dialogIDs_editTest"    ).removeAttribute(  "disabled"  );
  document.getElementById(  "wfxVersions_dialogIDs_editInsert"  ).removeAttribute(  "disabled"  );

  wfxVersions_dialogOps_setImageProperty(   document.getElementById( "wfxVersions_dialogIDs_tabs" ).firstChild,  "chrome://wfx_versions/skin/wfx_versions_edit.png"  );
  wfxVersions_dialogOps_setImageAttribute(  document.getElementById( "wfxVersions_dialogIDs_buttonEdit" ),       "chrome://wfx_versions/skin/wfx_versions_edit.png"  );
}


/****************************************************************************/
/*! \brief  Starts the throbber animation
**
**  An own throbber is used instead of the FF global one
**
**  \param  --
**
**  \return  --
*/
function  wfxVersions_dialogOps_throbberScan()
{
  wfxVersions_dialogOps_setImageProperty(  document.getElementById( "wfxVersions_dialogIDs_tabs" ).firstChild,  "chrome://wfx_versions/skin/wfx_versions_throbber.gif"  );

  document.getElementById(  "wfxVersions_dialogIDs_editCopy"    ).setAttribute(  "disabled",  "true"  );
  document.getElementById(  "wfxVersions_dialogIDs_editTest"    ).setAttribute(  "disabled",  "true"  );
  document.getElementById(  "wfxVersions_dialogIDs_editDelete"  ).setAttribute(  "disabled",  "true"  );
  document.getElementById(  "wfxVersions_dialogIDs_editInsert"  ).setAttribute(  "disabled",  "true"  );

  wfxVersions_overlayOps_changeThrobber(  true  );
}


/****************************************************************************/
/*! \brief  Changes the display corresponding to modified status
**
**  .status
**  - 0: idle, ready for scan
**  - 1: idle, no scan (edit?)
**  - 2: scanning
**  - 3: tab closed
**  - 4: scan complete, error
**  - 5: scan complete, ok
**  - 6: non-item, disabled/hidden
**
**  \param[in]  iItem    the list index of the item being scanned
**  \param[in]  iStatus  the changed status value
**
**  \return  --
*/
function  wfxVersions_dialogOps_scanStatus(  iItem,  iStatus  )
{
  var  szHideProgress = "true";
  var  szModeProgress = "determined";

  var  szHideReload   = "true";
  var  szHideStop     = "true";

  var  szHideText     = "true";
  var  szHideError    = "false";

  var  szHideEdit     = "false";
  var  szHideHelp     = "hidden";
  var  szEditError    = "true";

  switch(  iStatus  )
  {
    default:  /*  idle-scan  */
      szHideProgress = "false";
      szHideStop     = "false";
      szHideError    = "true";
    break;

    case  1:  /*  idle-edit  */
    break;

    case  2:  /*  scanning  */
      szHideProgress = "false";
      szModeProgress = "undetermined";
      szHideStop     = "false";
      szHideError    = "true";
      szHideEdit     = "true";
    break;

    case  3:  /*  tab-closed  */
    break;

    case  4:  /*  done-error  */
      szHideReload = "false";
      szHideHelp   = "visible";
    break;

    case  5:  /*  done-ok  */
      szHideText     = "false";
      szHideError    = "true";
      szHideReload   = "false";
      szEditError    = "false";
    break;

    case  6:  /*  tab-closed  */
    break;
  }
  wfxVersions_dialogVars_itemsList[iItem].status = iStatus;

  if(  document.getElementById( "wfxVersions_dialogIDs_tabs" ).childNodes.length > 2  )
  {
    document.getElementById(  "m" + iItem  ).setAttribute(  "hidden",  szHideProgress   );
    document.getElementById(  "m" + iItem  ).setAttribute(  "mode",    szModeProgress   );
    document.getElementById(  "m" + iItem  ).setAttribute(  "value",   "0"  );

    wfxVersions_dialogOps_hideAndFocus(  "x" + iItem,  szHideText  );
    wfxVersions_dialogOps_hideAndFocus(  "u" + iItem,  szHideError  );
    wfxVersions_dialogOps_hideAndFocus(  "r" + iItem,  szHideReload  );
    wfxVersions_dialogOps_hideAndFocus(  "o" + iItem,  szHideStop  );

    if(  iStatus != 4  )
    {
      wfxVersions_dialogOps_hideAndFocus(  "h" + iItem,  "true"  );
    }
  }
  else
  {
    document.getElementById(  "wfxVersions_dialogIDs_editProgress"  ).setAttribute(  "hidden",  szHideProgress   );
    document.getElementById(  "wfxVersions_dialogIDs_editProgress"  ).setAttribute(  "mode",    szModeProgress   );
    document.getElementById(  "wfxVersions_dialogIDs_editProgress"  ).setAttribute(  "value",   "0"  );

    document.getElementById(  "wfxVersions_dialogIDs_editVersion"  ).setAttribute(  "wfxIsError",  szEditError  );
    document.getElementById(  "wfxVersions_dialogIDs_editVersion"  ).setAttribute(  "hidden",  szHideEdit  );
    document.getElementById(  "wfxVersions_dialogIDs_editHelp"     ).style.setProperty(  "visibility",  szHideHelp,  "important"  );
  }
}


/****************************************************************************/
/*! \brief  Hide or reveal an element, with changing focus to different one
**
**  When an element that has focus is hidden, we want to change the focus to
**  another element so that the keyboard-tab order is maintained
**
**  \param[in]  szItem  the ID of the element being revealed/hidden
**  \param[in]  szHide  the hide (true) / reveal (false) attribute string
**
**  \return  --
*/
function  wfxVersions_dialogOps_hideAndFocus(  szItem,  szHide  )
{
  var  oBlur = document.getElementById(  szItem  );
  oBlur.setAttribute(  "hidden",  szHide  );

  /*  on-hide we must change the focus to a new element  */
  if((  szHide == "true"  )&&(  document.commandDispatcher.focusedElement == oBlur  ))
  {
    var  szFocus = oBlur.getAttribute(  "wfxVersions_customFocus"  );
    if((  szFocus  )&&(  document.getElementById( szFocus )  ))
    {
      wfxVersions_globalOps_setTimeout(  "wfxVersions_dialogOps_delayedFocus('" + szFocus + "')",  40  );
    }
  }
}


/****************************************************************************/
/*! \brief  Delayed focus event generation
**
**  The delay is required such that hide/reveal of previous elements is
**  completed and focus will be successful
**
**  \param[in]  szItem  the ID of the element being focused
**
**  \return  --
*/
function  wfxVersions_dialogOps_delayedFocus(  szItem  )
{
  if(  document.getElementById( szItem ).getAttribute( "hidden" ) != "true"  )
  {
    document.getElementById(  szItem  ).focus();
  }
}


/****************************************************************************/
/*! \brief  Called when the user clicks the small-scan button
**
**  \param[in]  iItem  the scanned item being resumed
**
**  \return  --
*/
function  wfxVersions_scanOps_itemResume(  iItem  )
{
  wfxVersions_dialogOps_updateScanLinks(  iItem,  wfxVersions_dialogVars_itemsList[iItem].version  );

  wfxVersions_dialogOps_scanStatus(  iItem,  0/*idle*/  );
  wfxVersions_dialogOps_throbberTabs();

  if(  wfxVersions_dialogVars_progress >= wfxVersions_dialogVars_itemsList.length  )
  {
    /*  rescan clicked after all scans complete  */
    wfxVersions_dialogVars_itemsList[iItem].step = 0;
    wfxVersions_dialogOps_throbberScan();
    wfxVersions_dialogOps_setImageAttribute(  document.getElementById( "wfxVersions_dialogIDs_buttonScan" ),  "chrome://wfx_versions/skin/wfx_versions_throbber.gif"  );
    wfxVersions_globalOps_setTimeout(  "wfxVersions_httpOps_headLaunch()",  40  );
  }
}


/****************************************************************************/
/*! \brief  Called when the user clicks the small-abort button
**
**  \param[in]  iItem  the scanned item being stopped
**
**  \return  --
*/
function  wfxVersions_scanOps_itemAbort(  iItem  )
{
  wfxVersions_dialogOps_updateScanLinks(  iItem,  wfxVersions_dialogVars_itemsList[iItem].version  );

  if(  iItem == wfxVersions_dialogVars_progress  )
  {
    /*  abort only the entry clicked by user not all entries with same URL  */
    var  szURL = wfxVersions_dialogOps_likeMRT(  wfxVersions_dialogVars_itemsList[iItem].url,  wfxVersions_dialogVars_itemsList[iItem].version  );
    for(  var  i = 0;  i < wfxVersions_dialogVars_itemsList.length;  i++  )
    {
      var  szLink = wfxVersions_dialogOps_likeMRT(  wfxVersions_dialogVars_itemsList[i].url,  wfxVersions_dialogVars_itemsList[i].version  );
      if((  iItem != i  )&&(  szLink == szURL  )&&(  wfxVersions_dialogVars_itemsList[i].status == 2  ))
      {
        wfxVersions_dialogVars_progress = i;
        break;
      }
    }
  }

  if(  wfxVersions_dialogVars_progress == iItem  )
  {
    wfxVersions_dialogOps_notifyOnError(  wfxVersions_dialogOps_localeParams( "szSoloAbort" ),  null  );
  }
  else
  {
    document.getElementById( "u" + iItem ).value = wfxVersions_dialogOps_localeParams(  "szSoloAbort"  );
    wfxVersions_dialogOps_scanStatus(  iItem,  4  );
  }
  wfxVersions_dialogOps_throbberTabs();
}


/****************************************************************************/
/*! \brief  Notify error on all equal URLs
**
**  \param[in]  szError   the message to be displayed
**  \param[in]  szDetail  error details for [?] button (disabled if null)
**
**  \return  --
*/
function  wfxVersions_dialogOps_notifyOnError(  szError,  szDetail  )
{
  if(  wfxVersions_dialogVars_httpRequest != null  )
  {
    wfxVersions_httpOps_ghost();
  }

  if(  document.getElementById( "wfxVersions_dialogIDs_tabs" ).childNodes.length > 2  )
  {
    /*  notify error on all equal URLs  */
    var  szURL = wfxVersions_dialogVars_itemsList[wfxVersions_dialogVars_progress].url;
    var  i;
    for(  i = 0;  i < wfxVersions_dialogVars_itemsList.length;  i++  )
    {
      if((  wfxVersions_dialogVars_itemsList[i].url == szURL  )
       &&(  wfxVersions_dialogVars_itemsList[i].status == 2  ))
      {
        wfxVersions_dialogOps_scanStatus(  i,  4  );
        document.getElementById(  "u" + i  ).value = szError;
        if((  szDetail  )&&(  szDetail.length  ))
        {
          document.getElementById(  "h" + i  ).removeAttribute(  "hidden"  );
          document.getElementById(  "h" + i  ).setAttribute(  "oncommand",  "wfxVersions_globalOps_alert('" + szError + "','" + szDetail + "')"  );
        }
        else
        {
          wfxVersions_dialogOps_hideAndFocus(  "h" + i,  "true"  );
        }
      }
    }
    wfxVersions_dialogOps_throbberTabs();

    /*  avoid recursivity by delayed call  */
    wfxVersions_globalOps_setTimeout(  "wfxVersions_httpOps_headLaunch()",  40  );
  }
  else
  {
    wfxVersions_dialogOps_scanStatus(  wfxVersions_dialogVars_progress,  4  );
    wfxVersions_dialogOps_throbberIdle();
    wfxVersions_dialogOps_setImageAttribute(  document.getElementById( "wfxVersions_dialogIDs_buttonEdit" ),  "chrome://wfx_versions/skin/wfx_versions_edit.gif"  );

    document.getElementById(  "wfxVersions_dialogIDs_editVersion"  ).value = szError;
    document.getElementById(  "wfxVersions_dialogIDs_editVersion"  ).setAttribute(  "wfxIsError",  "true"  );
    document.getElementById(  "wfxVersions_dialogIDs_editHelp"  ).setAttribute(  "tooltiptext",  wfxVersions_dialogOps_localeParams( "szSoloHelp" )  );
    document.getElementById(  "wfxVersions_dialogIDs_editHelp"  ).setAttribute(  "oncommand",    "wfxVersions_globalOps_alert('" + szError + "','" + szDetail + "')"  );
  }
}


/****************************************************************************/
/*! \brief  Scan for current item in http-loaded page
**
**  \param[in]  szPage    the page content
**  \param[in]  szStart   page-source point where to start the scan
**  \param[in]  szPre     page-source content exactly before version-string
**  \param[in]  szPost    page-source content exactly after version-string
**  \param[in]  iForward  scan forwards (0) or backwards (1) in the page
**
**  \return  encapsulation of the scan-status and the scan-result
**           - .status = 0 (ok), 1 (error_@start), 2 (@pre), 3 (@post)
**           - .result = the version-string if all ok, null in case of error
*/
function  wfxVersions_scanOps_itemSearch(  szPage,  szStart,  szPre,  szPost,  iForward  )
{
  var  szPageLower = szPage.toLowerCase();
  var  iStart,  iPre,  iPost;

  if(  iForward == 0  )
  {
    /*  scan forwards  */
    iStart = 0;
    iPre   = 0;
    iPost  = szPageLower.length - szPost.length;

    if(  szStart.length != 0  )
    {
      iStart = szPageLower.indexOf(  szStart.toLowerCase()  );
    }
    iPre = iStart + szStart.length;
    if(  szPre.length != 0  )
    {
      iPre = szPageLower.indexOf(  szPre.toLowerCase(),  iPre  );
    }
    iPost = iPre + szPre.length;
    if(  szPost.length != 0  )
    {
      iPost = szPageLower.indexOf(  szPost.toLowerCase(),  iPost  );
    }
  }
  else
  {
    /*  scan backwards  */
    iStart = szPageLower.length - szStart.length;
    iPre   = 0;
    iPost  = szPageLower.length - szPost.length;

    if(  szStart.length != 0  )
    {
      iStart = szPageLower.lastIndexOf(  szStart.toLowerCase()  );
    }
    iPost = iStart - szPost.length;
    if(  szPost.length != 0  )
    {
      iPost = szPageLower.lastIndexOf(  szPost.toLowerCase(),  iPost  );
    }
    iPre = iPost - szPre.length;
    if(  szPre.length != 0  )
    {
      iPre = szPageLower.lastIndexOf(  szPre.toLowerCase(),  iPre  );
    }
  }

  var  oRetVal = {  status: 0,  result: null  };
  if(  iStart < 0  )
  {
    oRetVal.status = 1;
  }
  else
  if(  iPre < 0  )
  {
    oRetVal.status = 2;
  }
  else
  if(  iPost < 0  )
  {
    oRetVal.status = 3;
  }
  else
  {
    oRetVal.result = szPage.substring(  iPre + szPre.length,  iPost  );
  }

  return  oRetVal;
}


/****************************************************************************/
/*! \brief  Called when the user clicks a different button than [Ok] / [Apply]
**
**  The [Edit] and [Scan] operations need to start with valid TC plugin path
**
**  \param  --
**
**  \return  --
*/
function  wfxVersions_dialogOps_loadPrefs()
{
  var  oPrefs = Components.classes[ "@mozilla.org/preferences-service;1" ]
                          .getService(  Components.interfaces.nsIPrefBranch  );

  var  szOld = oPrefs.getComplexValue(  "extensions.wfx_Versions.tcPluginPath",
                                        Components.interfaces.nsISupportsString  );
  document.getElementById(  "wfxVersions_dialogIDs_pathText"  ).value = szOld.data;
  wfxVersions_dialogOps_loadPathsCombo(  2  );

  var  iOld = oPrefs.getIntPref(  "extensions.wfx_Versions.httpReqTimeout"  );
  document.getElementById(  "wfxVersions_dialogIDs_delayText"  ).value = iOld;

  iOld = oPrefs.getIntPref(  "extensions.wfx_Versions.start.statusbar"  );
  document.getElementById(  "wfxVersions_dialogIDs_startStatusBar"  ).selectedIndex = iOld;
  iOld = oPrefs.getIntPref(  "extensions.wfx_Versions.start.toolbar"  );
  document.getElementById(  "wfxVersions_dialogIDs_startToolsBar"  ).selectedIndex = iOld;
  iOld = oPrefs.getIntPref(  "extensions.wfx_Versions.start.toolsmenuitem"  );
  document.getElementById(  "wfxVersions_dialogIDs_startToolsMenu"  ).selectedIndex = iOld;

  document.getElementById(  "wfxVersions_dialogIDs_showStatusBar"  ).checked = oPrefs.getBoolPref(  "extensions.wfx_Versions.show.statusbar"  );
  document.getElementById(  "wfxVersions_dialogIDs_showToolsBar"   ).checked = oPrefs.getBoolPref(  "extensions.wfx_Versions.show.toolbar"  );
  document.getElementById(  "wfxVersions_dialogIDs_showToolsMenu"  ).checked = oPrefs.getBoolPref(  "extensions.wfx_Versions.show.toolsmenuitem"  );
  document.getElementById(  "wfxVersions_dialogIDs_allScan"        ).checked = oPrefs.getBoolPref(  "extensions.wfx_Versions.selections.all"  );

  wfxVersions_globalOps_loadConfigColors();
  document.getElementById(  "wfxVersions_dialogIDs_bgColorDark"     ).color = wfxVersions_globalVars_bgColorDark;
  document.getElementById(  "wfxVersions_dialogIDs_bgColorLight"    ).color = wfxVersions_globalVars_bgColorLight;
  document.getElementById(  "wfxVersions_dialogIDs_linkColorText"   ).color = wfxVersions_globalVars_linkColorText;
  document.getElementById(  "wfxVersions_dialogIDs_linkColorHover"  ).color = wfxVersions_globalVars_linkColorHover;
  document.getElementById(  "wfxVersions_dialogIDs_fgColorOk"       ).color = wfxVersions_globalVars_fgColorOk;
  document.getElementById(  "wfxVersions_dialogIDs_fgColorError"    ).color = wfxVersions_globalVars_fgColorError;
  document.getElementById(  "wfxVersions_dialogIDs_editColorText"   ).color = wfxVersions_globalVars_editColorText;
  document.getElementById(  "wfxVersions_dialogIDs_editColorHover"  ).color = wfxVersions_globalVars_editColorHover;
}


/****************************************************************************/
/*! \brief  Called when the user changes the preferences
**
**  If the TC plugin path changes, the previous scan results need to be closed
**
**  \param  --
**
**  \return  --
*/
function  wfxVersions_dialogOps_storePrefs()
{
  var  oPrefs = Components.classes[ "@mozilla.org/preferences-service;1" ]
                          .getService(  Components.interfaces.nsIPrefBranch  );

  var  iOld = oPrefs.getIntPref(  "extensions.wfx_Versions.httpReqTimeout"  );
  var  szOld = oPrefs.getComplexValue(  "extensions.wfx_Versions.tcPluginPath",
                                        Components.interfaces.nsISupportsString  );

  var  iNew = parseInt(  document.getElementById( "wfxVersions_dialogIDs_delayText" ).value  );
  var  szNew = Components.classes[ "@mozilla.org/supports-string;1" ]
                         .createInstance(  Components.interfaces.nsISupportsString  );
  szNew.data = document.getElementById(  "wfxVersions_dialogIDs_pathText"  ).value;

  oPrefs.setIntPref(  "extensions.wfx_Versions.httpReqTimeout",
                      iNew  );
  oPrefs.setComplexValue(  "extensions.wfx_Versions.tcPluginPath",
                           Components.interfaces.nsISupportsString,
                           szNew  );

  wfxVersions_dialogOps_loadPathsCombo(  2  );
  document.getElementById(  "wfxVersions_dialogIDs_buttonScan"  ).removeAttribute(  "disabled"  );
  document.getElementById(  "wfxVersions_dialogIDs_buttonEdit"  ).removeAttribute(  "disabled"  );

  if(  szNew.data != szOld.data  )
  {
    /*  the Timeout config can be changed while scanning
    **  but the .list files path can't
    */
    wfxVersions_dialogOps_destroyTabs();
  }

  oPrefs.setCharPref(  "extensions.wfx_Versions.background.main",
                       document.getElementById( "wfxVersions_dialogIDs_bgColorDark" ).color  );
  oPrefs.setCharPref(  "extensions.wfx_Versions.background.alternated",
                       document.getElementById( "wfxVersions_dialogIDs_bgColorLight" ).color  );
  oPrefs.setCharPref(  "extensions.wfx_Versions.link.text.main",
                       document.getElementById( "wfxVersions_dialogIDs_linkColorText"  ).color  );
  oPrefs.setCharPref(  "extensions.wfx_Versions.link.hover.main",
                       document.getElementById( "wfxVersions_dialogIDs_linkColorHover" ).color  );
  oPrefs.setCharPref(  "extensions.wfx_Versions.scan.text.idle",
                       document.getElementById( "wfxVersions_dialogIDs_fgColorOk" ).color  );
  oPrefs.setCharPref(  "extensions.wfx_Versions.scan.text.error",
                       document.getElementById( "wfxVersions_dialogIDs_fgColorError"   ).color  );
  oPrefs.setCharPref(  "extensions.wfx_Versions.link.text.edited",
                       document.getElementById( "wfxVersions_dialogIDs_editColorText" ).color  );
  oPrefs.setCharPref(  "extensions.wfx_Versions.link.hover.edited",
                       document.getElementById( "wfxVersions_dialogIDs_editColorHover" ).color  );

  oPrefs.setIntPref(  "extensions.wfx_Versions.start.statusbar",
                      document.getElementById( "wfxVersions_dialogIDs_startStatusBar" ).selectedIndex  );
  oPrefs.setIntPref(  "extensions.wfx_Versions.start.toolbar",
                      document.getElementById( "wfxVersions_dialogIDs_startToolsBar" ).selectedIndex  );
  oPrefs.setIntPref(  "extensions.wfx_Versions.start.toolsmenuitem",
                      document.getElementById( "wfxVersions_dialogIDs_startToolsMenu" ).selectedIndex  );

  oPrefs.setBoolPref(  "extensions.wfx_Versions.selections.all",
                       document.getElementById( "wfxVersions_dialogIDs_allScan" ).checked  );

  oPrefs.setBoolPref(  "extensions.wfx_Versions.show.toolsmenuitem",
                       document.getElementById( "wfxVersions_dialogIDs_showToolsMenu" ).checked  );
  oPrefs.setBoolPref(  "extensions.wfx_Versions.show.toolbar",
                       document.getElementById( "wfxVersions_dialogIDs_showToolsBar" ).checked  );
  oPrefs.setBoolPref(  "extensions.wfx_Versions.show.statusbar",
                       document.getElementById( "wfxVersions_dialogIDs_showStatusBar" ).checked  );

  oPrefs.setBoolPref(  "extensions.wfx_Versions.expandedOps.path",
                       document.getElementById(  "wfxVersions_dialogIDs_expandPath"  ).parentNode.parentNode.hasAttribute( "hidden" )  );
  oPrefs.setBoolPref(  "extensions.wfx_Versions.expandedOps.timeout",
                       document.getElementById(  "wfxVersions_dialogIDs_expandTimeout"  ).parentNode.parentNode.hasAttribute( "hidden" )  );
  oPrefs.setBoolPref(  "extensions.wfx_Versions.expandedOps.display",
                       document.getElementById(  "wfxVersions_dialogIDs_expandShows"  ).parentNode.parentNode.hasAttribute( "hidden" )  );
  oPrefs.setBoolPref(  "extensions.wfx_Versions.expandedOps.colors",
                       document.getElementById(  "wfxVersions_dialogIDs_expandColors"  ).parentNode.parentNode.hasAttribute( "hidden" )  );
  oPrefs.setBoolPref(  "extensions.wfx_Versions.expandedOps.memorize",
                       document.getElementById(  "wfxVersions_dialogIDs_expandMemos"  ).parentNode.parentNode.hasAttribute( "hidden" )  );

  wfxVersions_globalOps_loadConfigColors();
  wfxVersions_globalOps_redoVisibilityPlaces();

  /*  re-colorize dialogs  */
  if((  wfxVersions_dialogVars_aboutBoxWindow != null  )&&(  wfxVersions_dialogVars_aboutBoxWindow.document != null  ))
  {
    wfxVersions_dialogVars_aboutBoxWindow.wfxVersions_aboutOps_reColorize();
  }

  /*  modify style according to configuration  */
  wfxVersions_globalOps_styleConfig();
}


/****************************************************************************/
/*! \brief  Moves the currently edited item up or down in the .list
**
**  HTTP Request for [Test] in progress is not interrupted
**
**  \param[in]  szTip  button tooltip (used for error messages)
**  \param[in]  iDir   move direction up (-1) / down (+1)
**  \param[in]  bAks   disable confirmation for new-style insert-new-entry
**
**  \return  --
*/
function  wfxVersions_editOps_moveEntry(  szTip,  iDir,  bAsk  )
{
  wfxVersions_editOps_clickEntry(  wfxVersions_dialogVars_progress  );

  var  bConfirmEnabled = Components.classes[ "@mozilla.org/preferences-service;1" ]
                                   .getService(  Components.interfaces.nsIPrefBranch  )
                                   .getBoolPref(  "extensions.wfx_Versions.confirm.edit.move"  );

  var  szConfirm = wfxVersions_dialogOps_localeParams(  "szMove"  );
  if((  szConfirm.length != 0  )&&(  bConfirmEnabled  )&&(  bAsk  ))
  {
    if(  !wfxVersions_globalOps_confirm( szTip, szConfirm )  )
    {
      return;
    }
  }

  /*  create backup  */
  var  i,  jDir = +1,  aItems = [];
  for(  i = 0;  i < wfxVersions_dialogVars_itemsList.length;  i++  )
  {
    aItems.splice(  0,  0,  wfxVersions_dialogVars_itemsList[i]  );
  }

  if(  iDir < 0  )
  {
    jDir = -1;
  }

  while(  iDir != 0  )
  {
    var  iItem,  eItem = wfxVersions_dialogVars_itemsList[wfxVersions_dialogVars_progress].editrow;

    var  oldNode = document.getElementById(  "wfxVersions_dialogIDs_editItems"  ).childNodes[ eItem        ].firstChild;
    var  newNode = document.getElementById(  "wfxVersions_dialogIDs_editItems"  ).childNodes[ eItem + jDir ].firstChild;

    var  iOld = parseInt(  oldNode.getAttribute( "onclick" ).split( "(" )[1]  );
    var  iNew = parseInt(  newNode.getAttribute( "onclick" ).split( "(" )[1]  );

    /*  now make the replacements
    **
    **  move up (jDir > 0):
    **    xOld [...] xNew
    **      =>
    **    xNew xOld [...]
    **
    **  move down (jDir < 0):
    **    xNew [...] xOld
    **      =>
    **    xOld xNew [...]
    */
    var  xOld = wfxVersions_dialogVars_itemsList[iOld];
    var  xNew = wfxVersions_dialogVars_itemsList[iNew];

    if(  jDir > 0  )
    {
      wfxVersions_dialogVars_itemsList.splice(  iNew,  1  );
      wfxVersions_dialogVars_itemsList.splice(  iOld,  1,  xNew,  xOld  );
      iItem = iOld + 1;
    }
    else
    {
      wfxVersions_dialogVars_itemsList.splice(  iOld,  1  );
      wfxVersions_dialogVars_itemsList.splice(  iNew,  1,  xOld,  xNew  );
      iItem = iNew;
    }

    /*  attempt to save the file  */
    var  txt = "";
    for(  i = 0;  i < (wfxVersions_dialogVars_itemsList.length - 1)/*skip new-line entry*/;  i++  )
    {
      if(  txt.length != 0  )
      {
        txt += "\r\n";
      }
      txt += wfxVersions_dialogOps_lineFromItem(  i  );
    }

    if(  wfxVersions_fileOps_storeList( wfxVersions_dialogVars_itemsList[0].list, txt )  )
    {
      txt = oldNode.value;
      oldNode.value = newNode.value;
      newNode.value = txt;

      txt = oldNode.getAttribute(  "tooltiptext"  );
      oldNode.setAttribute(  "tooltiptext",  newNode.getAttribute( "tooltiptext" )  );
      newNode.setAttribute(  "tooltiptext",  txt  );

      txt = oldNode.getAttribute(  "wfxIsError"  );
      oldNode.setAttribute(  "wfxIsError",  newNode.getAttribute( "wfxIsError" )  );
      newNode.setAttribute(  "wfxIsError",  txt  );

      oldNode.removeAttribute(  "wfxIsEdit"  );
      newNode.removeAttribute(  "wfxIsEdit"  );

      var  jItem = iItem - jDir;
      oldNode.setAttribute(  "onclick",  "wfxVersions_editOps_clickEntry(" + jItem + ")"  );
      newNode.setAttribute(  "onclick",  "wfxVersions_editOps_clickEntry(" + iItem + ")"  );

      wfxVersions_dialogVars_itemsList[iItem-jDir].editrow = eItem;
      wfxVersions_dialogVars_itemsList[iItem     ].editrow = eItem + jDir;

      /*  after move, select the new entry position  */
      wfxVersions_dialogVars_progress = iItem;
    }
    else
    {
      /*  restore from backup  */
      while(  wfxVersions_dialogVars_itemsList.length  )
      {
        wfxVersions_dialogVars_itemsList.pop();
      }
      while(  aItems.length  )
      {
        wfxVersions_dialogVars_itemsList.push(  aItems.pop()  );
      }
      iDir = jDir;
    }

    iDir -= jDir;
  }

  wfxVersions_editOps_clickEntry(  wfxVersions_dialogVars_progress  );
}


/****************************************************************************/
/*! \brief  Copies the current .list line into the edit area
**
**  \param[in]  iPos  items-list position / displayed-list position
**
**  If \e iPos is larger than \e wfxVersions_dialogVars_itemsList.length,
**  the click is on the `empty entry`
**
**  \return  --
*/
function  wfxVersions_editOps_clickEntry(  iPos  )
{
  if(  wfxVersions_dialogVars_httpRequest != null  )
  {
    wfxVersions_httpOps_ghost();
  }

  var  eOld = wfxVersions_dialogVars_itemsList[wfxVersions_dialogVars_progress].editrow;
  var  eNew = wfxVersions_dialogVars_itemsList[iPos].editrow;
  var  eMax = document.getElementById(  "wfxVersions_dialogIDs_editItems"  ).childNodes.length - 1;

  document.getElementById(  "wfxVersions_dialogIDs_editMoveUp"  ).removeAttribute(  "disabled"  );
  if((  eNew <= 0  )||(  eNew >= eMax  ))
  {
    document.getElementById(  "wfxVersions_dialogIDs_editMoveUp"  ).setAttribute(  "disabled",  "true"  );
  }

  document.getElementById(  "wfxVersions_dialogIDs_editMoveDown"  ).removeAttribute(  "disabled"  );
  if(  eNew >= (eMax - 1)  )
  {
    document.getElementById(  "wfxVersions_dialogIDs_editMoveDown"  ).setAttribute(  "disabled",  "true"  );
  }

  /*  un-select the old edit item  */
  var  oNode = document.getElementById(  "wfxVersions_dialogIDs_editItems"  ).childNodes[ eOld ].firstChild;
  oNode.removeAttribute(  "wfxIsEdit"  );
  oNode.removeAttribute(  "wfxIsError"  );
  var  szURL = wfxVersions_dialogOps_likeMRT(  wfxVersions_dialogVars_itemsList[wfxVersions_dialogVars_progress].url,  wfxVersions_dialogVars_itemsList[wfxVersions_dialogVars_progress].version  );
  if(  !wfxVersions_httpOps_isValidURL( szURL )  )
  {
    oNode.setAttribute(  "wfxIsError",  "true"  );
  }
  if(  wfxVersions_dialogVars_itemsList[wfxVersions_dialogVars_progress].status != 6  )
  {
    wfxVersions_dialogVars_itemsList[wfxVersions_dialogVars_progress].status = 1;
  }

  /*  select the new edit item  */
  wfxVersions_dialogVars_progress = iPos;
  oNode = document.getElementById(  "wfxVersions_dialogIDs_editItems"  ).childNodes[ eNew ].firstChild;
  oNode.setAttribute(  "wfxIsEdit",  "true"  );
  oNode.removeAttribute(  "wfxIsError"  );
  var  szURL = wfxVersions_dialogOps_likeMRT(  wfxVersions_dialogVars_itemsList[wfxVersions_dialogVars_progress].url,  wfxVersions_dialogVars_itemsList[wfxVersions_dialogVars_progress].version  );
  if(  !wfxVersions_httpOps_isValidURL( szURL )  )
  {
    oNode.setAttribute(  "wfxIsError",  "true"  );
  }
  try{  oNode.focus();  }catch(e){}

  wfxVersions_dialogOps_throbberIdle();
  document.getElementById(  "wfxVersions_dialogIDs_editVersion"  ).removeAttribute(  "wfxIsError"  );

  if(  wfxVersions_dialogVars_itemsList[wfxVersions_dialogVars_progress].status == 6  )
  {
    document.getElementById(  "wfxVersions_dialogIDs_editLine"     ).value = wfxVersions_dialogVars_itemsList[wfxVersions_dialogVars_progress].line;
    document.getElementById(  "wfxVersions_dialogIDs_editHome"     ).value = "";
    document.getElementById(  "wfxVersions_dialogIDs_editDown"     ).value = "";
    document.getElementById(  "wfxVersions_dialogIDs_editURL"      ).value = "";
    document.getElementById(  "wfxVersions_dialogIDs_editStart"    ).value = "";
    document.getElementById(  "wfxVersions_dialogIDs_editPre"      ).value = "";
    document.getElementById(  "wfxVersions_dialogIDs_editPost"     ).value = "";
    document.getElementById(  "wfxVersions_dialogIDs_editVersion"  ).value = "";

    document.getElementById(  "wfxVersions_dialogIDs_editHome"      ).setAttribute(  "disabled",  "true"  );
    document.getElementById(  "wfxVersions_dialogIDs_editDown"      ).setAttribute(  "disabled",  "true"  );
    document.getElementById(  "wfxVersions_dialogIDs_editURL"       ).setAttribute(  "disabled",  "true"  );
    document.getElementById(  "wfxVersions_dialogIDs_editStart"     ).setAttribute(  "disabled",  "true"  );
    document.getElementById(  "wfxVersions_dialogIDs_editPre"       ).setAttribute(  "disabled",  "true"  );
    document.getElementById(  "wfxVersions_dialogIDs_editPost"      ).setAttribute(  "disabled",  "true"  );
    document.getElementById(  "wfxVersions_dialogIDs_editVersion"   ).setAttribute(  "disabled",  "true"  );
    document.getElementById(  "wfxVersions_dialogIDs_editForwards"  ).setAttribute(  "disabled",  "true"  );
    document.getElementById(  "wfxVersions_dialogIDs_editTest"      ).setAttribute(  "disabled",  "true"  );

    document.getElementById(  "wfxVersions_dialogIDs_editForwards"  ).selectedIndex = 0;
    document.getElementById(  "wfxVersions_dialogIDs_editForwards"  ).firstChild.setAttribute(  "disabled",  "true"  );
    document.getElementById(  "wfxVersions_dialogIDs_editForwards"  ). lastChild.setAttribute(  "disabled",  "true"  );

    document.getElementById(  "wfxVersions_dialogIDs_editClickHome"  ).setAttribute(  "tooltiptext",  "about:blank"  );
    document.getElementById(  "wfxVersions_dialogIDs_editClickDown"  ).setAttribute(  "tooltiptext",  "about:blank"  );
    document.getElementById(  "wfxVersions_dialogIDs_editClickURL"   ).setAttribute(  "tooltiptext",  "about:blank"  );
  }
  else
  {
    wfxVersions_dialogVars_itemsList[wfxVersions_dialogVars_progress].status = 0;
    wfxVersions_dialogVars_itemsList[wfxVersions_dialogVars_progress].update = wfxVersions_dialogVars_itemsList[wfxVersions_dialogVars_progress].version;

    document.getElementById(  "wfxVersions_dialogIDs_editLine"     ).value =                                     wfxVersions_dialogVars_itemsList[wfxVersions_dialogVars_progress].line;
    document.getElementById(  "wfxVersions_dialogIDs_editHome"     ).value =                                     wfxVersions_dialogVars_itemsList[wfxVersions_dialogVars_progress].home;
    document.getElementById(  "wfxVersions_dialogIDs_editDown"     ).value =                                     wfxVersions_dialogVars_itemsList[wfxVersions_dialogVars_progress].down;
    document.getElementById(  "wfxVersions_dialogIDs_editURL"      ).value =                                     wfxVersions_dialogVars_itemsList[wfxVersions_dialogVars_progress].url;
    document.getElementById(  "wfxVersions_dialogIDs_editStart"    ).value =                                     wfxVersions_dialogVars_itemsList[wfxVersions_dialogVars_progress].start;
    document.getElementById(  "wfxVersions_dialogIDs_editPre"      ).value =                                     wfxVersions_dialogVars_itemsList[wfxVersions_dialogVars_progress].pre;
    document.getElementById(  "wfxVersions_dialogIDs_editPost"     ).value =                                     wfxVersions_dialogVars_itemsList[wfxVersions_dialogVars_progress].post;
    document.getElementById(  "wfxVersions_dialogIDs_editVersion"  ).value = wfxVersions_globalOps_convertHTML(  wfxVersions_dialogVars_itemsList[wfxVersions_dialogVars_progress].update  );

    document.getElementById(  "wfxVersions_dialogIDs_editHome"      ).removeAttribute(  "disabled"  );
    document.getElementById(  "wfxVersions_dialogIDs_editDown"      ).removeAttribute(  "disabled"  );
    document.getElementById(  "wfxVersions_dialogIDs_editURL"       ).removeAttribute(  "disabled"  );
    document.getElementById(  "wfxVersions_dialogIDs_editStart"     ).removeAttribute(  "disabled"  );
    document.getElementById(  "wfxVersions_dialogIDs_editPre"       ).removeAttribute(  "disabled"  );
    document.getElementById(  "wfxVersions_dialogIDs_editPost"      ).removeAttribute(  "disabled"  );
    document.getElementById(  "wfxVersions_dialogIDs_editVersion"   ).removeAttribute(  "disabled"  );
    document.getElementById(  "wfxVersions_dialogIDs_editForwards"  ).removeAttribute(  "disabled"  );
    document.getElementById(  "wfxVersions_dialogIDs_editTest"      ).removeAttribute(  "disabled"  );

    document.getElementById(  "wfxVersions_dialogIDs_editForwards"  ).selectedIndex = wfxVersions_dialogVars_itemsList[wfxVersions_dialogVars_progress].dir % 2;
    document.getElementById(  "wfxVersions_dialogIDs_editForwards"  ).firstChild.removeAttribute(  "disabled"  );
    document.getElementById(  "wfxVersions_dialogIDs_editForwards"  ). lastChild.removeAttribute(  "disabled"  );

    document.getElementById(  "wfxVersions_dialogIDs_editClickHome"  ).setAttribute(  "tooltiptext",  wfxVersions_dialogOps_likeMRT( wfxVersions_dialogVars_itemsList[wfxVersions_dialogVars_progress].home, wfxVersions_dialogVars_itemsList[wfxVersions_dialogVars_progress].version )  );
    document.getElementById(  "wfxVersions_dialogIDs_editClickDown"  ).setAttribute(  "tooltiptext",  wfxVersions_dialogOps_likeMRT( wfxVersions_dialogVars_itemsList[wfxVersions_dialogVars_progress].down, wfxVersions_dialogVars_itemsList[wfxVersions_dialogVars_progress].version )  );
    document.getElementById(  "wfxVersions_dialogIDs_editClickURL"   ).setAttribute(  "tooltiptext",  wfxVersions_dialogOps_likeMRT( wfxVersions_dialogVars_itemsList[wfxVersions_dialogVars_progress].url,  wfxVersions_dialogVars_itemsList[wfxVersions_dialogVars_progress].version )  );
  }

  document.getElementById(  "wfxVersions_dialogIDs_editHelp"  ).style.setProperty(  "visibility",  "hidden",  "important"  );
  document.getElementById(  "wfxVersions_dialogIDs_editProgress"  ).setAttribute(  "hidden",  "true"  );
  document.getElementById(  "wfxVersions_dialogIDs_editVersion"  ).removeAttribute(  "hidden"  );
  wfxVersions_dialogOps_setImageAttribute(  document.getElementById( "wfxVersions_dialogIDs_buttonEdit" ),  "chrome://wfx_versions/skin/wfx_versions_edit.gif"  );

  /*  scroll the clicked entry into view  */
  wfxVersions_dialogOps_ensureVisible(  oNode.parentNode,  "wfxVersions_dialogIDs_editItems"  );
}


/****************************************************************************/
/*! \brief  Add a `new entry` field in the edit-area list
**
**  Called on Edit loading and after previous `new entry` has been modified
**
**  \param  --
**
**  \return  --
*/
function  wfxVersions_dialogOps_addEmpty()
{
  var  szList = document.getElementById(  "wfxVersions_dialogIDs_tabs"  ).firstChild.getAttribute(  "tooltiptext"  );
  var  szLine = wfxVersions_dialogOps_localeParams(  "szEmpty"  );

  /*  application name  */
  var  newLine = document.createElement(  "hbox"  );
  newLine.setAttribute(  "align",     "center"  );
  newLine.setAttribute(  "class",     "wfxVersions_skinBGnd"  );
  newLine.setAttribute(  "wfxIsOdd",  document.getElementById( "wfxVersions_dialogIDs_editItems" ).childNodes.length % 2  );
  document.getElementById(  "wfxVersions_dialogIDs_editItems"  ).appendChild(  newLine  );

  var  newNode = document.createElement(  "label"  );
  newNode.setAttribute(  "value",        szLine.split( '|' )[0]  );
  newNode.setAttribute(  "tooltiptext",  szLine  );
  newNode.setAttribute(  "onclick",      "wfxVersions_editOps_clickEntry(" + wfxVersions_dialogVars_itemsList.length + ")"  );
  newNode.setAttribute(  "onfocus",      "wfxVersions_dialogOps_ensureVisible(parentNode,'wfxVersions_dialogIDs_editItems')"  );
  newNode.setAttribute(  "class",        "wfxVersions_skinLink"  );
  newNode.style.setProperty(  "cursor",           "e-resize",          "important"  );
  newNode.style.setProperty(  "-moz-margin-end",  "16px",              "important"  );
  newNode.setAttribute(  "wfxIsError",   "true"  );
  newLine.appendChild(  newNode  );

  wfxVersions_dialogVars_itemsList.push(  new wfxVersions_dialogOps_createItem( szLine, szList )  );
  wfxVersions_dialogVars_itemsList[wfxVersions_dialogVars_itemsList.length - 1].editrow = document.getElementById( "wfxVersions_dialogIDs_editItems" ).childNodes.length - 1;

  /*  enlarge window to show entire content  */
  wfxVersions_dialogOps_resizeNow();
}


/****************************************************************************/
/*! \brief  Verify edited entries by scanning them individually
**
**  Set-up the Loading... stuff and go
**
**  \param  --
**
**  \return  --
*/
function  wfxVersions_editOps_testEntry()
{
  var  szAlert = "";
  if(  document.getElementById( "wfxVersions_dialogIDs_editLine" ).value.length == 0  )
  {
    szAlert += "\n" + wfxVersions_dialogOps_localeParams(  "szBlankLine"  );
  }
  if(  document.getElementById( "wfxVersions_dialogIDs_editURL" ).value.length == 0  )
  {
    szAlert += "\n" + wfxVersions_dialogOps_localeParams(  "szBlankURL"  );
  }
  if(  szAlert.length != 0  )
  {
    wfxVersions_globalOps_alert(  document.getElementById( "wfxVersions_dialogIDs_editTest" ).getAttribute( "tooltiptext" ),
                                  wfxVersions_dialogOps_localeParams( "szBlankEdit" ) + szAlert  );
    return;
  }

  szAlert = "";
  if(  document.getElementById( "wfxVersions_dialogIDs_editLine" ).value.indexOf( '|' ) >= 0  )
  {
    szAlert += "\n" + wfxVersions_dialogOps_localeParams(  "szPipeLine"  );
  }
  if(  document.getElementById( "wfxVersions_dialogIDs_editURL" ).value.indexOf( '|' ) >= 0  )
  {
    szAlert += "\n" + wfxVersions_dialogOps_localeParams(  "szPipeURL"  );
  }
  if(  document.getElementById( "wfxVersions_dialogIDs_editStart" ).value.indexOf( '|' ) >= 0  )
  {
    szAlert += "\n" + wfxVersions_dialogOps_localeParams(  "szPipeStart"  );
  }
  if(  document.getElementById( "wfxVersions_dialogIDs_editPre" ).value.indexOf( '|' ) >= 0  )
  {
    szAlert += "\n" + wfxVersions_dialogOps_localeParams(  "szPipePre"  );
  }
  if(  document.getElementById( "wfxVersions_dialogIDs_editPost" ).value.indexOf( '|' ) >= 0  )
  {
    szAlert += "\n" + wfxVersions_dialogOps_localeParams(  "szPipePost"  );
  }
  if(  szAlert.length != 0  )
  {
    wfxVersions_globalOps_alert(  document.getElementById( "wfxVersions_dialogIDs_editTest" ).getAttribute( "tooltiptext" ),
                                  wfxVersions_dialogOps_localeParams( "szReserved" ) + szAlert  );
    return;
  }

  wfxVersions_dialogOps_throbberScan();
  wfxVersions_dialogOps_setImageAttribute(  document.getElementById( "wfxVersions_dialogIDs_buttonEdit" ),  "chrome://wfx_versions/skin/wfx_versions_throbber.gif"  );
  wfxVersions_globalOps_setTimeout(  "wfxVersions_httpOps_headLaunch()",  40  );
}


/****************************************************************************/
/*! \brief  Remove a line from a .list file
**
**  \param  --
**
**  \return  --
*/
function  wfxVersions_editOps_deleteEntry()
{
  var  bConfirmEnabled = Components.classes[ "@mozilla.org/preferences-service;1" ]
                                   .getService(  Components.interfaces.nsIPrefBranch  )
                                   .getBoolPref(  "extensions.wfx_Versions.confirm.edit.delete"  );

  /*  ask the user for confirmation (0-length string = don't ask)  */
  var  szConfirm = wfxVersions_dialogOps_localeParams(  "szDelete"  );
  if((  szConfirm.length != 0  )&&(  bConfirmEnabled  ))
  {
    var  szTip = document.getElementById(  "wfxVersions_dialogIDs_editDelete"  ).getAttribute(  "tooltiptext"  );
    if(  !wfxVersions_globalOps_confirm( szTip, szConfirm )  )
    {
      return;
    }
  }

  var  i,  txt = "";
  for(  i = 0;  i < (wfxVersions_dialogVars_itemsList.length - 1)/*skip new-line entry*/;  i++  )
  {
    if(  i != wfxVersions_dialogVars_progress  )
    {
      if(  txt.length != 0  )
      {
        txt += "\r\n";
      }
      txt += wfxVersions_dialogOps_lineFromItem(  i  );
    }
  }

  if(  wfxVersions_fileOps_storeList( wfxVersions_dialogVars_itemsList[0].list, txt )  )
  {
    var  oldItem = wfxVersions_dialogVars_itemsList[wfxVersions_dialogVars_progress].editrow;
    var  oldNode = document.getElementById(  "wfxVersions_dialogIDs_editItems"  ).childNodes[ oldItem ];
    wfxVersions_globalOps_nodeCleanUp(  oldNode  );
    document.getElementById(  "wfxVersions_dialogIDs_editItems"  ).removeChild(  oldNode  );
    wfxVersions_dialogVars_itemsList.splice(  wfxVersions_dialogVars_progress,  1  );

    for(  i = wfxVersions_dialogVars_progress;  i < wfxVersions_dialogVars_itemsList.length;  i++  )
    {
      wfxVersions_dialogVars_itemsList[i].editrow--;
    }

    for(  i = oldItem;  i < document.getElementById(  "wfxVersions_dialogIDs_editItems"  ).childNodes.length;  i++  )
    {
      oldNode = document.getElementById(  "wfxVersions_dialogIDs_editItems"  ).childNodes[i];
      oldItem = parseInt(  oldNode.firstChild.getAttribute( "onclick" ).split( "(" )[1]  );

      var  jItem = oldItem - 1;
      oldNode.firstChild.setAttribute(  "onclick",  "wfxVersions_editOps_clickEntry(" + jItem + ")"  );
      oldNode.setAttribute(  "wfxIsOdd",  i % 2  );
    }

    /*  after delete, show the next entry  */
    var  oLast = document.getElementById(  "wfxVersions_dialogIDs_editItems"  ).childNodes[ wfxVersions_dialogVars_progress ];
    if((  oLast.getAttribute( "hidden" ) != "true"  )||(  wfxVersions_dialogVars_progress == 0  ))
    {
      wfxVersions_editOps_clickEntry(  wfxVersions_dialogVars_progress  );
    }
    else
    {
      wfxVersions_editOps_clickEntry(  wfxVersions_dialogVars_progress - 1  );
    }
  }
}


/****************************************************************************/
/*! \brief  Change/Add a line from in .list file
**
**  \param[in]  bLock  block auto-sorting (for insert-new-entry operation)
**
**  \return  --
*/
function  wfxVersions_editOps_modifyEntry(  bLock  )
{
  /*  just in case the user has edited the entry in the meantime  */
  var  szLine    = wfxVersions_dialogVars_itemsList[wfxVersions_dialogVars_progress].line;
  var  szHome    = wfxVersions_dialogVars_itemsList[wfxVersions_dialogVars_progress].home;
  var  szDown    = wfxVersions_dialogVars_itemsList[wfxVersions_dialogVars_progress].down;
  var  szURL     = wfxVersions_dialogVars_itemsList[wfxVersions_dialogVars_progress].url;
  var  szStart   = wfxVersions_dialogVars_itemsList[wfxVersions_dialogVars_progress].start;
  var  szPre     = wfxVersions_dialogVars_itemsList[wfxVersions_dialogVars_progress].pre;
  var  szPost    = wfxVersions_dialogVars_itemsList[wfxVersions_dialogVars_progress].post;
  var  szVersion = wfxVersions_dialogVars_itemsList[wfxVersions_dialogVars_progress].version;
  var  eDir      = wfxVersions_dialogVars_itemsList[wfxVersions_dialogVars_progress].dir;

  var  szNewLine  = document.getElementById(  "wfxVersions_dialogIDs_editLine"     ).value;
  var  szNewHome  = document.getElementById(  "wfxVersions_dialogIDs_editHome"     ).value;
  var  szNewDown  = document.getElementById(  "wfxVersions_dialogIDs_editDown"     ).value;
  var  szNewURL   = document.getElementById(  "wfxVersions_dialogIDs_editURL"      ).value;
  var  szNewStart = document.getElementById(  "wfxVersions_dialogIDs_editStart"    ).value;
  var  szNewPre   = document.getElementById(  "wfxVersions_dialogIDs_editPre"      ).value;
  var  szNewPost  = document.getElementById(  "wfxVersions_dialogIDs_editPost"     ).value;
  var  szUpdate   = document.getElementById(  "wfxVersions_dialogIDs_editVersion"  ).value.replace(  /\|/g,  "&#124;"  );
  var  eNewDir    = document.getElementById(  "wfxVersions_dialogIDs_editForwards"  ).selectedIndex;

  if(  wfxVersions_dialogVars_itemsList[wfxVersions_dialogVars_progress].status != 6  )
  {
    var  szAlert = "";
    if(  szNewLine.length == 0  )
    {
      szAlert += "\n" + wfxVersions_dialogOps_localeParams(  "szBlankLine"  );
    }
    if(  szNewURL.length == 0  )
    {
      szAlert += "\n" + wfxVersions_dialogOps_localeParams(  "szBlankURL"  );
    }
    if(  szAlert.length != 0  )
    {
      if(  !wfxVersions_globalOps_confirm( document.getElementById( "wfxVersions_dialogIDs_editInsert" ).getAttribute("tooltiptext"), wfxVersions_dialogOps_localeParams("szBlankEdit") + szAlert )  )
      {
        return;
      }
    }

    szAlert = "";
    if(  szNewLine.indexOf( '|' ) >= 0  )
    {
      szAlert += "\n" + wfxVersions_dialogOps_localeParams(  "szPipeLine"  );
    }
    if(  szNewURL.indexOf( '|' ) >= 0  )
    {
      szAlert += "\n" + wfxVersions_dialogOps_localeParams(  "szPipeURL"  );
    }
    if(  szNewHome.indexOf( '|' ) >= 0  )
    {
      szAlert += "\n" + wfxVersions_dialogOps_localeParams(  "szPipeHome"  );
    }
    if(  szNewDown.indexOf( '|' ) >= 0  )
    {
      szAlert += "\n" + wfxVersions_dialogOps_localeParams(  "szPipeDown"  );
    }
    if(  szNewStart.indexOf( '|' ) >= 0  )
    {
      szAlert += "\n" + wfxVersions_dialogOps_localeParams(  "szPipeStart"  );
    }
    if(  szNewPre.indexOf( '|' ) >= 0  )
    {
      szAlert += "\n" + wfxVersions_dialogOps_localeParams(  "szPipePre"  );
    }
    if(  szNewPost.indexOf( '|' ) >= 0  )
    {
      szAlert += "\n" + wfxVersions_dialogOps_localeParams(  "szPipePost"  );
    }
    if(  szAlert.length != 0  )
    {
      if(  !wfxVersions_globalOps_confirm( document.getElementById( "wfxVersions_dialogIDs_editInsert" ).getAttribute("tooltiptext"), wfxVersions_dialogOps_localeParams("szReserved") + szAlert )  )
      {
        return;
      }
    }
  }

  var  bConfirmEnabled = Components.classes[ "@mozilla.org/preferences-service;1" ]
                                   .getService(  Components.interfaces.nsIPrefBranch  )
                                   .getBoolPref(  "extensions.wfx_Versions.confirm.edit.append"  );

  /*  ask the user for confirmation (0-length string = don't ask)  */
  var  szConfirm = wfxVersions_dialogOps_localeParams(  "szAppend"  );
  if(  wfxVersions_dialogVars_progress < (wfxVersions_dialogVars_itemsList.length - 1)  )
  {
    bConfirmEnabled = Components.classes[ "@mozilla.org/preferences-service;1" ]
                                .getService(  Components.interfaces.nsIPrefBranch  )
                                .getBoolPref(  "extensions.wfx_Versions.confirm.edit.modify"  );

    szConfirm = wfxVersions_dialogOps_localeParams(  "szOverEdit"  );
    if((  szLine != szNewLine  )&&(  szConfirm.length != 0  ))
    {
      szConfirm += "\n" + wfxVersions_dialogOps_localeParams(  "szOverLine"  );
    }
    if((  szURL != szNewURL  )&&(  szConfirm.length != 0  ))
    {
      szConfirm += "\n" + wfxVersions_dialogOps_localeParams(  "szOverURL"  );
    }
    if((  szHome != szNewHome  )&&(  szConfirm.length != 0  ))
    {
      szConfirm += "\n" + wfxVersions_dialogOps_localeParams(  "szOverHome"  );
    }
    if((  szDown != szNewDown  )&&(  szConfirm.length != 0  ))
    {
      szConfirm += "\n" + wfxVersions_dialogOps_localeParams(  "szOverDown"  );
    }
    if((  (eDir % 2) != (eNewDir % 2)  )&&(  szConfirm.length != 0  ))
    {
      szConfirm += "\n" + wfxVersions_dialogOps_localeParams(  "szOverDir"  );
    }
    if((  szStart != szNewStart  )&&(  szConfirm.length != 0  ))
    {
      szConfirm += "\n" + wfxVersions_dialogOps_localeParams(  "szOverStart"  );
    }
    if((  szPre != szNewPre  )&&(  szConfirm.length != 0  ))
    {
      szConfirm += "\n" + wfxVersions_dialogOps_localeParams(  "szOverPre"  );
    }
    if((  szPost != szNewPost  )&&(  szConfirm.length != 0  ))
    {
      szConfirm += "\n" + wfxVersions_dialogOps_localeParams(  "szOverPost"  );
    }
    if((  szVersion != szUpdate  )&&(  szConfirm.length != 0  ))
    {
      szConfirm += "\n" + wfxVersions_dialogOps_localeParams(  "szOverVers"  );
    }
  }
  if((  szConfirm.length != 0  )&&(  bConfirmEnabled  ))
  {
    if(  !wfxVersions_globalOps_confirm( document.getElementById( "wfxVersions_dialogIDs_editInsert" ).getAttribute("tooltiptext"), szConfirm )  )
    {
      return;
    }
  }

  wfxVersions_dialogVars_itemsList[wfxVersions_dialogVars_progress].version = wfxVersions_dialogVars_itemsList[wfxVersions_dialogVars_progress].update;

  /*  skip new-line entry, unless it's the item currently being appended  */
  var  bSkipEOF = false;
  var  iMax = wfxVersions_dialogVars_itemsList.length;
  if(  wfxVersions_dialogVars_progress < (iMax - 1)  )
  {
    iMax--;
  }
  else
  {
    bSkipEOF = true;
  }

  wfxVersions_dialogVars_itemsList[wfxVersions_dialogVars_progress].line    = szNewLine;
  wfxVersions_dialogVars_itemsList[wfxVersions_dialogVars_progress].home    = szNewHome;
  wfxVersions_dialogVars_itemsList[wfxVersions_dialogVars_progress].down    = szNewDown;
  wfxVersions_dialogVars_itemsList[wfxVersions_dialogVars_progress].url     = szNewURL;
  wfxVersions_dialogVars_itemsList[wfxVersions_dialogVars_progress].start   = szNewStart;
  wfxVersions_dialogVars_itemsList[wfxVersions_dialogVars_progress].pre     = szNewPre;
  wfxVersions_dialogVars_itemsList[wfxVersions_dialogVars_progress].post    = szNewPost;
  wfxVersions_dialogVars_itemsList[wfxVersions_dialogVars_progress].version = szUpdate;
  wfxVersions_dialogVars_itemsList[wfxVersions_dialogVars_progress].dir     = eNewDir;

  var  i,  txt = "";
  for(  i = 0;  i < iMax;  i++  )
  {
    var  txAdd = wfxVersions_dialogOps_lineFromItem(  i  );
    if((  bSkipEOF  )&&(  i == (iMax-2)  )&&(  txAdd.length == 0  ))
    {
      /*  no EOL (re-use the empty line from EOF)  */
    }
    else
    if(  txt.length != 0  )
    {
      txt += "\r\n";
    }
    else
    {
      /*  no EOL (first line in file)  */
    }
    txt += txAdd;
  }

  if(  wfxVersions_fileOps_storeList( wfxVersions_dialogVars_itemsList[0].list, txt )  )
  {
    /*  switch display between comment / valid item  */
    txt = wfxVersions_dialogOps_lineFromItem(  wfxVersions_dialogVars_progress  );
    var  xItem;
    if(  wfxVersions_dialogOps_isComment( txt )  )
    {
      xItem = new wfxVersions_dialogOps_createNonItem(  txt,  wfxVersions_dialogVars_itemsList[wfxVersions_dialogVars_progress].list  );
    }
    else
    {
      xItem = new wfxVersions_dialogOps_createItem(  txt,  wfxVersions_dialogVars_itemsList[wfxVersions_dialogVars_progress].list  );
    }
    var  eItem = wfxVersions_dialogVars_itemsList[wfxVersions_dialogVars_progress].editrow;
    wfxVersions_dialogVars_itemsList.splice(  wfxVersions_dialogVars_progress,  1,  xItem  );
    wfxVersions_dialogVars_itemsList[wfxVersions_dialogVars_progress].editrow = eItem;

    /*  change the appl. name and tooltip to indicate new content  */
    var  oNode = document.getElementById(  "wfxVersions_dialogIDs_editItems"  ).childNodes[ eItem ].firstChild;
    oNode.setAttribute(  "tooltiptext",  txt  );
    txt = wfxVersions_dialogVars_itemsList[ wfxVersions_dialogVars_progress ].line;
    if(  txt.length > 44  )
    {
      txt = txt.substring(  0,  44  ) + unescape( "%u2026" );
    }
    oNode.value = txt;

    if(  wfxVersions_dialogVars_progress == wfxVersions_dialogVars_itemsList.length - 1  )
    {
      /*  we need another new-line entry  */
      wfxVersions_dialogOps_addEmpty();
    }

    /*  after insert, show the updated entry  */
    wfxVersions_editOps_clickEntry(  wfxVersions_dialogVars_progress  );

    /*  enlarge window to show entire content  */
    wfxVersions_dialogOps_resizeNow();
    wfxVersions_editOps_sortList((  !bLock  )&&(  document.getElementById( "wfxVersions_dialogIDs_sortAuto" ).checked  ));
  }
  else
  {
    wfxVersions_dialogVars_itemsList[wfxVersions_dialogVars_progress].line    = szLine;
    wfxVersions_dialogVars_itemsList[wfxVersions_dialogVars_progress].home    = szHome;
    wfxVersions_dialogVars_itemsList[wfxVersions_dialogVars_progress].down    = szDown;
    wfxVersions_dialogVars_itemsList[wfxVersions_dialogVars_progress].url     = szURL;
    wfxVersions_dialogVars_itemsList[wfxVersions_dialogVars_progress].start   = szStart;
    wfxVersions_dialogVars_itemsList[wfxVersions_dialogVars_progress].pre     = szPre;
    wfxVersions_dialogVars_itemsList[wfxVersions_dialogVars_progress].post    = szPost;
    wfxVersions_dialogVars_itemsList[wfxVersions_dialogVars_progress].version = szVersion;
    wfxVersions_dialogVars_itemsList[wfxVersions_dialogVars_progress].dir     = eDir;
  }
}


/****************************************************************************/
/*! \brief  Sorting of the currently edited .list
**
**  \param[in]  bSort  actual sorting vs. only updating the display
**
**  \return  --
*/
function  wfxVersions_editOps_sortList(  bSort  )
{
  var  bAuto     = document.getElementById(  "wfxVersions_dialogIDs_sortAuto"      ).checked;
  var  bCaSense  = document.getElementById(  "wfxVersions_dialogIDs_sortCaps"      ).checked;
  var  bReversed = document.getElementById(  "wfxVersions_dialogIDs_sortReversed"  ).checked;

  var  oPrefs = Components.classes[ "@mozilla.org/preferences-service;1" ]
                          .getService(  Components.interfaces.nsIPrefBranch  );
  oPrefs.setBoolPref(  "extensions.wfx_Versions.Edit.sort.auto",      bAuto  );
  oPrefs.setBoolPref(  "extensions.wfx_Versions.Edit.sort.caps",      bCaSense  );
  oPrefs.setBoolPref(  "extensions.wfx_Versions.Edit.sort.reversed",  bReversed  );

  if(  bAuto  )
  {
    document.getElementById(  "wfxVersions_dialogIDs_sortButton"  ).setAttribute(  "disabled",  "true"  );

    document.getElementById(  "wfxVersions_dialogIDs_editMoveUp"  ).parentNode.previousSibling.style.setProperty(  "visibility",  "hidden",  "important"  );
    document.getElementById(  "wfxVersions_dialogIDs_editMoveUp"  )                           .style.setProperty(  "visibility",  "hidden",  "important"  );
    document.getElementById(  "wfxVersions_dialogIDs_editMoveDown"  )                         .style.setProperty(  "visibility",  "hidden",  "important"  );
    document.getElementById(  "wfxVersions_dialogIDs_editInsertBefore"  )                     .style.setProperty(  "visibility",  "hidden",  "important"  );
  }
  else
  {
    document.getElementById(  "wfxVersions_dialogIDs_sortButton"  ).removeAttribute(  "disabled"  );

    document.getElementById(  "wfxVersions_dialogIDs_editMoveUp"  ).parentNode.previousSibling.style.removeProperty(  "visibility"  );
    document.getElementById(  "wfxVersions_dialogIDs_editMoveUp"  )                           .style.removeProperty(  "visibility"  );
    document.getElementById(  "wfxVersions_dialogIDs_editMoveDown"  )                         .style.removeProperty(  "visibility"  );
    document.getElementById(  "wfxVersions_dialogIDs_editInsertBefore"  )                     .style.removeProperty(  "visibility"  );
  }

  var  aPanel = document.getElementById(  "wfxVersions_dialogIDs_editItems"  ).childNodes;
  var  oldRow = wfxVersions_dialogVars_itemsList[wfxVersions_dialogVars_progress].editrow;

  var  bComments = true;
  if(  wfxVersions_dialogVars_itemsList.length != aPanel.length  )
  {
    bComments = false;
  }

  var  iOrder = +1;
  if(  bReversed  )
  {
    iOrder = -1;
  }

  /*  work with a copy of the array, in case there is no modification  */
  var  i,  nMods = 0,  aListCopy = [];
  for(  i = 0;  i < (wfxVersions_dialogVars_itemsList.length - 1);  i++  )
  {
    aListCopy.push(  wfxVersions_dialogVars_itemsList[i]  );
  }

  aListCopy.sort
  (
    function(  A,  B  )
    {
      var  szA = A.line;
      var  szB = B.line;
      if(  !bCaSense  )
      {
        szA = szA.toLowerCase();
        szB = szB.toLowerCase();
      }

      var  iComp = 0;
      if(  szA > szB  )
      {
        nMods += iOrder + 1;
        iComp = +1 * iOrder;
      }
      else
      if(  szA < szB  )
      {
        nMods += iOrder - 1;
        iComp = -1 * iOrder;
      }
      else
      {}

      return(  iComp  );
    }
  );

  if((  bSort  )&&(  nMods != 0  ))
  {
    var  bConfirmEnabled = Components.classes[ "@mozilla.org/preferences-service;1" ]
                                     .getService(  Components.interfaces.nsIPrefBranch  )
                                     .getBoolPref(  "extensions.wfx_Versions.confirm.edit.sort"  );

    var  szConfirm = wfxVersions_dialogOps_localeParams(  "szSort"  );
    if((  szConfirm.length != 0  )&&(  bConfirmEnabled  ))
    {
      var  szTip = document.getElementById(  "wfxVersions_dialogIDs_sortButton"  ).getAttribute(  "tooltiptext"  );
      if(  !wfxVersions_globalOps_confirm( szTip, szConfirm )  )
      {
        /*  disable auto-sorting to avoid any inconsistencies  */
        if(  bAuto  )
        {
          document.getElementById(  "wfxVersions_dialogIDs_sortAuto"  ).click();
        }
        return;
      }
    }

    /*  attempt to save the file  */
    var  txt = "";
    for(  i = 0;  i < aListCopy.length;  i++  )
    {
      if(  txt.length != 0  )
      {
        txt += "\r\n";
      }
      txt += wfxVersions_dialogOps_lineFromItemArray(  i,  aListCopy  );
    }

    if(  wfxVersions_fileOps_storeList( aListCopy[0].list, txt )  )
    {
      wfxVersions_editOps_clickEntry(  wfxVersions_dialogVars_progress  );

      var  j = 0;
      for(  i = 0;  i < aListCopy.length;  i++  )
      {
        wfxVersions_dialogVars_itemsList[i] = aListCopy[i];
        if((  bComments  )||(  wfxVersions_dialogVars_itemsList[i].status != 6  ))
        {
          if(  wfxVersions_dialogVars_itemsList[i].editrow == oldRow  )
          {
            wfxVersions_dialogVars_progress = i;
          }
          wfxVersions_dialogVars_itemsList[i].editrow = j;

          txt = wfxVersions_dialogVars_itemsList[i].line;
          if(  txt.length > 44  )
          {
            txt = txt.substring(  0,  44  ) + unescape( "%u2026" );
          }

          aPanel[j].firstChild.setAttribute(  "value",        txt  );
          aPanel[j].firstChild.setAttribute(  "tooltiptext",  wfxVersions_dialogOps_lineFromItem( i )  );
          aPanel[j].firstChild.setAttribute(  "onclick",      "wfxVersions_editOps_clickEntry(" + i + ")"  );

          var  szURL = wfxVersions_dialogOps_likeMRT(  wfxVersions_dialogVars_itemsList[i].url,  wfxVersions_dialogVars_itemsList[i].version  );
          aPanel[j].firstChild.removeAttribute(  "wfxIsError"  );
          if((  wfxVersions_dialogVars_itemsList[i].status == 6  )||(  !wfxVersions_httpOps_isValidURL( szURL )  ))
          {
            aPanel[j].firstChild.setAttribute(  "wfxIsError",  "true"  );
          }

          j++;
        }
      }

      var  newRow = wfxVersions_dialogVars_itemsList[wfxVersions_dialogVars_progress].editrow;
      if(  oldRow != newRow  )
      {
        aPanel[oldRow].firstChild.click();
        aPanel[newRow].firstChild.click();
      }
    }
  }
}


/****************************************************************************/
/*! \brief  Scroll the clicked/focused entry into view inside a scrolled area
**
**  \param[in]  oParent        parent of the node which must be made visible
**  \param[in]  szScrollBoxID  ID of the scrollbox object containing the node
**
**  \return  --
*/
function  wfxVersions_dialogOps_ensureVisible(  oParent,  szScrollBoxID  )
{
  /*  scroll the clicked/focused entry into view  */
  document.getElementById(  szScrollBoxID  ).boxObject
                                            .QueryInterface(  Components.interfaces.nsIScrollBoxObject  )
                                            .ensureElementIsVisible(  oParent  );
}


/****************************************************************************/
/*! \brief  Closes a list tab with scanning in progress
**
**  All list entries loaded for current tab must be removed from scanning
**
**  \param[in]  szList  the .list file path (compare with tab's tooltip)
**
**  \return  --
*/
function  wfxVersions_dialogOps_closeTab(  szList  )
{
  if(  szList == null  )
  {
    wfxVersions_dialogOps_destroyTabs();
    return;
  }

  var  oTabs   = document.getElementById(  "wfxVersions_dialogIDs_tabs"  );
  var  oPanels = document.getElementById(  "wfxVersions_dialogIDs_panels"  );

  var  bAdvance = false;
  if(  oTabs.selectedIndex != 1  )
  {
    bAdvance = true;
  }

  if(  oTabs.childNodes.length == 3  )
  {
    /*  cancel the periodic re-scan timer  */
    wfxVersions_globalOps_stopTimer(  wfxVersions_dialogVars_reScanTimer  );

    if(  bAdvance  )
    {
      oTabs.advanceSelectedTab(  -1,  true  );
    }
    wfxVersions_dialogOps_destroyTabs();
    return;
  }

  /*  find the tab corresponding to current list  */
  var  iTab = 2;
  while(  iTab < oTabs.childNodes.length  )
  {
    if(  oTabs.childNodes[iTab].getAttribute( "tooltiptext" ) == szList  )
    {
      break;
    }
    else
    {
      iTab++;
    }
  }

  /*  remove all list entries loaded from current .list file  */
  var  i;
  for(  i = 0;  i < wfxVersions_dialogVars_itemsList.length;  i++  )
  {
    if((  i != wfxVersions_dialogVars_progress  )&&(  wfxVersions_dialogVars_itemsList[i].list == szList  ))
    {
      wfxVersions_dialogVars_itemsList[i].status = 3;
    }
  }

  /*  abort HTML request in progress  */
  if((  wfxVersions_dialogVars_progress < wfxVersions_dialogVars_itemsList.length  )&&(  wfxVersions_dialogVars_itemsList[wfxVersions_dialogVars_progress].list == szList  ))
  {
    wfxVersions_scanOps_itemAbort(  wfxVersions_dialogVars_progress  );
    wfxVersions_dialogVars_itemsList[wfxVersions_dialogVars_progress].status = 3;
  }

  if(  bAdvance  )
  {
    oTabs.advanceSelectedTab(  -1,  true  );
  }

  var  oldTab   = oTabs.childNodes[iTab];
  var  oldPanel = oPanels.childNodes[iTab];
  wfxVersions_globalOps_nodeCleanUp(  oldTab  );
  wfxVersions_globalOps_nodeCleanUp(  oldPanel  );
  oTabs.removeChild(  oldTab  );
  oPanels.removeChild(  oldPanel  );

  if(  bAdvance  )
  {
    oTabs.advanceSelectedTab(  1,  false  );
  }

  var  iShrink = Components.classes[ "@mozilla.org/preferences-service;1" ]
                           .getService(  Components.interfaces.nsIPrefBranch  )
                           .getIntPref(  "extensions.wfx_Versions.resize.shrinkBack"  );
  if(  iShrink > 1  )
  {
    wfxVersions_dialogOps_resizeBack();
  }
}


/****************************************************************************/
/*! \brief  Closes all tabs except the Options
**
**  Called if the TC plugin path preference has changed while Scanning/Editing
**
**  \param  --
**
**  \return  --
*/
function  wfxVersions_dialogOps_destroyTabs()
{
  var  oTabs   = document.getElementById(  "wfxVersions_dialogIDs_tabs"  );
  var  oPanels = document.getElementById(  "wfxVersions_dialogIDs_panels"  );

  /*  abort HTTP request in progress  */
  if(  wfxVersions_dialogVars_httpRequest != null  )
  {
    wfxVersions_httpOps_ghost();
  }

  /*  we need to re-assess the Scan  */
  while(  oTabs.childNodes.length > 2  )
  {
    var  oldTab   = oTabs.childNodes[2];
    var  oldPanel = oPanels.childNodes[2];
    wfxVersions_globalOps_nodeCleanUp(  oldTab  );
    wfxVersions_globalOps_nodeCleanUp(  oldPanel  );
    oTabs.removeChild(  oldTab  );
    oPanels.removeChild(  oldPanel  );
  }
  wfxVersions_dialogOps_throbberIdle();
  wfxVersions_dialogOps_setImageAttribute(  document.getElementById( "wfxVersions_dialogIDs_buttonEdit" ),  "chrome://wfx_versions/skin/wfx_versions_edit.png"  );
  wfxVersions_dialogOps_setImageAttribute(  document.getElementById( "wfxVersions_dialogIDs_buttonScan" ),  "chrome://wfx_versions/skin/wfx_versions_scan.png"  );

  /*  we need to re-assess the Edit  */
  wfxVersions_globalOps_nodeCleanUp(  document.getElementById( "wfxVersions_dialogIDs_editItems" )  );
  document.getElementById(  "wfxVersions_dialogIDs_editProgress"  ).setAttribute(  "hidden",  "true"   );
  document.getElementById(  "wfxVersions_dialogIDs_editVersion"   ).setAttribute(  "hidden",  "false"  );
  document.getElementById(  "wfxVersions_dialogIDs_editEnable"    ).setAttribute(  "hidden",  "true"   );

  oTabs.firstChild.setAttribute(  "hidden",  "true"  );
  oTabs.advanceSelectedTab(  1,  true  );

  wfxVersions_dialogVars_itemsList = null;
  wfxVersions_dialogVars_progress = 0;
  document.getElementById(  "wfxVersions_dialogIDs_statusBar"  ).value = " ";

  var  iShrink = Components.classes[ "@mozilla.org/preferences-service;1" ]
                           .getService(  Components.interfaces.nsIPrefBranch  )
                           .getIntPref(  "extensions.wfx_Versions.resize.shrinkBack"  );
  if(  iShrink > 0  )
  {
    wfxVersions_dialogOps_resizeBack();
  }
}


/****************************************************************************/
/*! \brief  Create a new button with custom properties
**
**  Needed for making plain buttons in contrast with theme-specific buttons
**  which may have rounded corners, background images, etc.
**
**  \param[in]  oParent     the document element which will contain the button
**  \param[in]  szID        the ID of the new button
**  \param[in]  szTip       the tooltiptext of the new button
**  \param[in]  szImage     the chrome image for the new button
**  \param[in]  szScrollID  the ID of scrollbox object containing the button
**
**  \return  the newly created button
*/
function  wfxVersions_dialogOps_newButton(  oParent,  szID,  szTip,  szImage,  szScrollID  )
{
  var  oButton = document.createElement(  "button"  );
  oButton.setAttribute(  "class",  "wfxVersions_skinButton"  );

  if(  szImage  )
  {
    wfxVersions_dialogOps_setImageAttribute(  oButton,  szImage  );
  }
  if(  szID  )
  {
    oButton.setAttribute(  "id",  szID  );
  }
  if(  szTip  )
  {
    oButton.setAttribute(  "tooltiptext",  szTip  );
  }
  oButton.setAttribute(  "onfocus",  "wfxVersions_dialogOps_ensureVisible(parentNode,'" + szScrollID + "')"  );

  oParent.appendChild(  oButton  );
  return(  oButton  );
}


/****************************************************************************/
/*! \brief  Append additional URLs during Scan mode
**
**  The URL is appended in the format of image with clickable link
**
**  \param[in]  oParent     parent node in the dialog-box
**  \param[in]  szImage     image for the pseudo-button link
**  \param[in]  szURL       the appended URL
**  \param[in]  bVisible    enable/disable additional URL
**  \param[in]  szScrollID  the ID of scrollbox object containing the button
**
**  \return  --
*/
function  wfxVersions_dialogOps_newImageURL(  oParent,  szImage,  szURL,  bVisible,  szScrollID  )
{
  /*  create a button, but make it look like a label  */
  var  oButton = wfxVersions_dialogOps_newButton(  oParent,  null,  szURL,  szImage,  szScrollID  );
  oButton.setAttribute(  "class",       "wfxVersions_skinURL"  );
  oButton.setAttribute(  "onclick",     "wfxVersions_globalOps_clickURL(event)"  );
  oButton.setAttribute(  "onkeypress",  "if(event.keyCode==event.DOM_VK_RETURN) wfxVersions_globalOps_clickURL(event)"  );
  oButton.setAttribute(  "onfocus",     "style.setProperty('border-color',wfxVersions_globalVars_editColorText,'important');wfxVersions_dialogOps_ensureVisible(parentNode,'"+szScrollID+"')"  );
  oButton.setAttribute(  "onblur",      "style.setProperty('border-color','transparent',                       'important')"  );
  oButton.style.setProperty(  "border",  "1px dotted transparent",  "important"  );

  /*  enable/disable the appended label  */
  if((  bVisible  )&&(  szURL.length != 0  ))
  {
    oButton.style.setProperty(  "visibility",  "visible",  "important"  );
  }
  else
  {
    oButton.style.setProperty(  "visibility",  "hidden",  "important"  );
  }

  var  bShowLinks = Components.classes[ "@mozilla.org/preferences-service;1" ]
                              .getService(  Components.interfaces.nsIPrefBranch  )
                              .getBoolPref(  "extensions.wfx_Versions.show.alternate.URLs"  );
  if(  bShowLinks  )
  {
    oButton.removeAttribute(  "hidden"  );
  }
  else
  {
    oButton.setAttribute(  "hidden",  "true"  );
  }
}


/****************************************************************************/
/*! \brief  Adjusts the update-string/errors area width
**
**  Adjust such that the version-string area is near the application-name area
**
**  \param  --
**
**  \return  --
**
**  \see  wfxVersions_dialogOps_limitScanLines()
*/
function  wfxVersions_dialogOps_adjustScanLines()
{
  var  j,  aTabs = document.getElementById(  "wfxVersions_dialogIDs_tabs"  ).childNodes;
  for(  j = 2;  j < aTabs.length;  j++  )
  {
    /*  find the smallest spacer width  */
    var  i,  n = 4000,  aPanel = document.getElementById(  wfxVersions_fileOps_leafName( aTabs[j].getAttribute("tooltiptext") )  ).childNodes;
    for(  i = 0;  i < aPanel.length;  i++  )
    {
      if(  aPanel[i].childNodes.length > 3  )
      {
        var  m = aPanel[i].childNodes[5].boxObject.screenX - ( aPanel[i].childNodes[1].boxObject.screenX + aPanel[i].childNodes[1].boxObject.width );
        if(  m < n  )
        {
          n = m;
        }
      }
    }
    n -= 5;
    if(  n < 0  )
    {
      n = 0;
    }
    n += 400;

    /*  adjust by same increase from 400px  */
    for(  i = 0;  i < aPanel.length;  i++  )
    {
      if(  aPanel[i].childNodes.length > 3  )
      {
        aPanel[i].childNodes[6].style.setProperty(  "width",  n + "px",  "important"  );
      }
    }
  }
}


/****************************************************************************/
/*! \brief  Reduces the update-string/errors area width
**
**  To be used before a resize operation, to allow reducing the width
**
**  \param  --
**
**  \return  --
**
**  \see  wfxVersions_dialogOps_adjustScanLines()
*/
function  wfxVersions_dialogOps_limitScanLines()
{
  var  j,  aTabs = document.getElementById(  "wfxVersions_dialogIDs_tabs"  ).childNodes;
  for(  j = 2;  j < aTabs.length;  j++  )
  {
    /*  start from 400px  */
    var  i,  aPanel = document.getElementById(  wfxVersions_fileOps_leafName( aTabs[j].getAttribute("tooltiptext") )  ).childNodes;
    for(  i = 0;  i < aPanel.length;  i++  )
    {
      if(  aPanel[i].childNodes.length > 3  )
      {
        aPanel[i].childNodes[6].style.setProperty(  "width",  "400px",  "important"  );
      }
    }
  }
}


/****************************************************************************/
/*! \brief  Prepares the document for scanning and launches the process
**
**  The .lists are actually delay-loaded as not to cause too much load on the
**  application display
**
**  \param[in]  szOps  name of the operation performed, used in error alerts
**
**  \return  --
*/
function  wfxVersions_scanOps_start(  szOps  )
{
  /*  same as Cancel changes to preferences without close  */
  wfxVersions_dialogOps_loadPrefs();

  if(  wfxVersions_dialogVars_filesList.length == 0  )
  {
    return;
  }
  wfxVersions_memoOps_storeSelections(  "extensions.wfx_Versions.scanSelections",  "extensions.wfx_Versions.scanMemorize"  );

  /*  delay load  */
  document.getElementById(  "wfxVersions_dialogIDs_statusBar"  ).value = " " + unescape( "%u2026" ) + "  0  ( " + wfxVersions_dialogVars_filesList[0].leafName + " )";
  wfxVersions_dialogVars_progress = 0;

  var  iList,  bScanning = false;
  for(  iList = 0;  iList < wfxVersions_dialogVars_filesList.length;  iList++  )
  {
    /*  create a file-object for each file/folder in the plugin folder  */
    var  fsList = wfxVersions_dialogVars_filesList[iList];

    /*  create a stream-objects for UTF-8 encoding  */
    var  ioFile = Components.classes[ "@mozilla.org/network/file-input-stream;1" ]
                            .createInstance(  Components.interfaces.nsIFileInputStream  );
    ioFile.QueryInterface(  Components.interfaces.nsILineInputStream  );
    ioFile.init(  fsList,  0x01/*read-only*/,  0444,  0  );

    var  ioStream = Components.classes[ "@mozilla.org/intl/converter-input-stream;1" ]
                              .createInstance(  Components.interfaces.nsIConverterInputStream  );
    ioStream.QueryInterface(  Components.interfaces.nsIUnicharLineInputStream  );
    ioStream.init(  ioFile,  "UTF-8",  1024,  0/*force exception on readLine encoding error*/  );

    /*  detect encoding  */
    var  bMoreLines;
    var  ozLine = {};
    try
    {
      do
      {
        bMoreLines = ioStream.readLine(  ozLine  );
      }
      while(  bMoreLines  );

      ioStream.close();
      ioFile.close();

      ioFile.init(  fsList,  0x01/*read-only*/,  0444,  0  );
      ioStream.init(  ioFile,  "UTF-8",  1024,  0  );
    }
    catch(  e  )
    {
      ioStream.close();
      ioFile.close();

      ioStream = ioFile;
      ioStream.init(  fsList,  0x01/*read-only*/,  0444,  0  );
    }

    var  aDump = [];
    var  bNewList = true;
    do
    {
      /*  parse line-by-line, content is valid if:
      **  - 6 items per line (7, 8, 9 with mods)
      **  - non-empty application name & URL
      */
      bMoreLines = ioStream.readLine(  ozLine  );
      if(  wfxVersions_dialogOps_isComment( ozLine.value )  )
      {
        /*  add to items-list as non-item (disabled/hidden)  */
        if(  bNewList  )
        {
          aDump.push(  ozLine.value  );
        }
        else
        {
          wfxVersions_dialogVars_progress = wfxVersions_dialogVars_itemsList.length;
          wfxVersions_dialogVars_itemsList.push(  new wfxVersions_dialogOps_createNonItem( ozLine.value, fsList.path )  );
        }
      }
      else
      {
        if(  bNewList  )
        {
          if(  !bScanning  )
          {
            /*  just in case the user clicks [Scan] while Edit in progress  */
            if(  !document.getElementById( "wfxVersions_dialogIDs_tabs" ).firstChild.hasAttribute( "hidden" )  )
            {
              wfxVersions_dialogOps_destroyTabs();
            }
            wfxVersions_dialogVars_itemsList = [];
            bScanning = true;
          }

          var  i;
          for(  i = 0;  i < aDump.length;  i++  )
          {
            wfxVersions_dialogVars_progress = wfxVersions_dialogVars_itemsList.length;
            wfxVersions_dialogVars_itemsList.push(  new wfxVersions_dialogOps_createNonItem( aDump[i], fsList.path )  );
          }

          bNewList = false;
        }

        /*  add to items-list  */
        wfxVersions_dialogVars_progress = wfxVersions_dialogVars_itemsList.length;
        wfxVersions_dialogVars_itemsList.push(  new wfxVersions_dialogOps_createItem( ozLine.value, fsList.path )  );
      }
    }
    while(  bMoreLines  );
    ioStream.close();
    ioFile.close();
  }

  if(  bScanning  )
  {
    wfxVersions_dialogOps_throbberScan();
    wfxVersions_dialogOps_setImageAttribute(  document.getElementById( "wfxVersions_dialogIDs_buttonScan" ),  "chrome://wfx_versions/skin/wfx_versions_throbber.gif"  );
    wfxVersions_globalOps_setTimeout(  "wfxVersions_scanOps_startDelayed( 0 )",  10  );

    window[ "wfxListsLoading" ] = true;
    openDialog(  "chrome://wfx_versions/content/wfx_versions_progress.xul",
                 "",
                 "chrome,centerscreen,modal"  );
  }
  else
  {
    /*  files were opened, but had incorrect format  */
    var  szListValid = "";
    for(  iList = 0;  iList < wfxVersions_dialogVars_filesList.length;  iList++  )
    {
      szListValid += "\n" + wfxVersions_dialogVars_filesList[iList].path;
    }
    wfxVersions_globalOps_alert(  szOps,  wfxVersions_dialogOps_localeParams( "szLines" ) + "\n" + szListValid  );
  }
}


/****************************************************************************/
/*! \brief  Loads the .lists item-by-item to reduce the load
**
**  Opens each .list file from the .list files folder, and dispatches
**  each line that has the format of 6 fields separated by '|':
**  - application name/label
**  - URL of application
**  - start point (to search from in page)
**  - pre-string (just before version number/string)
**  - post-string (just after version number/string)
**  - version number/string (old)
**
**  The following changes have been introduced to the original .list line
**  format, which are not compatible with the TC plugin:
**  - the line can start with '|' to indicate backwards scanning in a page
**  - the line can contain 3 URL fields (2 additional):
**    - homepage link
**    - scan page (from the original .list format)
**    - direct download link
**  - the above mods can be combined
**
**  The document will contain separate tabs for each .list file, and each tab
**  will contain 3 columns:
**  - application name (with hyperlink to application URL)
**  - old version (from .list file)
**  - new version (to be updated after scanning)
**
**  A HTTP Request is constructed for first URL, and next HTTP Requests
**  will be launched from the HTTP Request handler.
**
**  \param[in]  kItem  the current item being loaded
**
**  \return  --
*/
function  wfxVersions_scanOps_startDelayed(  kItem  )
{
  var  i;
  var  oPrefs = Components.classes[ "@mozilla.org/preferences-service;1" ]
                          .getService(  Components.interfaces.nsIPrefBranch  );

  if(  kItem >= wfxVersions_dialogVars_itemsList.length  )
  {
    document.getElementById(  "wfxVersions_dialogIDs_tabs"  ).advanceSelectedTab(  1,  true  );
    document.getElementById(  "wfxVersions_dialogIDs_statusBar"  ).value = " ";

    /*  enlarge window to show entire content  */
    wfxVersions_dialogOps_resizeNow();

    /*  launch the first HTTP Request
    **  (future requests are launched from within onload() notification)
    */
    wfxVersions_globalOps_setTimeout(  "wfxVersions_httpOps_headLaunch()",  40  );

    document.getElementById(  "wfxVersions_dialogIDs_tabs"  ).advanceSelectedTab(  1,  false  );

    window[ "wfxListsLoading" ] = false;
    return;
  }

  wfxVersions_dialogVars_progress = kItem;

  var  szFile = wfxVersions_fileOps_leafName(  wfxVersions_dialogVars_itemsList[kItem].list  );
  document.getElementById(  "wfxVersions_dialogIDs_statusBar"  ).value = " " + unescape( "%u2026" ) + "  " + kItem + "  ( " + szFile + " )";

  if((  kItem == 0  )||(  wfxVersions_dialogVars_itemsList[kItem].list != wfxVersions_dialogVars_itemsList[kItem-1].list  ))
  {
    /*  new tab needed for new .list file  */
    var  newTab = document.createElement(  "tab"  );
    newTab.setAttribute(  "label",        " " + szFile + " "  );
    newTab.setAttribute(  "tooltiptext",  wfxVersions_dialogVars_itemsList[kItem].list  );
    newTab.setAttribute(  "align",        "center"  );
    wfxVersions_dialogOps_setImageProperty(  newTab,  "chrome://wfx_versions/skin/wfx_versions_throbber.gif"  );
    document.getElementById( "wfxVersions_dialogIDs_tabs" ).appendChild(  newTab  );

    var  newPanel = document.createElement(  "tabpanel"  );
    document.getElementById( "wfxVersions_dialogIDs_panels" ).appendChild(  newPanel  );

    var  newBox = document.createElement(  "hbox"  );
    newBox.setAttribute(  "flex",  "1"  );
    newPanel.appendChild(  newBox  );
    newPanel = newBox;

    /*  container of application name, old and new version  */
    newBox = document.createElement(  "scrollbox"  );
    newBox.setAttribute(  "orient",  "vertical"  );
    newBox.setAttribute(  "id",      szFile  );
    newBox.setAttribute(  "flex",    "1"  );
    newBox.style.setProperty(  "overflow-x",  "hidden",  "important"  );
    newBox.style.setProperty(  "overflow-y",  "scroll",  "important"  );
    newBox.style.setProperty(  "padding",     "0px",     "important"  );
    newBox.style.setProperty(  "margin",      "0px",     "important"  );
    newPanel.appendChild(  newBox  );

    document.getElementById(  "wfxVersions_dialogIDs_tabs"  ).advanceSelectedTab(  1,  false  );
  }

  var  szLine = wfxVersions_dialogOps_lineFromItem(  kItem  );
  var  newNode,  newLine;
  if(  wfxVersions_dialogVars_itemsList[kItem].status != 0  )
  {
    if(  oPrefs.getBoolPref( "extensions.wfx_Versions.comments.display" )  )
    {
      /*  add a simple label for comments  */
      newLine = document.createElement(  "hbox"  );
      newLine.setAttribute(  "align",     "center"  );
      newLine.setAttribute(  "class",     "wfxVersions_skinBGnd"  );
      newLine.setAttribute(  "wfxIsOdd",  document.getElementById( szFile ).childNodes.length % 2  );
      newLine.style.setProperty(  "min-height",  "20px",  "important"  );
      document.getElementById(  szFile  ).appendChild(  newLine  );

      newNode = document.createElement(  "label"  );
      newNode.setAttribute(  "value",       " " + szLine + " "  );
      newNode.setAttribute(  "crop",        "center"  );
      newNode.setAttribute(  "flex",        "1"  );
      newNode.setAttribute(  "class",       "wfxVersions_skinText"  );
      newNode.setAttribute(  "wfxIsError",  "true"  );
      newNode.style.setProperty(  "margin",  "0px",  "important"  );
      newLine.appendChild(  newNode  );
    }
  }
  else
  {
    var  szArray = szLine.split(  '|'  );
    var  eDir    = szArray.length - 6;

    newLine = document.createElement(  "hbox"  );
    newLine.setAttribute(  "align",     "center"  );
    newLine.setAttribute(  "class",     "wfxVersions_skinBGnd"  );
    newLine.setAttribute(  "wfxIsOdd",  document.getElementById( szFile ).childNodes.length % 2  );
    document.getElementById(  szFile  ).appendChild(  newLine  );

    /*  application name and old version  */
    newNode = wfxVersions_dialogOps_newButton(  newLine,  "e" + wfxVersions_dialogVars_progress,
                                                wfxVersions_dialogOps_localeParams( "szSoloEdit" ),
                                                "chrome://wfx_versions/skin/wfx_versions_edit.png",  szFile  );
    newNode.setAttribute(  "oncommand",  "wfxVersions_scanOps_editOne(" + wfxVersions_dialogVars_progress + ",getAttribute('tooltiptext'))"  );
    if(  !oPrefs.getBoolPref( "extensions.wfx_Versions.show.individual.scans" )  )
    {
      newNode.style.setProperty(  "visibility",  "hidden",  "important"  );
      newNode.style.setProperty(  "margin",      "-8px",    "important"  );
    }

    wfxVersions_dialogOps_newImageURL(  newLine,  "chrome://wfx_versions/skin/wfx_versions_home.png",  wfxVersions_dialogOps_likeMRT( szArray[1+(eDir%2)], szArray[5+eDir] ),  eDir > 1,  szFile  );
    wfxVersions_dialogOps_newImageURL(  newLine,  "chrome://wfx_versions/skin/wfx_versions_down.png",  wfxVersions_dialogOps_likeMRT( szArray[2+(eDir%2)], szArray[5+eDir] ),  eDir > 1,  szFile  );

    newNode = document.createElement(  "label"  );
    newNode.setAttribute(  "id",           "n" + wfxVersions_dialogVars_progress  );
    newNode.setAttribute(  "value",        szArray[eDir%2]  );
    newNode.setAttribute(  "tooltiptext",  wfxVersions_dialogOps_likeMRT( szArray[1+eDir], szArray[5+eDir] )  );
    newNode.setAttribute(  "onclick",      "wfxVersions_globalOps_clickURL(event)"  );
    newNode.setAttribute(  "onfocus",      "wfxVersions_dialogOps_ensureVisible(parentNode,'"+szFile+"')"  );
    newNode.setAttribute(  "class",        "wfxVersions_skinLink"  );
    if(  !wfxVersions_httpOps_isValidURL( wfxVersions_dialogOps_likeMRT(szArray[1+eDir],szArray[5+eDir]) )  )
    {
      newNode.setAttribute(  "wfxIsError",  "true"  );
    }
    newLine.appendChild(  newNode  );

    newNode = document.createElement(  "spacer"  );
    newNode.setAttribute(  "flex",  "1"  );
    newLine.appendChild(  newNode  );

    var  szVer = wfxVersions_globalOps_convertHTML(  szArray[5+eDir]  );
    if(  szVer.length > 36  )
    {
      szVer = szVer.substring(  0,  36  ) + unescape( "%u2026" );
    }
    newNode = document.createElement(  "label"  );
    newNode.setAttribute(  "id",     "y" + wfxVersions_dialogVars_progress  );
    newNode.setAttribute(  "value",  szVer  );
    newNode.setAttribute(  "class",  "wfxVersions_skinText"  );
    newLine.appendChild(  newNode  );

    /*  new version  */
    newNode = document.createElement(  "vbox"  );
    newNode.style.setProperty(  "width",       "400px",  "important"  );
    newNode.style.setProperty(  "min-height",  "20px",   "important"  );
    newLine.appendChild(  newNode  );
    newLine.setAttribute(  "height",  "0"  );

    newLine = document.createElement(  "hbox"  );
    newLine.setAttribute(  "height",  "0"  );
    newLine.setAttribute(  "align",   "center"  );
    newNode.appendChild(  newLine  );

    newNode = wfxVersions_dialogOps_newButton(  newLine,  "o" + wfxVersions_dialogVars_progress,
                                                wfxVersions_dialogOps_localeParams( "szSoloStop" ),
                                                "chrome://wfx_versions/skin/wfx_versions_stop.png",  szFile  );
    newNode.setAttribute(  "wfxVersions_customFocus",  "r" + wfxVersions_dialogVars_progress  );
    newNode.setAttribute(  "oncommand",                "wfxVersions_scanOps_itemAbort(" + wfxVersions_dialogVars_progress + ")"  );
    if(  !oPrefs.getBoolPref( "extensions.wfx_Versions.show.individual.scans" )  )
    {
      newNode.style.setProperty(  "visibility",  "hidden",  "important"  );
      newNode.style.setProperty(  "margin",      "-8px",    "important"  );
    }

    newNode = wfxVersions_dialogOps_newButton(  newLine,  "r" + wfxVersions_dialogVars_progress,
                                                wfxVersions_dialogOps_localeParams( "szSoloRLoad" ),
                                                "chrome://wfx_versions/skin/wfx_versions_scan.png",  szFile  );
    newNode.setAttribute(  "wfxVersions_customFocus",  "o" + wfxVersions_dialogVars_progress  );
    newNode.setAttribute(  "oncommand",                "wfxVersions_scanOps_itemResume(" + wfxVersions_dialogVars_progress + ")"  );
    newNode.setAttribute(  "hidden",                   "true"  );
    if(  !oPrefs.getBoolPref( "extensions.wfx_Versions.show.individual.scans" )  )
    {
      newNode.style.setProperty(  "visibility",  "hidden",  "important"  );
      newNode.style.setProperty(  "margin",      "-8px",    "important"  );
    }

    newNode = wfxVersions_dialogOps_newButton(  newLine,  "x" + wfxVersions_dialogVars_progress,  " ",  null,  szFile  );
    newNode.setAttribute(  "wfxVersions_customFocus",  "r" + wfxVersions_dialogVars_progress  );
    newNode.setAttribute(  "oncommand",                "wfxVersions_fileOps_modifyList(" + wfxVersions_dialogVars_progress + ")"  );
    newNode.setAttribute(  "hidden",                   "true"  );

    newNode = document.createElement(  "progressmeter"  );
    newNode.setAttribute(  "id",           "m" + wfxVersions_dialogVars_progress  );
    newNode.setAttribute(  "value",        "0"  );
    newNode.setAttribute(  "mode",         "determined"  );
    newNode.setAttribute(  "tooltiptext",  szLine  );
    newNode.style.setProperty(  "width",   "321px",            "important"  );
    newNode.style.setProperty(  "margin",  "1px 0px 0px 0px",  "important"  );
    newLine.appendChild(  newNode  );

    newNode = document.createElement(  "label"  );
    newNode.setAttribute(  "id",           "u" + wfxVersions_dialogVars_progress  );
    newNode.setAttribute(  "tooltiptext",  szLine  );
    newNode.setAttribute(  "hidden",       "true"  );
    newNode.setAttribute(  "class",        "wfxVersions_skinText"  );
    newNode.setAttribute(  "wfxIsError",   "true"  );
    newLine.appendChild(  newNode  );

    newNode = wfxVersions_dialogOps_newButton(  newLine,  "h" + wfxVersions_dialogVars_progress,
                                                wfxVersions_dialogOps_localeParams( "szSoloHelp" ),
                                                "chrome://wfx_versions/skin/wfx_versions_help.png",  szFile  );
    newNode.setAttribute(  "wfxVersions_customFocus",  "r" + wfxVersions_dialogVars_progress  );
    newNode.setAttribute(  "hidden",                   "true"  );
  }

  var  jItem = kItem + 1;
  wfxVersions_globalOps_setTimeout(  "wfxVersions_scanOps_startDelayed( " + jItem + " )",  10  );
}


/****************************************************************************/
/*! \brief  Edit one individual .list entry, during scan mode
**
**  The inline edit is separate from the current scan operation, so scanning
**  can go on undisturbed.
**
**  \param[in]  iItem  the scanned item being edited
**  \param[in]  szOps  name of the operation performed, used in error alerts
**
**  \return  --
*/
function  wfxVersions_scanOps_editOne(  iItem,  szOps  )
{
  /*  a separate object is used for "workspace"  */
  var  oItem = new wfxVersions_dialogOps_createItem(  wfxVersions_dialogOps_lineFromItem( iItem ),
                                                      wfxVersions_dialogVars_itemsList[iItem].list  );

  var  szLine  = wfxVersions_dialogVars_itemsList[iItem].line;
  var  szURL   = wfxVersions_dialogVars_itemsList[iItem].url;
  var  szStart = wfxVersions_dialogVars_itemsList[iItem].start;
  var  szPre   = wfxVersions_dialogVars_itemsList[iItem].pre;
  var  szPost  = wfxVersions_dialogVars_itemsList[iItem].post;
  var  eDir    = wfxVersions_dialogVars_itemsList[iItem].dir;
  var  szHome  = wfxVersions_dialogVars_itemsList[iItem].home;
  var  szDown  = wfxVersions_dialogVars_itemsList[iItem].down;

  var  oEdit = {  write: true  };
  while(  oEdit.write  )
  {
    /*  open the modal window for editing current item  */
    oEdit.write = false;
    openDialog(  "chrome://wfx_versions/content/wfx_versions_editem.xul",
                 "",
                 "chrome,centerscreen,modal",
                 oItem, szOps, oEdit  );

    var  szBlank = "";
    if(  oItem.line.length == 0  )
    {
      szBlank += "\n" + wfxVersions_dialogOps_localeParamsMod(  "szBlankLine",  wfxVersions_dialogVars_itemsList[iItem],  oItem  );
    }
    if(  oItem.url.length == 0  )
    {
      szBlank += "\n" + wfxVersions_dialogOps_localeParamsMod(  "szBlankURL",  wfxVersions_dialogVars_itemsList[iItem],  oItem  );
    }
    if(  szBlank.length != 0  )
    {
      szBlank = wfxVersions_dialogOps_localeParamsMod(  "szBlankEdit",  wfxVersions_dialogVars_itemsList[iItem],  oItem  ) + szBlank;
    }

    var  szAlert = "";
    if(  oItem.line.indexOf( '|' ) >= 0  )
    {
      szAlert += "\n" + wfxVersions_dialogOps_localeParamsMod(  "szPipeLine",  wfxVersions_dialogVars_itemsList[iItem],  oItem  );
    }
    if(  oItem.url.indexOf( '|' ) >= 0  )
    {
      szAlert += "\n" + wfxVersions_dialogOps_localeParamsMod(  "szPipeURL",  wfxVersions_dialogVars_itemsList[iItem],  oItem  );
    }
    if(  oItem.home.indexOf( '|' ) >= 0  )
    {
      szAlert += "\n" + wfxVersions_dialogOps_localeParamsMod(  "szPipeHome",  wfxVersions_dialogVars_itemsList[iItem],  oItem  );
    }
    if(  oItem.down.indexOf( '|' ) >= 0  )
    {
      szAlert += "\n" + wfxVersions_dialogOps_localeParamsMod(  "szPipeDown",  wfxVersions_dialogVars_itemsList[iItem],  oItem  );
    }
    if(  oItem.start.indexOf( '|' ) >= 0  )
    {
      szAlert += "\n" + wfxVersions_dialogOps_localeParamsMod(  "szPipeStart",  wfxVersions_dialogVars_itemsList[iItem],  oItem  );
    }
    if(  oItem.pre.indexOf( '|' ) >= 0  )
    {
      szAlert += "\n" + wfxVersions_dialogOps_localeParamsMod(  "szPipePre",  wfxVersions_dialogVars_itemsList[iItem],  oItem  );
    }
    if(  oItem.post.indexOf( '|' ) >= 0  )
    {
      szAlert += "\n" + wfxVersions_dialogOps_localeParamsMod(  "szPipePost",  wfxVersions_dialogVars_itemsList[iItem],  oItem  );
    }
    if(  szAlert.length != 0  )
    {
      szAlert = wfxVersions_dialogOps_localeParamsMod(  "szReserved",  wfxVersions_dialogVars_itemsList[iItem],  oItem  ) + szAlert;
    }

    var  szPrepend = wfxVersions_dialogOps_localeParamsMod(  "szOverEdit",  wfxVersions_dialogVars_itemsList[iItem],  oItem  );
    var  szConfirm = "";
    var  bReScan = false;
    if(  szLine != oItem.line  )
    {
      szConfirm += "\n" + wfxVersions_dialogOps_localeParamsMod(  "szOverLine",  wfxVersions_dialogVars_itemsList[iItem],  oItem  );
    }
    if(  szURL != oItem.url  )
    {
      szConfirm += "\n" + wfxVersions_dialogOps_localeParamsMod(  "szOverURL",  wfxVersions_dialogVars_itemsList[iItem],  oItem  );
      bReScan = true;
    }
    if(  szHome != oItem.home  )
    {
      szConfirm += "\n" + wfxVersions_dialogOps_localeParamsMod(  "szOverHome",  wfxVersions_dialogVars_itemsList[iItem],  oItem  );
    }
    if(  szDown != oItem.down  )
    {
      szConfirm += "\n" + wfxVersions_dialogOps_localeParamsMod(  "szOverDown",  wfxVersions_dialogVars_itemsList[iItem],  oItem  );
    }
    if(  (eDir % 2) != (oItem.dir % 2)  )
    {
      szConfirm += "\n" + wfxVersions_dialogOps_localeParamsMod(  "szOverDir",  wfxVersions_dialogVars_itemsList[iItem],  oItem  );
      bReScan = true;
    }
    if(  szStart != oItem.start  )
    {
      szConfirm += "\n" + wfxVersions_dialogOps_localeParamsMod(  "szOverStart",  wfxVersions_dialogVars_itemsList[iItem],  oItem  );
      bReScan = true;
    }
    if(  szPre != oItem.pre  )
    {
      szConfirm += "\n" + wfxVersions_dialogOps_localeParamsMod(  "szOverPre",  wfxVersions_dialogVars_itemsList[iItem],  oItem  );
      bReScan = true;
    }
    if(  szPost != oItem.post  )
    {
      szConfirm += "\n" + wfxVersions_dialogOps_localeParamsMod(  "szOverPost",  wfxVersions_dialogVars_itemsList[iItem],  oItem  );
      bReScan = true;
    }

    if(  !oEdit.write  )
    {
      /*  no change (edit canceled)  */
    }
    else
    if(  szBlank.length != 0  )
    {
      wfxVersions_globalOps_alert(  szOps,  szBlank  );
    }
    else
    if(  szAlert.length != 0  )
    {
      wfxVersions_globalOps_alert(  szOps,  szAlert  );
    }
    else
    if(  szConfirm.length != 0  )
    {
      var  bConfirmEnabled = Components.classes[ "@mozilla.org/preferences-service;1" ]
                                       .getService(  Components.interfaces.nsIPrefBranch  )
                                       .getBoolPref(  "extensions.wfx_Versions.confirm.quickedit.save"  );

      if((  szPrepend.length != 0  )&&(  bConfirmEnabled  ))
      {
        if(  !wfxVersions_globalOps_confirm( szOps, szPrepend + szConfirm )  )
        {
          /*  change not confirmed  */
          return;
        }
      }

      /*  change the item in the list  */
      wfxVersions_dialogVars_itemsList[iItem].line  = oItem.line;
      wfxVersions_dialogVars_itemsList[iItem].url   = oItem.url;
      wfxVersions_dialogVars_itemsList[iItem].dir   = oItem.dir;
      wfxVersions_dialogVars_itemsList[iItem].start = oItem.start;
      wfxVersions_dialogVars_itemsList[iItem].pre   = oItem.pre;
      wfxVersions_dialogVars_itemsList[iItem].post  = oItem.post;
      wfxVersions_dialogVars_itemsList[iItem].home  = oItem.home;
      wfxVersions_dialogVars_itemsList[iItem].down  = oItem.down;

      /*  save the modified list  */
      var  txt = "";
      for(  var  j = 0;  j < wfxVersions_dialogVars_itemsList.length;  j++  )
      {
        if(  wfxVersions_dialogVars_itemsList[j].list == wfxVersions_dialogVars_itemsList[iItem].list  )
        {
          if(  txt.length != 0  )
          {
            txt += "\r\n";
          }
          txt += wfxVersions_dialogOps_lineFromItem(  j  );
        }
      }

      if(  wfxVersions_fileOps_storeList( wfxVersions_dialogVars_itemsList[iItem].list, txt )  )
      {
        /*  change the display of the item  */
        document.getElementById(  "m" + iItem  ).setAttribute(  "tooltiptext",  wfxVersions_dialogOps_lineFromItem( iItem )  );
        document.getElementById(  "u" + iItem  ).setAttribute(  "tooltiptext",  wfxVersions_dialogOps_lineFromItem( iItem )  );
        document.getElementById(  "e" + iItem  ).setAttribute(  "tooltiptext",  wfxVersions_dialogOps_localeParamsMod( "szSoloEdit",  wfxVersions_dialogVars_itemsList[iItem], oItem )  );
        document.getElementById(  "o" + iItem  ).setAttribute(  "tooltiptext",  wfxVersions_dialogOps_localeParamsMod( "szSoloStop",  wfxVersions_dialogVars_itemsList[iItem], oItem )  );
        document.getElementById(  "r" + iItem  ).setAttribute(  "tooltiptext",  wfxVersions_dialogOps_localeParamsMod( "szSoloRLoad", wfxVersions_dialogVars_itemsList[iItem], oItem )  );
        document.getElementById(  "h" + iItem  ).setAttribute(  "tooltiptext",  wfxVersions_dialogOps_localeParamsMod( "szSoloHelp",  wfxVersions_dialogVars_itemsList[iItem], oItem )  );

        var  szScanTxt = wfxVersions_dialogVars_itemsList[iItem].version;
        if(  wfxVersions_dialogVars_itemsList[iItem].status == 5  )
        {
          szScanTxt = wfxVersions_dialogVars_itemsList[iItem].update;
        }
        var  szScanURL = wfxVersions_dialogOps_likeMRT(  oItem.url,  szScanTxt  );
        var  oNode = document.getElementById(  "n" + iItem  );
        oNode.setAttribute(  "value",        oItem.line  );
        oNode.setAttribute(  "tooltiptext",  szScanURL   );

        /*  re-color if valid/invalid URL  */
        oNode.removeAttribute(  "wfxIsError"  );
        if(  !wfxVersions_httpOps_isValidURL( szScanURL )  )
        {
          oNode.setAttribute(  "wfxIsError",  "true"  );
        }

        if(  oItem.down.length != 0  )
        {
          oNode.previousSibling.style.setProperty(  "visibility",  "visible",  "important"  )
        }
        else
        {
          oNode.previousSibling.style.setProperty(  "visibility",  "hidden",  "important"  )
        }
        oNode.previousSibling.setAttribute(  "tooltiptext",  wfxVersions_dialogOps_likeMRT( oItem.down, szScanTxt )  );

        if(  oItem.home.length != 0  )
        {
          oNode.previousSibling.previousSibling.style.setProperty(  "visibility",  "visible",  "important"  )
        }
        else
        {
          oNode.previousSibling.previousSibling.style.setProperty(  "visibility",  "hidden",  "important"  )
        }
        oNode.previousSibling.previousSibling.setAttribute(  "tooltiptext",  wfxVersions_dialogOps_likeMRT( oItem.home, szScanTxt )  );

        /*  enlarge window to show entire content  */
        wfxVersions_dialogOps_resizeNow();

        /*  abort scanning of currently edited item & re-scan with modified parameters  */
        if(  bReScan  )
        {
          if((  szURL != szScanURL  )&&(  wfxVersions_dialogVars_itemsList[iItem].status == 2  ))
          {
            wfxVersions_scanOps_itemAbort(  iItem  );
          }
          wfxVersions_scanOps_itemResume(  iItem  );
        }

        /*  edit completed  */
        return;
      }
      else
      {
        /*  file not written => restore version  */
        wfxVersions_dialogVars_itemsList[iItem].line  = szLine;
        wfxVersions_dialogVars_itemsList[iItem].url   = szURL;
        wfxVersions_dialogVars_itemsList[iItem].dir   = eDir;
        wfxVersions_dialogVars_itemsList[iItem].start = szStart;
        wfxVersions_dialogVars_itemsList[iItem].pre   = szPre;
        wfxVersions_dialogVars_itemsList[iItem].post  = szPost;
        wfxVersions_dialogVars_itemsList[iItem].home  = szHome;
        wfxVersions_dialogVars_itemsList[iItem].down  = szDown;
      }
    }
    else
    {
      /*  no change (edit with same values)  */
      return;
    }
  }
}


/****************************************************************************/
/*! \brief  Configure the .list files from within the extension
**
**  The .lists are actually delay-loaded as not to cause too much load on the
**  application display
**
**  \param[in]  szOps  name of the operation performed, used in error alerts
**
**  \return  --
*/
function  wfxVersions_editOps_start(  szOps  )
{
  /*  same as Cancel changes to preferences without close  */
  wfxVersions_dialogOps_loadPrefs();
  wfxVersions_memoOps_storeSelections(  "extensions.wfx_Versions.editSelections",  "extensions.wfx_Versions.editMemorize"  );

  /*  create a file-object for the .list file opened  */
  var  fsList = wfxVersions_dialogVars_filesList[0];

  /*  create a stream-object for UTF-8 encoding  */
  var  ioFile = Components.classes[ "@mozilla.org/network/file-input-stream;1" ]
                          .createInstance(  Components.interfaces.nsIFileInputStream  );
  ioFile.QueryInterface(  Components.interfaces.nsILineInputStream  );
  ioFile.init(  fsList,  0x01/*read-only*/,  0444,  0  );

  var  ioStream = Components.classes[ "@mozilla.org/intl/converter-input-stream;1" ]
                            .createInstance(  Components.interfaces.nsIConverterInputStream  );
  ioStream.QueryInterface(  Components.interfaces.nsIUnicharLineInputStream  );
  ioStream.init(  ioFile,  "UTF-8",  1024,  0/*force exception on readLine encoding error*/  );

  /*  detect encoding  */
  var  bMoreLines;
  var  ozLine = {};
  try
  {
    do
    {
      bMoreLines = ioStream.readLine(  ozLine  );
    }
    while(  bMoreLines  );

    ioStream.close();
    ioFile.close();

    ioFile.init(  fsList,  0x01/*read-only*/,  0444,  0  );
    ioStream.init(  ioFile,  "UTF-8",  1024,  0  );
  }
  catch(  e  )
  {
    ioStream.close();
    ioFile.close();

    ioStream = ioFile;
    ioStream.init(  fsList,  0x01/*read-only*/,  0444,  0  );
  }

  /*  just in case the user clicks [Edit] while Scan in progress  */
  if(  document.getElementById( "wfxVersions_dialogIDs_tabs" ).childNodes.length > 2  )
  {
    wfxVersions_dialogOps_destroyTabs();
  }
  wfxVersions_dialogVars_itemsList = [];

  document.getElementById(  "wfxVersions_dialogIDs_editEnable"  ).removeAttribute(  "hidden"  );
  document.getElementById(  "wfxVersions_dialogIDs_tabs"  ).firstChild.setAttribute(  "tooltiptext",  fsList.path    );
  document.getElementById(  "wfxVersions_dialogIDs_tabs"  ).firstChild.setAttribute(  "label",        " " + fsList.leafName + " "  );
  document.getElementById(  "wfxVersions_dialogIDs_tabs"  ).firstChild.removeAttribute(  "hidden"  );
  document.getElementById(  "wfxVersions_dialogIDs_tabs"  ).advanceSelectedTab(  1,  true  );
  document.getElementById(  "wfxVersions_dialogIDs_statusBar"  ).value = " " + unescape( "%u2026" ) + "  0  ( " + fsList.leafName + " )";

  do
  {
    /*  parse line-by-line, content is valid if:
    **  - 6 items per line
    **  - non-empty application name & URL
    */
    bMoreLines = ioStream.readLine(  ozLine  );
    if(  wfxVersions_dialogOps_isComment( ozLine.value )  )
    {
      /*  add to items-list as non-item (disabled/hidden)  */
      wfxVersions_dialogVars_itemsList.push(  new wfxVersions_dialogOps_createNonItem( ozLine.value, fsList.path )  );
    }
    else
    {
      var  anItem = new wfxVersions_dialogOps_createItem(  ozLine.value,  fsList.path  );
      anItem.status = 1;
      wfxVersions_dialogVars_itemsList.push(  anItem  );
    }
  }
  while(  bMoreLines  );
  ioStream.close();
  ioFile.close();

  wfxVersions_globalOps_setTimeout(  "wfxVersions_editOps_startDelayed( 0 )",  10  );

  window[ "wfxListsLoading" ] = true;
  openDialog(  "chrome://wfx_versions/content/wfx_versions_progress.xul",
               "",
               "chrome,centerscreen,modal"  );
}


/****************************************************************************/
/*! \brief  Loads the .list item-by-item to reduce the load
**
**  A .list file contains separate lines for each application.
**  Each line has the following content separated by '|' (without quotes):
**  - application name / label
**  - application URL (web page) http:// https:// or ftp://
**  - page-source code point where to start the scan
**    (e.g. if multiple applications from same URL)
**  - page-source code exactly before version string
**  - page-source code exactly after version string
**  - the version
**
**  \param[in]  kItem  the current item being loaded
**
**  \return  --
*/
function  wfxVersions_editOps_startDelayed(  kItem  )
{
  var  oPrefs = Components.classes[ "@mozilla.org/preferences-service;1" ]
                          .getService(  Components.interfaces.nsIPrefBranch  );

  if(  kItem >= wfxVersions_dialogVars_itemsList.length  )
  {
    if(  oPrefs.getBoolPref( "extensions.wfx_Versions.show.alternate.URLs" )  )
    {
      document.getElementById(  "wfxVersions_dialogIDs_editHome"  ).parentNode.removeAttribute(  "hidden"  );
      document.getElementById(  "wfxVersions_dialogIDs_editDown"  ).parentNode.removeAttribute(  "hidden"  );
    }
    else
    {
      document.getElementById(  "wfxVersions_dialogIDs_editHome"  ).parentNode.setAttribute(  "hidden",  "true"  );
      document.getElementById(  "wfxVersions_dialogIDs_editDown"  ).parentNode.setAttribute(  "hidden",  "true"  );
    }

    wfxVersions_dialogVars_progress = 0;
    wfxVersions_dialogOps_addEmpty();
    while(  wfxVersions_dialogVars_itemsList[wfxVersions_dialogVars_progress].status == 6  )
    {
      wfxVersions_dialogVars_progress++;
    }
    wfxVersions_editOps_clickEntry(  wfxVersions_dialogVars_progress  );

    if(  oPrefs.getBoolPref( "extensions.wfx_Versions.oldStyle.Edit.newEntry" )  )
    {
      /*  ¨old style¨ => hide the ¨new style¨ buttons  */
      document.getElementById(  "wfxVersions_dialogIDs_newStyle_newEntry"  ).setAttribute(  "hidden",  "true"  );
      document.getElementById(  "wfxVersions_dialogIDs_editItems"  ).lastChild.removeAttribute(  "hidden"  );
    }
    else
    {
      /*  ¨new style¨ => hide the ¨old style¨ line  */
      document.getElementById(  "wfxVersions_dialogIDs_newStyle_newEntry"  ).removeAttribute(  "hidden"  );
      document.getElementById(  "wfxVersions_dialogIDs_editItems"  ).lastChild.setAttribute(  "hidden",  "true"  );
    }
    
    wfxVersions_dialogOps_setImageAttribute(  document.getElementById( "wfxVersions_dialogIDs_buttonEdit" ),  "chrome://wfx_versions/skin/wfx_versions_edit.gif"  );

    /*  clear memorized Copy selections if "per Edit" is enabled  */
    var  szSelections = Components.classes[ "@mozilla.org/supports-string;1" ]
                                  .createInstance(  Components.interfaces.nsISupportsString  );
    szSelections.data = "";

    var  iOn = oPrefs.getIntPref(  "extensions.wfx_Versions.copyMemorize"  );
    if(  iOn == 1  )
    {
      oPrefs.setComplexValue(  "extensions.wfx_Versions.copySelections",  Components.interfaces.nsISupportsString,  szSelections  );
    }

    window[ "wfxListsLoading" ] = false;
    wfxVersions_editOps_sortList(  document.getElementById( "wfxVersions_dialogIDs_sortAuto" ).checked  );
    return;
  }

  wfxVersions_dialogVars_progress = kItem;
  wfxVersions_dialogVars_itemsList[kItem].editrow = document.getElementById( "wfxVersions_dialogIDs_editItems" ).childNodes.length;

  var  szFile = wfxVersions_fileOps_leafName(  wfxVersions_dialogVars_itemsList[kItem].list  );
  document.getElementById(  "wfxVersions_dialogIDs_statusBar"  ).value = " " + unescape( "%u2026" ) + "  " + kItem + "  ( " + szFile + " )";

  var  szLine = wfxVersions_dialogOps_lineFromItem(  kItem  );
  var  newNode,  newLine,  szAppl;
  if(  wfxVersions_dialogVars_itemsList[kItem].status != 1  )
  {
    if(  oPrefs.getBoolPref( "extensions.wfx_Versions.comments.display" )  )
    {
      szAppl = szLine;
      if(  szAppl.length > 44  )
      {
        szAppl = szAppl.substring(  0,  44  ) + unescape( "%u2026" );
      }

      /*  add a simple label for comments  */
      newLine = document.createElement(  "hbox"  );
      newLine.setAttribute(  "align",     "center"  );
      newLine.setAttribute(  "class",     "wfxVersions_skinBGnd"  );
      newLine.setAttribute(  "wfxIsOdd",  document.getElementById( "wfxVersions_dialogIDs_editItems" ).childNodes.length % 2  );
      document.getElementById(  "wfxVersions_dialogIDs_editItems"  ).appendChild(  newLine  );

      newNode = document.createElement(  "label"  );
      newNode.setAttribute(  "value",        szAppl  );
      newNode.setAttribute(  "tooltiptext",  szLine  );
      newNode.setAttribute(  "onclick",      "wfxVersions_editOps_clickEntry(" + kItem + ")"  );
      newNode.setAttribute(  "onfocus",      "wfxVersions_dialogOps_ensureVisible(parentNode,'wfxVersions_dialogIDs_editItems')"  );
      newNode.setAttribute(  "class",        "wfxVersions_skinLink"  );
      newNode.setAttribute(  "wfxIsError",   "true"  );
      newNode.style.setProperty(  "-moz-margin-end",  "16px",      "important"  );
      newNode.style.setProperty(  "cursor",           "e-resize",  "important"  );
      newLine.appendChild(  newNode  );
    }
  }
  else
  {
    var  szArray = szLine.split(  '|'  );
    var  eDir    = szArray.length - 6;

    szAppl = szArray[eDir%2];
    if(  szAppl.length > 44  )
    {
      szAppl = szAppl.substring(  0,  44  ) + unescape( "%u2026" );
    }

    /*  application name  */
    newLine = document.createElement(  "hbox"  );
    newLine.setAttribute(  "align",     "center"  );
    newLine.setAttribute(  "class",     "wfxVersions_skinBGnd"  );
    newLine.setAttribute(  "wfxIsOdd",  document.getElementById( "wfxVersions_dialogIDs_editItems" ).childNodes.length % 2  );
    document.getElementById(  "wfxVersions_dialogIDs_editItems"  ).appendChild(  newLine  );

    newNode = document.createElement(  "label"  );
    newNode.setAttribute(  "value",        szAppl  );
    newNode.setAttribute(  "tooltiptext",  szLine  );
    newNode.setAttribute(  "onclick",      "wfxVersions_editOps_clickEntry(" + kItem + ")"  );
    newNode.setAttribute(  "onfocus",      "wfxVersions_dialogOps_ensureVisible(parentNode,'wfxVersions_dialogIDs_editItems')"  );
    newNode.setAttribute(  "class",        "wfxVersions_skinLink"  );
    newNode.style.setProperty(  "-moz-margin-end",  "16px",      "important"  );
    newNode.style.setProperty(  "cursor",           "e-resize",  "important"  );
    if(  !wfxVersions_httpOps_isValidURL( wfxVersions_dialogOps_likeMRT(szArray[1+eDir],szArray[5+eDir]) )  )
    {
      newNode.setAttribute(  "wfxIsError",  "true"  );
    }
    newLine.appendChild(  newNode  );
  }

  var  jItem = kItem + 1;
  wfxVersions_globalOps_setTimeout(  "wfxVersions_editOps_startDelayed( " + jItem + " )",  10  );
}


/****************************************************************************/
/*! \brief  Called when the user presses a keyboard button
**
**  Associate ESC to [Cancel], and ENTER to {onclick} event
**
**  \param[in]  eKey  the key-press event
**
**  \return  event further handling allowed or not
*/
function  wfxVersions_dialogOps_keyPress(  eKey  )
{
  var  oTabz = document.getElementById(  "wfxVersions_dialogIDs_tabs"  );

  if(  eKey.keyCode == eKey.DOM_VK_RETURN  )
  {
    /*  ENTER = the {onclick} event  */
    if((  eKey.target  )&&(  eKey.target.nodeName  )&&(  eKey.target.click  ))
    {
      if((  eKey.target.nodeName == "checkbox"  )||(  eKey.target.nodeName == "colorpicker"  ))
      {
        eKey.target.click();
      }
    }
    if((  eKey.target  )&&(  eKey.target.id == "wfxVersions_dialogIDs_pathText"  ))
    {
      wfxVersions_dialogOps_loadPathsCombo(  2  );
    }
    return  false;
  }
  else
  if(  eKey.keyCode == eKey.DOM_VK_ESCAPE  )
  {
    /*  ESC = the [Cancel] button  */
    wfxVersions_globalOps_undoVisibilityPlaces();
    wfxVersions_globalOps_windowClose(  window  );
    return  false;
  }
  else
  if((  eKey.keyCode == eKey.DOM_VK_F4  )||(  eKey.charCode == 'w'.charCodeAt(0)  )||(  eKey.charCode == 'W'.charCodeAt(0)  ))
  {
    /*  Ctrl+W / Ctrl+F4 / Cmd+W / Cmd+F4 = close tab  */
    if((  eKey.altKey == false  )&&(  eKey.shiftKey == false  )&&(  eKey.ctrlKey == true  ))
    {
      /*  if non-Options tab, close it  */
      if(  oTabz.selectedIndex != 1  )
      {
        var  szTip = oTabz.childNodes[ oTabz.selectedIndex ].getAttribute(  "tooltiptext"  );
        if(  oTabz.selectedIndex == 0  )
        {
          szTip = null;
        }
        wfxVersions_dialogOps_closeTab(  szTip  );
        return  false;
      }
    }
  }
  else
  if((  eKey.charCode == 'r'.charCodeAt(0)  )||(  eKey.charCode == 'R'.charCodeAt(0)  ))
  {
    if((  eKey.altKey == false  )&&(  eKey.ctrlKey == true  ))
    {
      var  iItem;
      if(  eKey.shiftKey == false  )
      {
        /*  Ctrl+R = re-scan current .list  */
        if(  oTabz.selectedIndex > 1  )
        {
          var  szTip = oTabz.childNodes[ oTabz.selectedIndex ].getAttribute(  "tooltiptext"  );
          /*  stop current tab (if in progress)  */
          for(  iItem = 0;  iItem < wfxVersions_dialogVars_itemsList.length;  iItem++  )
          {
            if((  wfxVersions_dialogVars_itemsList[iItem].list == szTip  )&&(  wfxVersions_dialogVars_itemsList[iItem].status < 6  ))
            {
              wfxVersions_dialogOps_scanStatus(  iItem,  4  );
            }
          }
          if((  wfxVersions_dialogVars_progress < wfxVersions_dialogVars_itemsList.length  )&&(  wfxVersions_dialogVars_itemsList[wfxVersions_dialogVars_progress].list == szTip  ))
          {
            wfxVersions_scanOps_itemAbort(  wfxVersions_dialogVars_progress  );
          }
          /*  resume current tab  */
          wfxVersions_globalOps_setTimeout(  "wfxVersions_scanOps_reScanOne()",  80  );
          return  false;
        }
      }
      else
      {
        /*  Ctrl+Shift+R = re-scan all .lists  */
        if(  oTabz.childNodes.length > 2  )
        {
          /*  stop all (if not tab closed)  */
          for(  iItem = 0;  iItem < wfxVersions_dialogVars_itemsList.length;  iItem++  )
          {
            if((  wfxVersions_dialogVars_itemsList[iItem].status < 6  )&&(  wfxVersions_dialogVars_itemsList[iItem].status != 3  ))
            {
              wfxVersions_dialogOps_scanStatus(  iItem,  4  );
            }
          }
          if(  wfxVersions_dialogVars_progress < wfxVersions_dialogVars_itemsList.length  )
          {
            wfxVersions_scanOps_itemAbort(  wfxVersions_dialogVars_progress  );
          }
          /*  resume all  */
          wfxVersions_globalOps_setTimeout(  "wfxVersions_scanOps_reScanAll()",  80  );
          return  false;
        }
      }
    }
  }
  else
  if(  eKey.keyCode == eKey.DOM_VK_UP  )
  {
    /*  Alt+Up as alternative for Move[-]  */
    if((  eKey.altKey == true  )&&(  eKey.shiftKey == false  )&&(  eKey.ctrlKey == false  ))
    {
      if(  oTabz.selectedIndex == 0  )  /*  Edit-mode only  */
      {
        var  oButton = document.getElementById(  "wfxVersions_dialogIDs_editMoveUp"  );
        if((  oButton.getAttribute( "disabled" ) != "true"  )&&(  oButton.style.getPropertyValue( "visibility" ) != "hidden"  ))
        {
          eKey.preventDefault();
          wfxVersions_editOps_moveEntry(  oButton.getAttribute( "tooltiptext" ),  -1,  true  );
          wfxVersions_editOps_sortList(  document.getElementById( "wfxVersions_dialogIDs_sortAuto" ).checked  );
        }
        return  false;
      }
    }
  }
  else
  if(  eKey.keyCode == eKey.DOM_VK_DOWN  )
  {
    /*  Alt+Down as alternative for Move[+]  */
    if((  eKey.altKey == true  )&&(  eKey.shiftKey == false  )&&(  eKey.ctrlKey == false  ))
    {
      if(  oTabz.selectedIndex == 0  )  /*  Edit-mode only  */
      {
        var  oButton = document.getElementById(  "wfxVersions_dialogIDs_editMoveDown"  );
        if((  oButton.getAttribute( "disabled" ) != "true"  )&&(  oButton.style.getPropertyValue( "visibility" ) != "hidden"  ))
        {
          eKey.preventDefault();
          wfxVersions_editOps_moveEntry(  oButton.getAttribute( "tooltiptext" ),  +1,  true  );
          wfxVersions_editOps_sortList(  document.getElementById( "wfxVersions_dialogIDs_sortAuto" ).checked  );
        }
        return  false;
      }
    }
  }
  else
  if(  eKey.keyCode == eKey.DOM_VK_LEFT  )
  {
    /*  Alt+Left as alternative for Insert[Before]  */
    if((  eKey.altKey == true  )&&(  eKey.shiftKey == false  )&&(  eKey.ctrlKey == false  ))
    {
      if(  oTabz.selectedIndex == 0  )  /*  Edit-mode only  */
      {
        var  oButton = document.getElementById(  "wfxVersions_dialogIDs_editInsertBefore"  );
        var  oNewStyle = document.getElementById(  "wfxVersions_dialogIDs_newStyle_newEntry"  );
        if((  oNewStyle.getAttribute( "hidden" ) != "true"  )&&(  oButton.style.getPropertyValue( "visibility" ) != "hidden"  ))
        {
          eKey.preventDefault();
          wfxVersions_editOps_insertNew(  oButton.getAttribute( "tooltiptext" ),  -1  );
          return  false;
        }
      }
    }
  }
  else
  if(  eKey.keyCode == eKey.DOM_VK_RIGHT  )
  {
    /*  Alt+Right as alternative for Insert[After]  */
    if((  eKey.altKey == true  )&&(  eKey.shiftKey == false  )&&(  eKey.ctrlKey == false  ))
    {
      if(  oTabz.selectedIndex == 0  )  /*  Edit-mode only  */
      {
        var  oButton = document.getElementById(  "wfxVersions_dialogIDs_editInsertAfter"  );
        var  oNewStyle = document.getElementById(  "wfxVersions_dialogIDs_newStyle_newEntry"  );
        if((  oNewStyle.getAttribute( "hidden" ) != "true"  )&&(  oButton.style.getPropertyValue( "visibility" ) != "hidden"  ))
        {
          eKey.preventDefault();
          wfxVersions_editOps_insertNew(  oButton.getAttribute( "tooltiptext" ),  -0  );
          return  false;
        }
      }
    }
  }
  else
  {}

  return  true;
}


/****************************************************************************/
/*! \brief  launches a Re-Scan All Tabs operation
**
**  \param  --
**
**  \return  --
*/
function  wfxVersions_scanOps_reScanAll()
{
  var  iItem;
  for(  iItem = 0;  iItem < wfxVersions_dialogVars_itemsList.length;  iItem++  )
  {
    if(  wfxVersions_dialogVars_itemsList[iItem].status == 4  )
    {
      wfxVersions_dialogOps_scanStatus(  iItem,  0  );
      wfxVersions_dialogVars_itemsList[iItem].step = 0;
    }
  }
  wfxVersions_dialogOps_throbberScan();
  wfxVersions_dialogOps_setImageAttribute(  document.getElementById( "wfxVersions_dialogIDs_buttonScan" ),  "chrome://wfx_versions/skin/wfx_versions_throbber.gif"  );
  wfxVersions_globalOps_setTimeout(  "wfxVersions_httpOps_headLaunch()",  40  );
}


/****************************************************************************/
/*! \brief  launches a Re-Scan Current Tab operation
**
**  \param  --
**
**  \return  --
*/
function  wfxVersions_scanOps_reScanOne()
{
  var  oTabz = document.getElementById(  "wfxVersions_dialogIDs_tabs"  );
  var  szTip = oTabz.childNodes[ oTabz.selectedIndex ].getAttribute(  "tooltiptext"  );
  var  iItem,  bResume = true;
  for(  iItem = 0;  iItem < wfxVersions_dialogVars_itemsList.length;  iItem++  )
  {
    if((  wfxVersions_dialogVars_itemsList[iItem].list == szTip  )&&(  wfxVersions_dialogVars_itemsList[iItem].status == 4  ))
    {
      wfxVersions_dialogOps_scanStatus(  iItem,  0  );
      wfxVersions_dialogVars_itemsList[iItem].step = 0;
      if(  bResume  )
      {
        wfxVersions_scanOps_itemResume(  iItem  );
        bResume = false;
      }
    }
  }
}


/****************************************************************************/
/*! \brief  enable/disable the show-on-toolbar checkbox
**
**  depends on toolbar button actually displayed on toolbar
**
**  \param  --
**
**  \return  --
*/
function  wfxVersions_dialogOps_disableShowToolbar()
{
  var  bDisable = true;
  var  oEnum = Components.classes[ "@mozilla.org/appshell/window-mediator;1" ]
                         .getService(  Components.interfaces.nsIWindowMediator  )
                         .getEnumerator(  "navigator:browser"  );
  while(  oEnum.hasMoreElements()  )
  {
    var  oBrowser = oEnum.getNext();
    if(  oBrowser.document.getElementById( "wfxVersions_overlayIDs_toolsBar" ) != null  )
    {
      bDisable = false;
      break;
    }
  }

  var  oItem = document.getElementById(  "wfxVersions_dialogIDs_showToolsBar"  );
  wfxVersions_globalOps_recursiveDisable(  oItem,  bDisable  );

  if(  !oItem.checked  )
  {
    bDisable = true;
  }
  oItem = oItem.nextSibling;
  wfxVersions_globalOps_recursiveDisable(  oItem,  bDisable  );
}


/****************************************************************************/
/*! \brief  Window resize DELAYED event handler
**
**  sizeToContent() is used to enlarge the window if it becomes too small
**
**  \param  --
**
**  \return  --
*/
function  wfxVersions_dialogOps_resizeDelayed()
{
  /*  memorize old window dimensions (for resize if needed)  */
  var  iWidth  = innerWidth;
  var  iHeight = innerHeight;

  /*  enlarge to fit contents  */
  sizeToContent();

  /*  if it was a reduce instead of enlarge, resize back  */
  if((  innerWidth < iWidth  )||(  innerHeight < iHeight  ))
  {
    iWidth--;  /*  workaround for a tiny 1px bug in FF's sizeToContent()  */

    document.getElementById( "wfx_versions_window" ).style.setProperty(  "min-width",   iWidth  + "px",  "important"  );
    document.getElementById( "wfx_versions_window" ).style.setProperty(  "min-height",  iHeight + "px",  "important"  );

    sizeToContent();

    document.getElementById( "wfx_versions_window" ).style.removeProperty(  "min-width"  );
    document.getElementById( "wfx_versions_window" ).style.removeProperty(  "min-height"  );
  }

  /*  adjust background image gradient (if any)  */
  var  posX = document.getElementById(  "wfxVersions_dialogIDs_tabsSpacerLeft"  ).boxObject.width;
  document.getElementById(  "wfxVersions_dialogIDs_tabsSpacerRight"  ).style.setProperty(  "background-position",  "-" + posX + "px 0px",  "important"  );

  /*  adjust scan columns (if any)  */
  wfxVersions_dialogOps_adjustScanLines();

  /*  some themes cover the content when scroll:auto becomes visible (RTL)  */
  var  xPanels = document.getElementById(  "wfxVersions_dialogIDs_panels"  );
  for(  var  i = 2;  i < xPanels.childNodes.length;  i++  )
  {
    xPanels.childNodes[i].firstChild.firstChild.style.setProperty(  "overflow-y",  "auto",  "important"  );
  }

  /*  resize completed  */
  wfxVersions_globalOps_setTimeout(  "wfxVersions_dialogVars_resizeInProgress=false",  400  );
}


/****************************************************************************/
/*! \brief  Window resize event handler
**
**  Used for setting edit-boxes tooltips from label texts
**
**  \param  --
**
**  \return  --
*/
function  wfxVersions_dialogOps_resizeNow()
{
  /*  avoid multiple activation
  **  (sizeToContent used in wfxVersions_dialogOps_resizeDelayed)
  */
  if(  wfxVersions_dialogVars_resizeInProgress  )
  {
    return;
  }

  /*  show/hide the resizer box  */
  if(  windowState != STATE_NORMAL  )
  {
    if(  windowState == STATE_MAXIMIZED  )
    {
      document.getElementById(  "wfxVersions_dialogIDs_resizer"  ).setAttribute(  "hidden",  "true"  );
    }
    return;
  }
  wfxVersions_dialogVars_resizeInProgress = true;
  document.getElementById(  "wfxVersions_dialogIDs_resizer"  ).removeAttribute(  "hidden"  );

  /*  max sizes depending on screen sizes  */
  var  iWidth  = (screen.availWidth  - (screenX - wfxVersions_dialogVars_resizeOrigin.X)) - (outerWidth  - innerWidth);
  var  iHeight = (screen.availHeight - (screenY - wfxVersions_dialogVars_resizeOrigin.Y)) - (outerHeight - innerHeight);

  document.getElementById(  "wfx_versions_window"  ).style.setProperty(  "max-width",   iWidth  + "px",  "important"  );
  document.getElementById(  "wfx_versions_window"  ).style.setProperty(  "max-height",  iHeight + "px",  "important"  );

  /*  limit scan columns (if any)  */
  wfxVersions_dialogOps_limitScanLines();

  /*  resize in progress  */
  wfxVersions_globalOps_setTimeout(  "wfxVersions_dialogOps_resizeDelayed()",  40  );
}


/****************************************************************************/
/*! \brief  Shrinks back the window after having been enlarged by Scan/Edit
**
**  \param  --
**
**  \return  --
*/
function  wfxVersions_dialogOps_resizeBack()
{
  document.getElementById(  "wfx_versions_window"  ).style.setProperty(  "max-width",   "0px",  "important"  );
  document.getElementById(  "wfx_versions_window"  ).style.setProperty(  "max-height",  "0px",  "important"  );

  sizeToContent();
}


/****************************************************************************/
/*! \brief  Get the coordinates of the screen origin
**
**  Normally they should be {0,0}, but Firefox 3 implemented negative coords
**  for secondary monitors.
**
**  \see  wfxVersions_dialogVars_resizeOrigin
*/
function  wfxVersions_dialogOps_resizeOrigins()
{
  /**  \todo
  **          How do I find out the top-left coords of the current monitor?
  */
}


/****************************************************************************/
/*! \brief  Enable/disable the Scan/Edit buttons when the path changes
**
**  The path can change on ENTER in the textbox area or after Browse for path
**
**  Scan/Edit cannot be launched until we actually Apply the path change, and
**  it doesn't make sense to interrupt current Scan in progress just for that
**
**  \param  --
**
**  \return  --
*/
function  wfxVersions_dialogOps_onPathChange()
{
  document.getElementById(  "wfxVersions_dialogIDs_buttonScan"  ).removeAttribute(  "disabled"  );
  document.getElementById(  "wfxVersions_dialogIDs_buttonEdit"  ).removeAttribute(  "disabled"  );

  var  oCombo = document.getElementById(  "wfxVersions_dialogIDs_pathText"  );
  if(  oCombo.value  )
  {
    var  szStoredPath = Components.classes[ "@mozilla.org/preferences-service;1" ]
                                  .getService(  Components.interfaces.nsIPrefBranch  )
                                  .getComplexValue(  "extensions.wfx_Versions.tcPluginPath",  Components.interfaces.nsISupportsString  )
                                  .data;

    /*  special treatment for the [Clean History] entry  */
    if((  oCombo.menupopup  )&&(  oCombo.menupopup.childNodes  )&&(  oCombo.menupopup.childNodes.length == (1 + oCombo.selectedIndex)  ))
    {
      oCombo.value = "*";
      wfxVersions_dialogOps_loadPathsCombo(  1  );
      oCombo.value = szStoredPath;
    }

    /*  disable buttons until textbox path is stored  */
    if(  oCombo.value != szStoredPath  )
    {
      document.getElementById(  "wfxVersions_dialogIDs_buttonScan"  ).setAttribute(  "disabled",  "true"  );
      document.getElementById(  "wfxVersions_dialogIDs_buttonEdit"  ).setAttribute(  "disabled",  "true"  );
    }
  }
}


/****************************************************************************/
/*! \brief  Load/restore the paths history for the drop-down list
**
**  Duplicate entries are eliminated, and the edit-box paths are appended
**
**  The paths strings are validated, and invalid strings are removed
**
**  \param[in]  iValidation  return of wfxVersions_fileOps_isValidPathString()
**
**  \return  --
*/
function  wfxVersions_dialogOps_loadPathsCombo(  iValidation  )
{
  var  oPrefs = Components.classes[ "@mozilla.org/preferences-service;1" ]
                          .getService(  Components.interfaces.nsIPrefBranch  );

  var  szPathCurrent = document.getElementById(  "wfxVersions_dialogIDs_pathText"  ).value;
  var  ozPathStored = oPrefs.getComplexValue(  "extensions.wfx_Versions.tcPluginPath",
                                               Components.interfaces.nsISupportsString  );
  var  ozPathsCombox = oPrefs.getComplexValue(  "extensions.wfx_Versions.tcPluginPaths.collect",
                                                Components.interfaces.nsISupportsString  );
  var  azOldCollection = ozPathsCombox.data.split(  "*"  );
  var  azNewCollection = [];

  /*  append current textbox path to history  */
  if(  wfxVersions_fileOps_isValidPathString( szPathCurrent ) < iValidation  )
  {
    azNewCollection.push(  szPathCurrent  );
  }

  /*  clean-up duplicate history entries  */
  var  iRow,  jRow,  bAppend;
  for(  iRow = 0;  iRow < azOldCollection.length;  iRow++  )
  {
    if(  wfxVersions_fileOps_isValidPathString( azOldCollection[iRow] ) < iValidation  )
    {
      bAppend = true;
      for(  jRow = 0;  jRow < azNewCollection.length;  jRow++  )
      {
        if(  azOldCollection[iRow] == azNewCollection[jRow]  )
        {
          bAppend = false;
        }
      }
      if(  bAppend  )
      {
        azNewCollection.push(  azOldCollection[iRow]  );
      }
    }
  }

  /*  always append current stored path to history (without validation)  */
  bAppend = true;
  for(  jRow = 0;  jRow < azNewCollection.length;  jRow++  )
  {
    if(  ozPathStored.data == azNewCollection[jRow]  )
    {
      bAppend = false;
    }
  }
  if(  bAppend  )
  {
    azNewCollection.splice(  0,  0,  ozPathStored.data  );
  }

  /*  update the paths collection, in preferences and on dialog-box  */
  var  szPathsCollection = Components.classes[ "@mozilla.org/supports-string;1" ]
                                     .createInstance(  Components.interfaces.nsISupportsString  );
  szPathsCollection.data = azNewCollection.join(  "*"  );
  oPrefs.setComplexValue(  "extensions.wfx_Versions.tcPluginPaths.collect",
                           Components.interfaces.nsISupportsString,
                           szPathsCollection  );

  document.getElementById(  "wfxVersions_dialogIDs_pathText"  ).removeAllItems();
  for(  jRow = 0;  jRow < azNewCollection.length;  jRow++  )
  {
    var  oItem = document.getElementById(  "wfxVersions_dialogIDs_pathText"  ).appendItem(  azNewCollection[jRow]  );

    /*  highlight the currently stored path  */
    if(  azNewCollection[jRow] == ozPathStored.data  )
    {
      var  szItalic = "italic";
      if(  szItalic == getComputedStyle( oItem, null ).getPropertyValue( "font-style" )  )
      {
        szItalic = "normal";
      }
      oItem.style.setProperty(  "font-style",  szItalic,  "important"  );
    }
  }
  document.getElementById(  "wfxVersions_dialogIDs_pathText"  ).appendItem(  wfxVersions_dialogOps_localeParams( "szCleanHist" )  )
                                                               .style.setProperty(  "text-align",  "center",  "important"  );
  document.getElementById(  "wfxVersions_dialogIDs_pathText"  ).value = szPathCurrent;

  if(  iValidation > 1  )
  {
    wfxVersions_dialogOps_onPathChange();
  }
}


/****************************************************************************/
/*! \brief  Called when the preferences window loads
**
**  Used for setting edit-boxes tooltips from label texts
**
**  \param  --
**
**  \return  --
*/
function  wfxVersions_dialogOps_onLoad()
{
  /*  <panel> as a popup only works in Firefox 3  */
  var  xInfo = Components.classes[ "@mozilla.org/xre/app-info;1" ]
                         .getService(  Components.interfaces.nsIXULAppInfo  );
  var  xVersionChk = Components.classes[ "@mozilla.org/xpcom/version-comparator;1" ]
                               .getService(  Components.interfaces.nsIVersionComparator  );
  if(  xVersionChk.compare( xInfo.platformVersion, "1.9a1" ) >= 0   )
  {
    wfxVersions_dialogVars_Fx3 = true;
  }

  /*  detect if opened from addons options area
  **  force re-open as non-modal window (especially on Win32)
  */
  if((  !( "overlay" in window )  )||(  !window["overlay"]  ))
  {
    /*  open a resizable/minimizable window  */
    var  oBrowser = Components.classes[ "@mozilla.org/appshell/window-mediator;1" ]
                              .getService(  Components.interfaces.nsIWindowMediator  )
                              .getEnumerator(  "navigator:browser"  )
                              .getNext();
    oBrowser.wfxVersions_globalOps_setTimeout(  "wfxVersions_overlayOps_launchChrome('addons')",  120  );

    /*  close the current modal window  */
    wfxVersions_globalOps_windowClose(  window  );
    return;
  }

  wfxVersions_dialogOps_loadPrefs();
  var  oPrefs = Components.classes[ "@mozilla.org/preferences-service;1" ]
                          .getService(  Components.interfaces.nsIPrefBranch  );

  if(  oPrefs.getBoolPref( "extensions.wfx_Versions.expandedOps.path" )  )
  {
    document.getElementById(  "wfxVersions_dialogIDs_expandPath"  ).click();
  }
  if(  oPrefs.getBoolPref( "extensions.wfx_Versions.expandedOps.timeout" )  )
  {
    document.getElementById(  "wfxVersions_dialogIDs_expandTimeout"  ).click();
  }
  if(  oPrefs.getBoolPref( "extensions.wfx_Versions.expandedOps.display" )  )
  {
    document.getElementById(  "wfxVersions_dialogIDs_expandShows"  ).click();
  }
  if(  oPrefs.getBoolPref( "extensions.wfx_Versions.expandedOps.colors" )  )
  {
    document.getElementById(  "wfxVersions_dialogIDs_expandColors"  ).click();
  }
  if(  oPrefs.getBoolPref( "extensions.wfx_Versions.expandedOps.memorize" )  )
  {
    document.getElementById(  "wfxVersions_dialogIDs_expandMemos"  ).click();
  }
  if(  oPrefs.getBoolPref( "extensions.wfx_Versions.Edit.sort.caps" )  )
  {
    document.getElementById(  "wfxVersions_dialogIDs_sortCaps"  ).checked = true;
  }
  if(  oPrefs.getBoolPref( "extensions.wfx_Versions.Edit.sort.reversed" )  )
  {
    document.getElementById(  "wfxVersions_dialogIDs_sortReversed"  ).checked = true;
  }
  if(  oPrefs.getBoolPref( "extensions.wfx_Versions.Edit.sort.auto" )  )
  {
    document.getElementById(  "wfxVersions_dialogIDs_sortAuto"  ).checked = true;
  }

  wfxVersions_memoOps_onLoad();
  wfxVersions_globalOps_loadConfigColors();
  wfxVersions_dialogOps_storePrefs();
  wfxVersions_globalOps_styleInt(  document.getElementById( "wfxVersions_dialogIDs_delayText" )  );
  var  aLabels = document.getElementsByTagName(  "label"  );
  for(  var  i = 0;  i < aLabels.length;  i++  )
  {
    if(  aLabels[i].id.match( /^wfxVersions_dialogIDs_expand/ )  )
    {
      aLabels[i].parentNode.parentNode.style.setProperty(  "padding",  "0px",  "important"  );
    }
  }

  if(  !oPrefs.getBoolPref( "extensions.wfx_Versions.show.addIcons2Labels" )  )
  {
    window["noicons"] = true;
    wfxVersions_globalOps_removeIcons(  document.documentElement  );
  }
  if(  !oPrefs.getBoolPref( "extensions.wfx_Versions.show.tabButton.Arrows" )  )
  {
    document.getElementById(  "wfxVersions_dialogIDs_tabsPrev"  ).setAttribute(  "hidden",  "true"  );
    document.getElementById(  "wfxVersions_dialogIDs_tabsNext"  ).setAttribute(  "hidden",  "true"  );
  }
  if(  !oPrefs.getBoolPref( "extensions.wfx_Versions.show.tabButton.Close" )  )
  {
    document.getElementById(  "wfxVersions_dialogIDs_tabsClose"  ).setAttribute(  "hidden",  "true"  );
  }

  if(  oPrefs.getBoolPref( "extensions.wfx_Versions.expanded.toolsmenu" )  )
  {
    document.getElementById(  "wfxVersions_dialogIDs_startToolsMenu"  ).parentNode.style.setProperty(  "visibility",  "hidden",  "important"  );
  }

  /*  display simulated status-bar (initial empty string => not visible)  */
  document.getElementById( "wfxVersions_dialogIDs_statusBar" ).value = " ";

  /*  adjustments...  */
  wfxVersions_beautifyOps_editRedraw();
  wfxVersions_beautifyOps_scrollAdjust();

  var  szRight = getComputedStyle(  document.getElementById( "wfxVersions_dialogIDs_buttonScan" ).parentNode,  null  ).getPropertyValue(  "margin-right"  );
  document.getElementById(  "wfxVersions_dialogIDs_boxAbout"  ).style.setProperty(  "margin-right",  szRight,  "important"  );
  szRight = getComputedStyle(  document.getElementById( "wfxVersions_dialogIDs_buttonScan" ).parentNode,  null  ).getPropertyValue(  "padding-right"  );
  document.getElementById(  "wfxVersions_dialogIDs_boxAbout"  ).style.setProperty(  "padding-right",  szRight,  "important"  );

  var  szLeft = getComputedStyle(  document.getElementById( "wfxVersions_dialogIDs_buttonScan" ).parentNode,  null  ).getPropertyValue(  "padding-left"  );
  document.getElementById(  "wfxVersions_dialogIDs_boxAbout"  ).style.setProperty(  "padding-left",  szLeft,  "important"  );

  /*  the Edit tab is not hidden by default to show it's buttons properly  */
  document.getElementById( "wfxVersions_dialogIDs_tabs" ).advanceSelectedTab(  1,  true  );
  document.getElementById( "wfxVersions_dialogIDs_tabs" ).firstChild.setAttribute(  "hidden",  "true"  );

  wfxVersions_dialogOps_throbberIdle();

  wfxVersions_dialogOps_resizeOrigins();
  window.onresize = wfxVersions_dialogOps_resizeNow;
  addEventListener(  "focus",  wfxVersions_dialogOps_disableShowToolbar,    false  );
  addEventListener(  "close",  wfxVersions_globalOps_undoVisibilityPlaces,  false  );

  wfxVersions_globalOps_setTimeout(  "wfxVersions_beautifyOps_redShift()",  40  );
  wfxVersions_globalOps_setTimeout(  "wfxVersions_beautifyOps_darkRevisited()",  40  );

  /*  select operation-on-launch, default = Options  */
  var  iOps = 0;
  if(  window["overlay"] == "statusbar"  )
  {
    /*  launched from status-bar  */
    iOps = oPrefs.getIntPref(  "extensions.wfx_Versions.start.statusbar"  );
  }
  else
  if(  window["overlay"] == "toolbar"  )
  {
    /*  launched from toolbar button  */
    iOps = oPrefs.getIntPref(  "extensions.wfx_Versions.start.toolbar"  );
  }
  else
  if(  window["overlay"] == "menu"  )
  {
    /*  launched from Tools menu  */
    iOps = oPrefs.getIntPref(  "extensions.wfx_Versions.start.toolsmenuitem"  );
  }
  else
  if(  window["overlay"] == "options"  )
  {
    /*  forced Options open  */
  }
  else
  if(  window["overlay"] == "scan"  )
  {
    /*  forced Scan open  */
    iOps = 1;
  }
  else
  if(  window["overlay"] == "scan-all"  )
  {
    /*  forced Scan-All-Lists open  */
    document.getElementById(  "wfxVersions_dialogIDs_allScan"  ).checked = true;
    iOps = 1;
  }
  else
  if(  window["overlay"] == "scan-selected"  )
  {
    /*  forced Scan-Select-Lists open  */
    document.getElementById(  "wfxVersions_dialogIDs_allScan"  ).checked = false;
    iOps = 1;
  }
  else
  if(  window["overlay"] == "edit"  )
  {
    /*  forced Edit open  */
    iOps = 2;
  }
  else
  {
    /*  always open Options from addons area  */
  }

  document.getElementById(  "wfxVersions_dialogIDs_editEnable"  ).setAttribute(  "hidden",  "true"  );
  wfxVersions_globalOps_setTimeout(  "if(windowState!=STATE_MAXIMIZED) wfxVersions_dialogOps_resizeNow();",  10  );
  switch(  iOps  )
  {
    default:  /*  Options  */
    break;

    case  1:  /*  Scan  */
      wfxVersions_globalOps_setTimeout(  "wfxVersions_dialogOps_startScan()",  120  );
    break;

    case  2:  /*  Edit  */
      wfxVersions_globalOps_setTimeout(  "wfxVersions_dialogOps_startEdit()",  120  );
    break;
  }
}


/****************************************************************************/
/*! \brief  Correction for label text-color on RedShiftV2 theme
**
**  The default text color for \e label is black, same as the window/dialog
**  background-color on dark themes.
**
**  \param  --
**
**  \return  --
*/
function  wfxVersions_beautifyOps_redShift()
{
  var  oStatus = document.getElementById(  "wfxVersions_dialogIDs_statusBar"  );
  var  szTextColor = getComputedStyle(  oStatus,  null  ).getPropertyValue(  "color"  );
  var  szBackColor = getComputedStyle(  oStatus,  null  ).getPropertyValue(  "background-color"  );

  if(  szTextColor != "transparent"  )
  {
    while((  szBackColor == "transparent"  )&&(  oStatus.parentNode != null  )&&(  oStatus.parentNode != oStatus  ))
    {
      oStatus = oStatus.parentNode;
      szBackColor = getComputedStyle(  oStatus,  null  ).getPropertyValue(  "background-color"  );
    }
  }
  if(  szBackColor == "transparent"  )
  {
    szBackColor = "rgb( 255, 255, 255 )";
  }

  if(  szTextColor == szBackColor  )
  {
    var  iRed   = parseInt(  szBackColor.replace( / /g, "" ).split( '('                )[1].split( ',' )[0]  );
    var  iGreen = parseInt(  szBackColor.replace( / /g, "" ).split( "(" + iRed   + "," )[1].split( ',' )[0]  );
    var  iBlue  = parseInt(  szBackColor.replace( / /g, "" ).split( "," + iGreen + "," )[1].split( ',' )[0]  );

    /*  turn the invisible-ink visible  */
    if(  (iRed + iGreen + iBlue) >= (128 + 128 + 128)  )
    {
      document.getElementById(  "wfxVersions_dialogIDs_statusBar"  ).style.setProperty(  "color",  "rgb( 0, 0, 0 )",  "important"  );
    }
    else
    {
      document.getElementById(  "wfxVersions_dialogIDs_statusBar"  ).style.setProperty(  "color",  "rgb( 255, 255, 255 )",  "important"  );
    }
  }
}


/****************************************************************************/
/*! \brief  Correction for panel background-color on Dark Revisited theme
**
**  The background color for \e panel is transparent,
**  making the floating panel invisible.
**
**  \param  --
**
**  \return  --
*/
function  wfxVersions_beautifyOps_darkRevisited()
{
  var  oPanel = document.getElementById(  "wfxVersions_dialogIDs_filesSelector"  );
  var  szTextColor = getComputedStyle(  oPanel,  null  ).getPropertyValue(  "color"  );
  var  szBackColor = getComputedStyle(  oPanel,  null  ).getPropertyValue(  "background-color"  );

  if(  szBackColor == "transparent"  )
  {
    if(  szTextColor == "transparent"  )
    {
      szTextColor = "rgb( 0, 0, 0 )";
    }
    var  iRed   = parseInt(  szTextColor.replace( / /g, "" ).split( '('                )[1].split( ',' )[0]  );
    var  iGreen = parseInt(  szTextColor.replace( / /g, "" ).split( "(" + iRed   + "," )[1].split( ',' )[0]  );
    var  iBlue  = parseInt(  szTextColor.replace( / /g, "" ).split( "," + iGreen + "," )[1].split( ',' )[0]  );

    /*  turn the invisible-page visible  */
    if(  (iRed + iGreen + iBlue) >= (128 + 128 + 128)  )
    {
      oPanel.style.setProperty(  "background-color",  "rgb( 0, 0, 0 )",  "important"  );
    }
    else
    {
      oPanel.style.setProperty(  "background-color",  "rgb( 255, 255, 255 )",  "important"  );
    }
  }
}


/****************************************************************************/
/*! \brief  Adjust widths/margins/etc. for Edit mode
**
**  \param  --
**
**  \return  --
*/
function  wfxVersions_beautifyOps_editRedraw()
{
  document.getElementById(  "wfxVersions_dialogIDs_editVersion"  ).inputField.style.setProperty(  "background",  "none transparent",  "important"  );

  /**  set progressmeter margin equal to textbox margin  */
  var  iCopyTop    = getComputedStyle(  document.getElementById( "wfxVersions_dialogIDs_editVersion" ),  null  ).getPropertyValue(  "margin-top"  );
  var  iCopyLeft   = getComputedStyle(  document.getElementById( "wfxVersions_dialogIDs_editVersion" ),  null  ).getPropertyValue(  "margin-left"  );
  var  iCopyBottom = getComputedStyle(  document.getElementById( "wfxVersions_dialogIDs_editVersion" ),  null  ).getPropertyValue(  "margin-bottom"  );
  var  iCopyRight  = getComputedStyle(  document.getElementById( "wfxVersions_dialogIDs_editVersion" ),  null  ).getPropertyValue(  "margin-right"  );
  document.getElementById(  "wfxVersions_dialogIDs_editProgress"  ).style.setProperty(  "margin-top",     iCopyTop,     "important"  );
  document.getElementById(  "wfxVersions_dialogIDs_editProgress"  ).style.setProperty(  "margin-left",    iCopyLeft,    "important"  );
  document.getElementById(  "wfxVersions_dialogIDs_editProgress"  ).style.setProperty(  "margin-bottom",  iCopyBottom,  "important"  );
  document.getElementById(  "wfxVersions_dialogIDs_editProgress"  ).style.setProperty(  "margin-right",   iCopyRight,   "important"  );

  document.getElementById(  "wfxVersions_dialogIDs_editForwards"  ).style.setProperty(  "margin-top",     iCopyTop,     "important"  );
  document.getElementById(  "wfxVersions_dialogIDs_editForwards"  ).style.setProperty(  "margin-left",    iCopyLeft,    "important"  );
  document.getElementById(  "wfxVersions_dialogIDs_editForwards"  ).style.setProperty(  "margin-bottom",  iCopyBottom,  "important"  );
  document.getElementById(  "wfxVersions_dialogIDs_editForwards"  ).style.setProperty(  "margin-right",   iCopyRight,   "important"  );

  /**  adjust colors for the `see tutorial` link  */
  document.getElementById(  "wfxVersions_dialogIDs_editTutorial"  ).style.setProperty(  "margin-top",        iCopyTop,                              "important"  );
  document.getElementById(  "wfxVersions_dialogIDs_editTutorial"  ).style.setProperty(  "margin-left",       iCopyLeft,                             "important"  );
  document.getElementById(  "wfxVersions_dialogIDs_editTutorial"  ).style.setProperty(  "margin-bottom",     iCopyBottom,                           "important"  );
  document.getElementById(  "wfxVersions_dialogIDs_editTutorial"  ).style.setProperty(  "margin-right",      iCopyRight,                            "important"  );

  document.getElementById(  "wfxVersions_dialogIDs_editClickHome"  ).style.setProperty(  "margin-top",        iCopyTop,                              "important"  );
  document.getElementById(  "wfxVersions_dialogIDs_editClickHome"  ).style.setProperty(  "margin-left",       iCopyLeft,                             "important"  );
  document.getElementById(  "wfxVersions_dialogIDs_editClickHome"  ).style.setProperty(  "margin-bottom",     iCopyBottom,                           "important"  );
  document.getElementById(  "wfxVersions_dialogIDs_editClickHome"  ).style.setProperty(  "margin-right",      iCopyRight,                            "important"  );

  document.getElementById(  "wfxVersions_dialogIDs_editClickDown"  ).style.setProperty(  "margin-top",        iCopyTop,                              "important"  );
  document.getElementById(  "wfxVersions_dialogIDs_editClickDown"  ).style.setProperty(  "margin-left",       iCopyLeft,                             "important"  );
  document.getElementById(  "wfxVersions_dialogIDs_editClickDown"  ).style.setProperty(  "margin-bottom",     iCopyBottom,                           "important"  );
  document.getElementById(  "wfxVersions_dialogIDs_editClickDown"  ).style.setProperty(  "margin-right",      iCopyRight,                            "important"  );

  document.getElementById(  "wfxVersions_dialogIDs_editClickURL"  ).style.setProperty(  "margin-top",        iCopyTop,                              "important"  );
  document.getElementById(  "wfxVersions_dialogIDs_editClickURL"  ).style.setProperty(  "margin-left",       iCopyLeft,                             "important"  );
  document.getElementById(  "wfxVersions_dialogIDs_editClickURL"  ).style.setProperty(  "margin-bottom",     iCopyBottom,                           "important"  );
  document.getElementById(  "wfxVersions_dialogIDs_editClickURL"  ).style.setProperty(  "margin-right",      iCopyRight,                            "important"  );

  /**  make Move buttons have same width  */
  var  iPgUp = document.getElementById(  "wfxVersions_dialogIDs_editMoveUp"    ).boxObject.width;
  var  iPgDn = document.getElementById(  "wfxVersions_dialogIDs_editMoveDown"  ).boxObject.width;
  document.getElementById(  "wfxVersions_dialogIDs_editMoveUp"    ).style.setProperty(  "min-width",  (iPgUp>iPgDn)?iPgUp:iPgDn + "px",  "important"  );
  document.getElementById(  "wfxVersions_dialogIDs_editMoveDown"  ).style.setProperty(  "min-width",  (iPgUp>iPgDn)?iPgUp:iPgDn + "px",  "important"  );

  iPgUp = document.getElementById(  "wfxVersions_dialogIDs_editInsertBefore"  ).boxObject.width;
  iPgDn = document.getElementById(  "wfxVersions_dialogIDs_editInsertAfter"   ).boxObject.width;
  document.getElementById(  "wfxVersions_dialogIDs_editInsertBefore"  ).style.setProperty(  "min-width",  (iPgUp>iPgDn)?iPgUp:iPgDn + "px",  "important"  );
  document.getElementById(  "wfxVersions_dialogIDs_editInsertAfter"   ).style.setProperty(  "min-width",  (iPgUp>iPgDn)?iPgUp:iPgDn + "px",  "important"  );
}


/****************************************************************************/
/*! \brief  Make the scrollbar spacer look like part of the tabpanels
**
**  \param  --
**
**  \return  --
*/
function  wfxVersions_beautifyOps_scrollAdjust()
{
  /*  remove platform-native style after it is in effect  */
  document.getElementById(  "wfxVersions_dialogIDs_panels"  ).style.setProperty(  "-moz-appearance",  "none",  "important"  );

  var  oReplicate = getComputedStyle(  document.getElementById( "wfxVersions_dialogIDs_panels" ),  null  );
  var  szLeft     = "wfxVersions_dialogIDs_tabsSpacerLeft";
  var  szRight    = "wfxVersions_dialogIDs_tabsSpacerRight";

  document.getElementById(  szLeft   ).style.setProperty(  "-moz-border-left-colors",   oReplicate.getPropertyValue( "-moz-border-left-colors" ),   "important"  );
  document.getElementById(  szLeft   ).style.setProperty(  "border-left-style",         oReplicate.getPropertyValue( "border-left-style" ),         "important"  );
  document.getElementById(  szLeft   ).style.setProperty(  "border-left-width",         oReplicate.getPropertyValue( "border-left-width" ),         "important"  );
  document.getElementById(  szLeft   ).style.setProperty(  "border-left-color",         oReplicate.getPropertyValue( "border-left-color" ),         "important"  );
  document.getElementById(  szLeft   ).style.setProperty(  "background-color",          oReplicate.getPropertyValue( "background-color" ),          "important"  );
  document.getElementById(  szLeft   ).style.setProperty(  "background-image",          oReplicate.getPropertyValue( "background-image" ),          "important"  );
  document.getElementById(  szLeft   ).style.setProperty(  "background-position",       "0px 0px",                                                  "important"  );
  document.getElementById(  szRight  ).style.setProperty(  "background-image",          oReplicate.getPropertyValue( "background-image" ),          "important"  );
  document.getElementById(  szRight  ).style.setProperty(  "background-color",          oReplicate.getPropertyValue( "background-color" ),          "important"  );
  document.getElementById(  szRight  ).style.setProperty(  "border-right-style",        oReplicate.getPropertyValue( "border-right-style" ),        "important"  );
  document.getElementById(  szRight  ).style.setProperty(  "border-right-width",        oReplicate.getPropertyValue( "border-right-width" ),        "important"  );
  document.getElementById(  szRight  ).style.setProperty(  "border-right-color",        oReplicate.getPropertyValue( "border-right-color" ),        "important"  );
  document.getElementById(  szRight  ).style.setProperty(  "-moz-border-right-colors",  oReplicate.getPropertyValue( "-moz-border-right-colors" ),  "important"  );
}


/****************************************************************************/
/*! \brief  Re-loads the popup for .list files selection
**
**  Re-load is needed so that the popup opens at correct coordonates.
**
**  The window.move() functions cannot be used because of incorrect screen
**  attributes on Firefox 3 with multiple monitors.
**  Specifically, on the secondary screen:
**  - screen.left has positive value
**  - window.screenX has negative value
**  \n (so it's not possible to detect if secondary screen or just off-screen)
**
**  \param  --
**
**  \return  --
**
**  \see  wfxVersions_dialogVars_popupAdjust
*/
function  wfxVersions_dialogOps_reloadPopup()
{
  var  iNewX = wfxVersions_dialogVars_popupAdjust.moveX;
  var  iNewY = wfxVersions_dialogVars_popupAdjust.moveY;

  openDialog(  "chrome://wfx_versions/content/wfx_versions_popup2.xul",
               "",
               "chrome,dependent,titlebar=no,screenX=" + iNewX + ",screenY=" + iNewY  );
}


/****************************************************************************/
/*! \brief  Shows a popup to select which .list files to Scan
**
**  The popup will contain a number of checkboxes, based on the .lists found
**  at the configured path
**
**  \param  --
**
**  \return  --
**
**  \see  wfxVersions_dialogVars_popupAdjust
*/
function  wfxVersions_dialogOps_startScan()
{
  var  oPop = document.getElementById(  "wfxVersions_dialogIDs_filesSelector"  );
  wfxVersions_popupOps_backToMain();
  wfxVersions_globalOps_nodeCleanUp(  oPop  );

  if(  document.getElementById( "wfxVersions_dialogIDs_tabs" ).childNodes.length > 2  )
  {
    /*  just in case the user clicks [Scan] while already in progress  */
    document.getElementById( "wfxVersions_dialogIDs_tabs" ).advanceSelectedTab(  1,  true  );
    return;
  }

  document.getElementById(  "wfxVersions_dialogIDs_pathText"  ).value = Components.classes[ "@mozilla.org/preferences-service;1" ]
                                                                                  .getService(  Components.interfaces.nsIPrefBranch  )
                                                                                  .getComplexValue(  "extensions.wfx_Versions.tcPluginPath",  Components.interfaces.nsISupportsString  )
                                                                                  .data;

  var  szTip = document.getElementById(  "wfxVersions_dialogIDs_buttonScan"  ).getAttribute(  "tooltiptext"  );
  wfxVersions_fileOps_openDir(  szTip  );
  if(  wfxVersions_dialogVars_filesList.length != 0  )
  {
    var  bSkipSelect = document.getElementById(  "wfxVersions_dialogIDs_allScan"  ).checked;
    if(  bSkipSelect  )
    {
      wfxVersions_scanOps_start(  szTip  );
      return;
    }

    bSkipSelect = Components.classes[ "@mozilla.org/preferences-service;1" ]
                            .getService(  Components.interfaces.nsIPrefBranch  )
                            .getBoolPref(  "extensions.wfx_Versions.selections.skipSingle.Scan"  );
    if((  bSkipSelect  )&&(  wfxVersions_dialogVars_filesList.length == 1  ))
    {
      wfxVersions_scanOps_start(  szTip  );
      return;
    }

    /*  populate the popup with .lists  */
    wfxVersions_memoOps_recallSelections(  "extensions.wfx_Versions.scanSelections",  "extensions.wfx_Versions.scanMemorize"  );

    if(  wfxVersions_dialogVars_Fx3  )
    {
      /*  Fx3+ handling  */
      if(  !wfxVersions_popupOps_populateScan()  )
      {
        return;
      }

      /*  show at initial position, actual position to be determined later  */
      wfxVersions_globalOps_showPopup(  oPop,  screenX,  screenY  );
      wfxVersions_globalOps_setTimeout(  "wfxVersions_popupOps_posRelative( 'wfxVersions_dialogIDs_buttonScan' )",  10  );
    }
    else
    {
      /*  Fx2- handling  */
      document.getElementById(  "wfxVersions_dialogIDs_buttonScan"  ).setAttribute(  "disabled",  "true"  );

      /*  open the popup for selecting .lists  */
      wfxVersions_dialogVars_popupAdjust.moved = false;
      wfxVersions_dialogVars_popupAdjust.moveX = screenX;
      wfxVersions_dialogVars_popupAdjust.moveY = screenY;
      wfxVersions_dialogVars_popupAdjust.popup = 0/*Scan*/;
      openDialog(  "chrome://wfx_versions/content/wfx_versions_popup2.xul",
                   "",
                   "chrome,dependent,titlebar=no,screenX=" + screenX + ",screenY=" + screenY  );

      wfxVersions_globalOps_setTimeout(  "document.getElementById('wfxVersions_dialogIDs_buttonScan').removeAttribute('disabled')",  80  );
    }
  }
}


/****************************************************************************/
/*! \brief  Shows a popup to select which .list file to Edit
**
**  The popup will contain a number of radiobuttons, based on the .lists found
**  at the configured path
**
**  \param  --
**
**  \return  --
**
**  \see  wfxVersions_dialogVars_popupAdjust
*/
function  wfxVersions_dialogOps_startEdit()
{
  var  oPop = document.getElementById(  "wfxVersions_dialogIDs_filesSelector"  );
  wfxVersions_popupOps_backToMain();
  wfxVersions_globalOps_nodeCleanUp(  oPop  );

  if(  !document.getElementById( "wfxVersions_dialogIDs_tabs" ).firstChild.hasAttribute( "hidden" )  )
  {
    /*  just in case the user clicks [Edit] while already in progress  */
    document.getElementById( "wfxVersions_dialogIDs_tabs" ).advanceSelectedTab(  1,  true  );
    return;
  }

  document.getElementById(  "wfxVersions_dialogIDs_pathText"  ).value = Components.classes[ "@mozilla.org/preferences-service;1" ]
                                                                                  .getService(  Components.interfaces.nsIPrefBranch  )
                                                                                  .getComplexValue(  "extensions.wfx_Versions.tcPluginPath",  Components.interfaces.nsISupportsString  )
                                                                                  .data;

  var  szTip = document.getElementById(  "wfxVersions_dialogIDs_buttonEdit"  ).getAttribute(  "tooltiptext"  );
  var  fsDir = wfxVersions_fileOps_openDir(  szTip  );
  if(  wfxVersions_dialogVars_filesList.length != 0  )
  {
    var  bSkipSelect = Components.classes[ "@mozilla.org/preferences-service;1" ]
                                 .getService(  Components.interfaces.nsIPrefBranch  )
                                 .getBoolPref(  "extensions.wfx_Versions.selections.skipSingle.Edit"  );
    if((  bSkipSelect  )&&(  wfxVersions_dialogVars_filesList.length == 1  ))
    {
      wfxVersions_editOps_start(  szTip  );
      return;
    }

    /*  populate the popup with .lists  */
    wfxVersions_memoOps_recallSelections(  "extensions.wfx_Versions.editSelections",  "extensions.wfx_Versions.editMemorize"  );

    if(  wfxVersions_dialogVars_Fx3  )
    {
      /*  Fx3+ handling  */
      if(  !wfxVersions_popupOps_populateEdit()  )
      {
        return;
      }

      /*  show at initial position, actual position to be determined later  */
      wfxVersions_globalOps_showPopup(  oPop,  screenX,  screenY  );
      wfxVersions_globalOps_setTimeout(  "wfxVersions_popupOps_posRelative( 'wfxVersions_dialogIDs_buttonEdit' )",  10  );
    }
    else
    {
      /*  Fx2- handling  */
      document.getElementById(  "wfxVersions_dialogIDs_buttonEdit"  ).setAttribute(  "disabled",  "true"  );

      /*  open the popup for selecting .lists  */
      wfxVersions_dialogVars_popupAdjust.moved = false;
      wfxVersions_dialogVars_popupAdjust.moveX = screenX;
      wfxVersions_dialogVars_popupAdjust.moveY = screenY;
      wfxVersions_dialogVars_popupAdjust.popup = 1/*Edit*/;
      openDialog(  "chrome://wfx_versions/content/wfx_versions_popup2.xul",
                   "",
                   "chrome,dependent,titlebar=no,screenX=" + screenX + ",screenY=" + screenY  );

      wfxVersions_globalOps_setTimeout(  "document.getElementById('wfxVersions_dialogIDs_buttonEdit').removeAttribute('disabled')",  80  );
    }
  }
  else
  if(  fsDir  )
  {
    wfxVersions_dialogVars_filesList.push(  fsDir  );
    wfxVersions_editOps_createFile(  szTip  );
  }
  else
  {}
}


/****************************************************************************/
/*! \brief  Shows a popup to select the .list where to copy the current entry
**
**  The popup will contain a number of radiobuttons, based on the .lists found
**  at the configured path
**
**  \param  --
**
**  \return  --
**
**  \see  wfxVersions_dialogVars_popupAdjust
*/
function  wfxVersions_dialogOps_startCopy()
{
  var  oPop = document.getElementById(  "wfxVersions_dialogIDs_filesSelector"  );
  wfxVersions_popupOps_backToMain();
  wfxVersions_globalOps_nodeCleanUp(  oPop  );

  /*  clean temp status, to copy only content existing in current .list  */
  wfxVersions_editOps_clickEntry(  wfxVersions_dialogVars_progress  );

  var  szTip = document.getElementById(  "wfxVersions_dialogIDs_editCopy"  ).getAttribute(  "tooltiptext"  );
  var  fsDir = wfxVersions_fileOps_openDir(  szTip  );
  if(  wfxVersions_dialogVars_filesList.length != 0  )
  {
    /*  populate the popup with .lists  */
    wfxVersions_memoOps_recallSelections(  "extensions.wfx_Versions.copySelections",  "extensions.wfx_Versions.copyMemorize"  );

    if(  wfxVersions_dialogVars_Fx3  )
    {
      /*  Fx3+ handling  */
      if(  !wfxVersions_popupOps_populateCopy()  )
      {
        return;
      }

      /*  show at initial position, actual position to be determined later  */
      wfxVersions_globalOps_showPopup(  oPop,  screenX,  screenY  );
      wfxVersions_globalOps_setTimeout(  "wfxVersions_popupOps_posRelative( 'wfxVersions_dialogIDs_editCopy' )",  10  );
    }
    else
    {
      /*  Fx2- handling  */
      document.getElementById(  "wfxVersions_dialogIDs_editCopy"  ).setAttribute(  "disabled",  "true"  );

      /*  open the popup for selecting .lists  */
      wfxVersions_dialogVars_popupAdjust.moved = false;
      wfxVersions_dialogVars_popupAdjust.moveX = screenX;
      wfxVersions_dialogVars_popupAdjust.moveY = screenY;
      wfxVersions_dialogVars_popupAdjust.popup = 2/*Copy*/;
      openDialog(  "chrome://wfx_versions/content/wfx_versions_popup2.xul",
                   "",
                   "chrome,dependent,titlebar=no,screenX=" + screenX + ",screenY=" + screenY  );

      wfxVersions_globalOps_setTimeout(  "document.getElementById('wfxVersions_dialogIDs_editCopy').removeAttribute('disabled')",  80  );
    }
  }
}


/****************************************************************************/
/*! \brief  Shows throbbers on/off individual per scanned .list tab
**
**  \param  --
**
**  \return  --
*/
function  wfxVersions_dialogOps_throbberTabs()
{
  /*  cancel the periodic re-scan timer  */
  wfxVersions_globalOps_stopTimer(  wfxVersions_dialogVars_reScanTimer  );

  var  szImage = "chrome://wfx_versions/skin/wfx_versions_scan.png";
  if(  document.getElementById( "wfxVersions_dialogIDs_tabs" ).childNodes.length > 2  )
  {
    /*  progress throbbers individual per tab  */
    var  aImgs = [];
    var  i,  j;
    for(  j = 2;  j < document.getElementById( "wfxVersions_dialogIDs_tabs" ).childNodes.length;  j++  )
    {
      aImgs.push({  list:  document.getElementById( "wfxVersions_dialogIDs_tabs" ).childNodes[j].getAttribute( "tooltiptext" ),
                    image: "chrome://wfx_versions/skin/wfx_versions_scan.png"  });
    }
    for(  i = 0;  i < wfxVersions_dialogVars_itemsList.length;  i++  )
    {
      if((  wfxVersions_dialogVars_itemsList[i].status == 0/*idle*/  )
       ||(  wfxVersions_dialogVars_itemsList[i].status == 2/*scan*/  ))
      {
        for(  j = 0;  j < aImgs.length;  j++  )
        {
          if(  aImgs[j].list == wfxVersions_dialogVars_itemsList[i].list  )
          {
            aImgs[j].image = "chrome://wfx_versions/skin/wfx_versions_throbber.gif";
            szImage = "chrome://wfx_versions/skin/wfx_versions_throbber.gif";

            j = i + 1;
            while((  j < wfxVersions_dialogVars_itemsList.length  )&&(  wfxVersions_dialogVars_itemsList[j].list == wfxVersions_dialogVars_itemsList[i].list  ))
            {
              j++;
            }
            i = j - 1;

            break;
          }
        }
      }
    }
    for(  j = 0;  j < aImgs.length;  j++  )
    {
      wfxVersions_dialogOps_setImageProperty(  document.getElementById( "wfxVersions_dialogIDs_tabs" ).childNodes[j+2],  aImgs[j].image  );
    }
    while(  aImgs.length  )
    {
      aImgs.pop();
    }

    /*  arm the periodic re-scan timer  */
    if(  szImage != "chrome://wfx_versions/skin/wfx_versions_throbber.gif"  )
    {
      i = Components.classes[ "@mozilla.org/preferences-service;1" ]
                    .getService(  Components.interfaces.nsIPrefBranch  )
                    .getIntPref(  "extensions.wfx_Versions.force.reScan.onScanDone"  );
      if(  i > 0  )
      {
        wfxVersions_dialogVars_reScanTimer = wfxVersions_globalOps_setTimeout(  "wfxVersions_scanPrep_reScanAll()",  i * 1000  );
      }
    }
  }

  wfxVersions_dialogOps_setImageAttribute(  document.getElementById( "wfxVersions_dialogIDs_buttonScan" ),  szImage  );
}


/****************************************************************************/
/*! \brief  Prepares the periodic re-scan process
**
**  \param  --
**
**  \return  --
**
**  \see  wfxVersions_scanOps_reScanAll()
*/
function  wfxVersions_scanPrep_reScanAll()
{
  for(  var  iItem = 0;  iItem < wfxVersions_dialogVars_itemsList.length;  iItem++  )
  {
    if((  wfxVersions_dialogVars_itemsList[iItem].status < 6  )&&(  wfxVersions_dialogVars_itemsList[iItem].status != 3  ))
    {
      wfxVersions_dialogOps_scanStatus(  iItem,  4  );
    }
  }

  wfxVersions_scanOps_reScanAll();
}


/****************************************************************************/
/*! \brief  show/hide the close-tab button depending on which tab is selected
**
**  \param  --
**
**  \return  --
*/
function  wfxVersions_dialogOps_tabsSelect()
{
  var  iTab = document.getElementById(  "wfxVersions_dialogIDs_tabs"  ).selectedIndex;
  wfxVersions_globalOps_setTimeout(  "document.getElementById('wfxVersions_dialogIDs_tabs').childNodes[" + iTab + "].focus()",  20  );

  var  szDisable = "false";
  if(  iTab == 1  )
  {
    szDisable = "true";
  }
  document.getElementById(  "wfxVersions_dialogIDs_tabsClose"  ).setAttribute(  "disabled",  szDisable  );
  document.getElementById(  "wfxVersions_dialogIDs_tabsClose"  ).focus();

  wfxVersions_globalOps_setTimeout(  "wfxVersions_dialogOps_tabsSelectDelayed()",  25  );
}


/****************************************************************************/
/*! \brief  show/hide the tab-prev/-next buttons depending on number of tabs
**
**  \param  --
**
**  \return  --
*/
function  wfxVersions_dialogOps_tabsSelectDelayed()
{
  var  szDisable = "false";
  var  nTabs = document.getElementById(  "wfxVersions_dialogIDs_tabs"  ).childNodes.length;
  if((  nTabs <= 2  )&&(  document.getElementById( "wfxVersions_dialogIDs_tabs" ).firstChild.hasAttribute( "hidden" )  ))
  {
    szDisable = "true";
  }
  document.getElementById(  "wfxVersions_dialogIDs_tabsPrev"  ).setAttribute(  "disabled",  szDisable  );
  document.getElementById(  "wfxVersions_dialogIDs_tabsNext"  ).setAttribute(  "disabled",  szDisable  );
}


/****************************************************************************/
/*! \brief  selects the close operation corresponding to currently open tab
**
**  \param  --
**
**  \return  --
*/
function  wfxVersions_dialogOps_tabsClose()
{
  var  szTip = null;
  var  iTab  = document.getElementById(  "wfxVersions_dialogIDs_tabs"  ).selectedIndex;
  if(  iTab > 1  )
  {
    szTip = document.getElementById(  "wfxVersions_dialogIDs_tabs"  ).childNodes[iTab].getAttribute(  "tooltiptext"  );
  }
  wfxVersions_dialogOps_closeTab(  szTip  );
}


/****************************************************************************/
/*! \brief  set the image for a button
**
**  Images on buttons may be disabled via about:config option
**
**  \param[in]  oNode  the node for which the image is set
**  \param[in]  szImg  the chrome URL of the image
**
**  \return  --
*/
function  wfxVersions_dialogOps_setImageAttribute(  oNode,  szImg  )
{
  if((  "noicons" in window  )&&(  window["noicons"]  )&&(  oNode.getAttribute( "label" )  ))
  {
    oNode.removeAttribute(  "image"  );
  }
  else
  {
    oNode.setAttribute(  "image",  szImg  );
  }
}


/****************************************************************************/
/*! \brief  set the image for a tab
**
**  Images on tabs may be disabled via about:config option
**
**  \param[in]  oNode  the node for which the image is set
**  \param[in]  szImg  the chrome URL of the image
**
**  \return  --
*/
function  wfxVersions_dialogOps_setImageProperty(  oNode,  szImg  )
{
  if((  "noicons" in window  )&&(  window["noicons"]  )&&(  oNode.getAttribute( "label" )  ))
  {
    oNode.style.removeProperty(  "list-style-image"  );
  }
  else
  {
    oNode.style.setProperty(  "list-style-image",  'url("' + szImg + '")',  "important"  );
  }
}


/****************************************************************************/
/*! \brief  Check memorization options when starting wfx_Versions
**
**  The selections are cleared if the settings are "per session" or less
**
**  \param  --
**
**  \return  --
**
**  \see  wfxVersions_dialogVars_selections[], wfxVersions_memoOps_onClose()
*/
function  wfxVersions_memoOps_onLoad()
{
  var  oPrefs = Components.classes[ "@mozilla.org/preferences-service;1" ]
                          .getService(  Components.interfaces.nsIPrefBranch  );

  /*  create strings with "no selections memorized"  */
  var  szSelections = Components.classes[ "@mozilla.org/supports-string;1" ]
                                .createInstance(  Components.interfaces.nsISupportsString  );
  szSelections.data = "";

  /*  clear selections where applicable  */
  var  iOn = oPrefs.getIntPref(  "extensions.wfx_Versions.scanMemorize"  );
  if((  iOn > 0  )&&(  iOn < 3  ))
  {
    oPrefs.setComplexValue(  "extensions.wfx_Versions.scanSelections",  Components.interfaces.nsISupportsString,  szSelections  );
  }

  if(  iOn <= 0  )
  {
    iOn = 2;
  }
  else
  if(  iOn >= 3  )
  {
    iOn = 0;
  }
  else
  {
    iOn = 1;
  }
  document.getElementById(  "wfxVersions_dialogIDs_memoScan"  ).selectedIndex = iOn;

  iOn = oPrefs.getIntPref(  "extensions.wfx_Versions.editMemorize"  );
  if((  iOn > 0  )&&(  iOn < 3  ))
  {
    oPrefs.setComplexValue(  "extensions.wfx_Versions.editSelections",  Components.interfaces.nsISupportsString,  szSelections  );
  }

  if(  iOn <= 0  )
  {
    iOn = 2;
  }
  else
  if(  iOn >= 3  )
  {
    iOn = 0;
  }
  else
  {
    iOn = 1;
  }
  document.getElementById(  "wfxVersions_dialogIDs_memoEdit"  ).selectedIndex = iOn;

  iOn = oPrefs.getIntPref(  "extensions.wfx_Versions.copyMemorize"  );
  if((  iOn > 0  )&&(  iOn < 3  ))
  {
    oPrefs.setComplexValue(  "extensions.wfx_Versions.copySelections",  Components.interfaces.nsISupportsString,  szSelections  );
  }

  if(  iOn <= 0  )
  {
    iOn = 2;
  }
  else
  if(  iOn >= 3  )
  {
    iOn = 0;
  }
  else
  if(  iOn == 2  )
  {
    iOn = 1;
  }
  else
  {
    iOn = 3;
  }
  document.getElementById(  "wfxVersions_dialogIDs_memoCopy"  ).selectedIndex = iOn;
}


/****************************************************************************/
/*! \brief  Save memorization options when closing wfx_Versions
**
**  \param  --
**
**  \return  --
**
**  \see  wfxVersions_dialogVars_selections[], wfxVersions_memoOps_onLoad()
*/
function  wfxVersions_memoOps_onClose()
{
  var  oPrefs = Components.classes[ "@mozilla.org/preferences-service;1" ]
                          .getService(  Components.interfaces.nsIPrefBranch  );

  var  iOn = document.getElementById(  "wfxVersions_dialogIDs_memoScan"  ).selectedIndex;
  if(  iOn <= 0  )
  {
    iOn = 3;
  }
  else
  if(  iOn >= 2  )
  {
    iOn = 0;
  }
  else
  {
    iOn = 2;
  }
  oPrefs.setIntPref(  "extensions.wfx_Versions.scanMemorize",  iOn  );

  var  iOn = document.getElementById(  "wfxVersions_dialogIDs_memoEdit"  ).selectedIndex;
  if(  iOn <= 0  )
  {
    iOn = 3;
  }
  else
  if(  iOn >= 2  )
  {
    iOn = 0;
  }
  else
  {
    iOn = 2;
  }
  oPrefs.setIntPref(  "extensions.wfx_Versions.editMemorize",  iOn  );

  var  iOn = document.getElementById(  "wfxVersions_dialogIDs_memoCopy"  ).selectedIndex;
  if(  iOn <= 0  )
  {
    iOn = 3;
  }
  else
  if(  iOn >= 3  )
  {
    iOn = 1;
  }
  else
  if(  iOn == 1  )
  {
    iOn = 2;
  }
  else
  {
    iOn = 0;
  }
  oPrefs.setIntPref(  "extensions.wfx_Versions.copyMemorize",  iOn  );
}


/****************************************************************************/
/*! \brief  Read memorized Scan/Edit/Copy selections
**
**  \param[in]  szPref  preference name (Scan/Edit/Copy)
**  \param[in]  szOn    preference name enabled (true/false)
**
**  \return  --
**
**  \see  wfxVersions_dialogVars_selections[],
**        wfxVersions_memoOps_storeSelections()
*/
function  wfxVersions_memoOps_recallSelections(  szPref,  szOn  )
{
  var  oPrefs = Components.classes[ "@mozilla.org/preferences-service;1" ]
                          .getService(  Components.interfaces.nsIPrefBranch  );

  /*  read selections from storage  */
  var  iOn = oPrefs.getIntPref(  szOn  );
  if(  iOn > 0  )
  {
    var  szSelections = oPrefs.getComplexValue(  szPref,  Components.interfaces.nsISupportsString  );
    wfxVersions_dialogVars_selections = szSelections.data.split(  "*"  );
  }
  else
  {
    wfxVersions_dialogVars_selections = [];
  }
}


/****************************************************************************/
/*! \brief  Write memorized Scan/Edit/Copy selections
**
**  \param[in]  szPref  preference name (Scan/Edit/Copy)
**  \param[in]  szOn    preference name enabled (true/false)
**
**  \return  --
**
**  \see  wfxVersions_dialogVars_selections[],
**        wfxVersions_memoOps_recallSelections()
*/
function  wfxVersions_memoOps_storeSelections(  szPref,  szOn  )
{
  var  iDir = wfxVersions_dialogVars_filesList[0].path.lastIndexOf(  '\\'  );
  if(  iDir < 0  )
  {
    iDir = wfxVersions_dialogVars_filesList[0].path.lastIndexOf(  '/'  );
  }
  var  szDir = wfxVersions_dialogVars_filesList[0].path.substring(  0,  1 + iDir  );

  /*  filter out old selections  */
  var  iPos   = wfxVersions_dialogVars_selections.length - 1;
  var  aMemos = [];

  while(  iPos >= 0  )
  {
    iDir = wfxVersions_dialogVars_selections[iPos].lastIndexOf(  '\\'  );
    if(  iDir < 0  )
    {
      iDir = wfxVersions_dialogVars_selections[iPos].lastIndexOf(  '/'  );
    }

    if(  wfxVersions_dialogVars_selections[iPos].substring( 0, 1 + iDir ) == szDir  )
    {
      wfxVersions_dialogVars_selections.pop();
    }
    else
    {
      aMemos.push(  wfxVersions_dialogVars_selections.pop()  );
    }
    iPos--;
  }

  /*  memorize new selections  */
  while(  aMemos.length != 0  )
  {
    wfxVersions_dialogVars_selections.push(  aMemos.pop()  );
  }

  for(  iPos = 0;  iPos < wfxVersions_dialogVars_filesList.length;  iPos++  )
  {
    wfxVersions_dialogVars_selections.push(  wfxVersions_dialogVars_filesList[iPos].path  );
  }

  /*  write selections to storage  */
  var  oPrefs = Components.classes[ "@mozilla.org/preferences-service;1" ]
                          .getService(  Components.interfaces.nsIPrefBranch  );

  var  iOn = oPrefs.getIntPref(  szOn  );
  if(  iOn > 0  )
  {
    var  szSelections = Components.classes[ "@mozilla.org/supports-string;1" ]
                                  .createInstance(  Components.interfaces.nsISupportsString  );
    szSelections.data = wfxVersions_dialogVars_selections.join(  "*"  );
    oPrefs.setComplexValue(  szPref,  Components.interfaces.nsISupportsString,  szSelections  );
  }
}


/****************************************************************************/
/*! \brief  dynamic link based on version-string
**
**  Using TC MRT-style notation like [V1-4]:
**  - [%0A] = full string
**  - [%0Ax] = character at index 'x' (where 'x' range is: 1 to string length)
**  - [%0Ax-] = string starting at index 'x'
**  - [%0Ax-y] = string between indexes 'x' and 'y' (where 'y' >= 'x')
**
**  \param[in]  szLink  the URL before conversion
**  \param[in]  szTxt   the current-version/updated-version string
**
**  \return  the URL after conversion
*/
function  wfxVersions_dialogOps_likeMRT(  szLink,  szTxt  )
{
  var  szURL  = szLink;
  var  aMatch = szURL.match(  /\[%0A\d*\-*\d*\]/gi  );
  if(  aMatch  )
  {
    for(  var  i = 0;  i < aMatch.length;  i++  )
    {
      var  iStart = 0;  /*  default: error  */
      var  iEnd   = szTxt.length;

      if(  aMatch[i].toUpperCase() == "[%0A]"  )
      {
        iStart = 1;
      }
      else
      if(  aMatch[i].match( /\[%0A(\d+)\]/gi )  )
      {
        iStart = parseInt(  aMatch[i].substring( 4 )  );
        iEnd   = iStart;
      }
      else
      if(  aMatch[i].match( /\[%0A\d+\-\]/gi )  )
      {
        iStart = parseInt(  aMatch[i].substring( 4 )  );
      }
      else
      if(  aMatch[i].match( /\[%0A\d+\-\d+\]/gi )  )
      {
        iStart = parseInt(  aMatch[i].substring( 4 )  );
        iEnd   = parseInt(  aMatch[i].substring( 1+aMatch[i].lastIndexOf('-') )  );
      }
      else
      {}

      if((  iStart > 0  )&&(  iStart <= iEnd  ))
      {
        szURL = szURL.replace(  aMatch[i],  szTxt.substring( iStart-1, iEnd )  );
      }
    }
  }

  return(  szURL  );
}


/****************************************************************************/
/*! \brief  updates the Edit-mode URLs while typing, using MRT-style replaces
**
**  \param  --
**
**  \return  --
*/
function  wfxVersions_dialogOps_updateEditLinks()
{
  var  szTxt = document.getElementById(  "wfxVersions_dialogIDs_editVersion"  ).value;

  var  oEdit = document.getElementById(  "wfxVersions_dialogIDs_editURL"  );
  oEdit.previousSibling.setAttribute(  "tooltiptext",  wfxVersions_dialogOps_likeMRT( oEdit.value, szTxt )  );

  oEdit = document.getElementById(  "wfxVersions_dialogIDs_editHome"  );
  oEdit.previousSibling.setAttribute(  "tooltiptext",  wfxVersions_dialogOps_likeMRT( oEdit.value, szTxt )  );

  oEdit = document.getElementById(  "wfxVersions_dialogIDs_editDown"  );
  oEdit.previousSibling.setAttribute(  "tooltiptext",  wfxVersions_dialogOps_likeMRT( oEdit.value, szTxt )  );
}


/****************************************************************************/
/*! \brief  updates the Scan-mode URLs while typing, using MRT-style replaces
**
**  \param[in]  iItem  which item from the list is being updated
**  \param[in]  iItem  current-version/updated-version string
**
**  \return  --
*/
function  wfxVersions_dialogOps_updateScanLinks(  iItem,  szTxt  )
{
  var  oScan = document.getElementById(  "n" + iItem  );
  oScan.setAttribute(  "tooltiptext",  wfxVersions_dialogOps_likeMRT( wfxVersions_dialogVars_itemsList[iItem].url, szTxt )  );

  oScan.previousSibling.setAttribute(  "tooltiptext",  wfxVersions_dialogOps_likeMRT( wfxVersions_dialogVars_itemsList[iItem].down, szTxt )  );

  oScan.previousSibling.previousSibling.setAttribute(  "tooltiptext",  wfxVersions_dialogOps_likeMRT( wfxVersions_dialogVars_itemsList[iItem].home, szTxt )  );
}


/****************************************************************************/
/*! \brief  insert a new entry before or after the currently edited entry
**
**  \param[in]  szOps   operation string (for message alerts)
**  \param[in]  iDelta  offset for before (-1) or after (+0)
**
**  \return  --
*/
function  wfxVersions_editOps_insertNew(  szOps,  iDelta  )
{
  var  oldPos = wfxVersions_dialogVars_progress;
  var  oldLen = wfxVersions_dialogVars_itemsList.length;

  /*  simply append the old-style new-entry  */
  document.getElementById(  "wfxVersions_dialogIDs_editItems"  ).lastChild.removeAttribute(  "hidden"  );
  wfxVersions_editOps_clickEntry(  oldLen - 1  );
  wfxVersions_editOps_modifyEntry(  true  );
  document.getElementById(  "wfxVersions_dialogIDs_editItems"  ).lastChild.setAttribute(  "hidden",  "true"  );

  if(  wfxVersions_dialogVars_itemsList.length != oldLen  )
  {
    /*  move (without confirmations)  */
    oldLen -= 2 + iDelta;
    wfxVersions_editOps_moveEntry(  szOps,  oldPos - oldLen,  false  );
    wfxVersions_editOps_sortList(  document.getElementById( "wfxVersions_dialogIDs_sortAuto" ).checked  );
  }
  else
  {
    /*  operation was canceled by user  */
    wfxVersions_editOps_clickEntry(  oldPos  );
  }
}


/****************************************************************************/
/*! \brief  make tooltip texts appear in the status-bar
**
**  \param[in]  oItem  element which contains the tooltip-text to be shown
**
**  \return  --
*/
function  wfxVersions_dialogOps_copyTip(  oItem  )
{
  var  oStatusBar = document.getElementById(  "wfxVersions_dialogIDs_statusBar"  );

  if((  oStatusBar  )&&(  oItem  )&&(  "hasAttribute" in oItem  )&&(  oItem.hasAttribute( "tooltiptext" )  ))
    oStatusBar.value = " " + oItem.getAttribute(  "tooltiptext"  ) + " ";
  else
    oStatusBar.value = " ";
}
